#*****************************************************************************
#*****************************************************************************
# R code for plotting root tracing and phenotype data for the RNAseq June2013
# seedling root traces produced using ImageJ.
#
# The code below in R language is run to read the Excel file produced when the
# raw root outlines were traced, and produce pdf output files containing plot
# results.
#
# EDIT FILE Include_setAnalysisAndDate.R BEFORE RUNNING THIS CODE.
#*****************************************************************************
#*****************************************************************************

################################################################################
# Set working directory.
################################################################################

setwd("/Users/tedtoal/Documents/UCDavis/BradyLab/GrowthExperiments/Tomato/RootRNAseqJun2013_Phenotypes")

################################################################################
# Include other files used by this.
################################################################################

source("Include_setAnalysisAndDate.R")
source("Include_general.R")
source("Include_xlsx.R")
source("Include_plateLayout.R")
source("Include_dataColumnTypes.R")

################################################################################
# Load libraries used by this.
################################################################################

library("beanplot")

################################################################################
# Set constants controlling the processing of the data.
################################################################################

# Output files.
platePlotFile = paste(dataPathPrefix, "PlotRoots.pdf", sep="")
beanPlotFile = paste(dataPathPrefix, "BeanPlots.pdf", sep="")

################################################################################
# Set constants related to plotting the plates.
################################################################################

# Set this TRUE if every plate should have a data entry for every possible seed
# on the plate, regardless of whether or not the seed germinated or had problems.
# Set it FALSE to allow each plate to have data entries for subsets of all the
# numSeedsPerPlate possible seeds.
requireAllSeeds = FALSE

# Title of a plate plot.  Specify a template string for the titles here, using data
# column names surrounded by % characters to generate custom titles based on the
# data.  %column_name% is replaced by the value in that data column.  It only makes
# sense to use column names that are the same for all seeds of a plate.
plotTitle = "%genotype% %replicate% plate %plateLetter% - Experiment  %experiment%\n%filename%"

# Font size factor for title.
title.cex = 0.9

# Set these to the width and height of the plate plotting pages in inches.
plate.pdf.width.in = 12
plate.pdf.height.in = 12

# Set this to the number of rows and columns of plates plotted on a single page.
# For the case here, there are SO MANY plates that it is advisable NOT to set
# both these to 1.
mfrow = c(nrows=4, ncols=4)

# Set this to the percentage of the available plate plotting area width to be
# used for plotting phenotype data.  The remaining percent (up to 100%) is used
# to plot the plate, which will be plotted as square.  Set this to 0 to not plot
# any phenotype data.  For the case here, there are SO MANY plates that it is
# advisable to set this to 0.  The phenotype data is plotted to the left of the
# plate itself.
plotPhenotypeDataPctWidth = 0

# If phenotype data is to be plotted, list the data columns to be plotted here.
{
if (analysisVariant == "Orig")
    plotPhenotypeCols = c("straightLen","rootLen","waviness","wholeAng","quarterAng",
        "tenthAng","tipAng","avgAbsAng")
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    plotPhenotypeCols = c("straightLen","rootLen","waviness","tipAng","avgAbsAng",
        "first3rdAng", "mid3rdAng", "last3rdAng")
else
    stop("Bad analysisVariant value")
}

# Set a fixed size in inches for all four page margins.  We plot a title, but
# otherwise the margins are empty and can be very small, just enough to put
# some space between plates.
mai = c(0.1, 0.1, 0.25, 0.1)

# Enclose following section in braces because it has if-else statements at
# indent level 0.
{
# If the plates are plotted more than one to a page, the page margins are specified
# with "omi" instead of "mai", and "mai" is used for the margins around each
# individual plot.  However, we do not plot anything in the page margins, only
# a title for each individual plot, so set omi to 0.  
omi = rep(0, 4)

# Each plate is plotted as a square, with either nothing at all plotted outside
# the square (plotPhenotypeDataPctWidth=0), or with phenotype data plotted
# (plotPhenotypeDataPctWidth>0) on the left side of the plate square.  Define
# the coordinates of the lower-left corner of the plate as (0,0) and the
# upper-right corner as (plateSize.cm, plateSize.cm), and set xlim and ylim
# accordingly.  These values will not change regardless of the settings of
# plotPhenotypeDataPctWidth.  This allows all plotting values to be specified
# in cm relative to the plate size in cm, i.e. user coordinates are in plate
# centimeters, not plotting centimeters (which are determined by the size of
# the plot page and by mfrow).
plateLowerLeft.usr = c(0, 0)
plateUpperRight.usr = c(plateSize.cm, plateSize.cm)
xlim = c(plateLowerLeft.usr[1], plateUpperRight.usr[1])
ylim = c(plateLowerLeft.usr[2], plateUpperRight.usr[2])

# Compute the size of the plotting area in cm, and double-check that the value
# of plotPhenotypeDataPctWidth is valid.  Account for mai perhaps occurring
# multiple times if mfRow entries are not 1.
xplot.cm = (plate.pdf.width.in-omi[2]-omi[4]-mfrow[2]*(mai[2]+mai[4]))*cmPerInch
yplot.cm = (plate.pdf.height.in-omi[1]-omi[3]-mfrow[1]*(mai[1]+mai[3]))*cmPerInch
if (plotPhenotypeDataPctWidth < 0 || plotPhenotypeDataPctWidth > 100)
    stop("plotPhenotypeDataPctWidth out of bounds")

# Adjust the margins in omi if necessary to make the plate square, while
# accommodating plotPhenotypeDataPctWidth.
# The plotting area width (xplot.cm) is split between the plate and the
# phenotype data according to the setting of plotPhenotypeDataPctWidth.
# The necessary height of the plate plotting area is then equal to the PLATE
# plotting area width, since plates are square.  If the resulting size is
# less than what is available (yplot.cm), the top and bottom margins are
# increased in equal amounts to center the plate vertically.  If the
# resulting size is more than yplot.cm, the left and right margins are
# increased in equal amounts to reduce xplot.cm and hence reduce the
# plotted plate size to just fit in yplot.cm.
# We account for the number of plates plotted per page both vertically (mfrow[1])
# and horizontally (mfrow[2]).
xplates.cm = xplot.cm*(100-plotPhenotypeDataPctWidth)/100
yplates.cm = xplates.cm*mfrow[1]/mfrow[2]
if (yplates.cm < yplot.cm)
    {
    dy = yplot.cm-yplates.cm
    omi[1] = omi[1] + dy/2
    omi[3] = omi[3] + dy/2
    yplot.cm = (plate.pdf.height.in-omi[1]-omi[3]-mfrow[1]*(mai[1]+mai[3]))*cmPerInch # Recompute as above.
    }
else if (yplates.cm > yplot.cm)
    {
    dx = yplates.cm-yplot.cm
    omi[2] = omi[2] + dx/2
    omi[4] = omi[4] + dx/2
    xplot.cm = (plate.pdf.width.in-omi[2]-omi[4]-mfrow[2]*(mai[2]+mai[4]))*cmPerInch # Recompute as above.
    }
}

# Compute the width and left user coordinate of the phenotype data plotting area
# (if any).  The right side position is equal to the position of the left side of
# the plate (plateLowerLeft.usr[1]), and the top and bottom positions are equal to
# the plate top and bottom positions (plateLowerLeft.usr[2] and plateUpperRight.usr[2]).
# Let X = plotPhenotypeDataPctWidth.  Then since the plate user coordinate width
# is plateUpperRight.usr[1]-plateLowerLeft.usr[1], the entire plot area width in
# user coordinates is that divided by (100-X) percent, and therefore the user
# coordinate width of the phenotype plotting area is the result of that times X
# percent, and the left coordinate of the phenotype area is that result subtracted
# from plateLowerLeft.usr[1].
X = plotPhenotypeDataPctWidth
dataWidth.usr = (plateUpperRight.usr[1]-plateLowerLeft.usr[1])*X/(100-X)
dataLeft.usr = plateLowerLeft.usr[1] - dataWidth.usr

# Calculate vertical spacing of phenotype data rows (if any), in user coordinates.
# Divide the height of one seed row by the number of phenotypes + 1 for space.
dataHeight.usr = seedSpaceY.cm/(length(plotPhenotypeCols)+1) # Vertical spacing of data columns.

# Width of data label column.  It needs enough space, but if the available space
# is too small, cut it down.
labelWidthX.usr = 2.0
if (labelWidthX.usr > dataWidth.usr/4) # Somewhat arbitrary.
    {
    labelWidthX.usr = dataWidth.usr/4
    cat("Phenotype data column width reduced to", labelWidthX.usr, "\n")
    }

# Width of each data column.  Just divide available space by number of columns.
dataColWidth.usr = (dataWidth.usr - labelWidthX.usr)/(seedCols*ifelse(sided, 2, 1))

# Phenotype data values are plotted to the left of the plate, each data row located
# to the left of the seed row it provides phenotype data for.  The data is plotted
# in one (sided=FALSE) or two (sided=TRUE) sets of seedCols columns, with a column
# of labels describing the data in each row.  The label is located left of the one
# set of seedCols columns if sided=FALSE, or between the two sets if sided=TRUE.
# Calculate the offset, in user coordinates, from left side of data area to the
# label column and to the start of each set of "seedCols" columns.  Since user
# coordinates are in plate cm, we can specify these values in terms of plate cm.
{
if (!sided)
    {
    # Offset of the data label column from left edge of data area.
    labelOffsetX.usr = 0
    # Offset of the first column of data from the left edge of the data area.
    dataOffsetLeftX.usr = labelOffsetX.usr + labelWidthX.usr
    }
else
    {
    # Offset of the first column of data from the left edge of the data area.
    dataOffsetLeftX.usr = 0.0
    # Offset of the data label column from left edge of data area.
    labelOffsetX.usr = dataOffsetLeftX.usr + (colsOneSide * dataColWidth.usr)
    # Position of data column colsOneSide+1 from the left edge of the plot page, in centimeters.
    dataOffsetRightX.usr = labelOffsetX.usr + labelWidthX.usr
    }
}

# TRUE to plot seed number labels above seed data columns.
seedNumLabel = TRUE

# Amount to place seed number label above seed position, NA to omit labels.
seedNum.dy = 0.5

# Font size factor for seed number label.
seedNum.cex = 0.7

# Amount to place seed flags (NA for no seed, [CA] to indicate "collide" and
# "along" flags) above seed position, NA to omit flags.
seedFlags.dy = 1.0

# Font size factor for seed flags.
seedFlags.cex = 0.7

# Font size factor for data lines.
data.cex = 0.7

# Plotting colors to use, cycling through these for each root on a plate.
plotColors = c(rainbow(3))

################################################################################
# Set constants related to bean plots.
################################################################################

# Define a vector of data column names that each correspond to a bean plot of a
# trait (the x-axis of that bean plot will have the values from that data column).
beanSubsetCols = c("genotype", "experiment", "germAge", "rootAge", "replicate", "collide", "along")

{
if (analysisVariant == "Orig")
    beanTraits = c("straightLen", "rootLen", "waviness",
        "wholeAng", "quarterAng", "tenthAng", "tipAng", "avgAbsAng",
        "wholeAngSign", "quarterAngSign", "tenthAngSign", "tipAngSign", "avgAngSign")
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    beanTraits = c("straightLen", "rootLen", "waviness",
        "tipAng", "avgAbsAng", "first3rdAng", "mid3rdAng", "last3rdAng",
        "tipAngSign", "avgAngSign", "first3rdAngSign", "mid3rdAngSign", "last3rdAngSign")
else
    stop("Bad analysisVariant value")
}

# Set these to the width and height of the bean plotting pages in inches.
bean.pdf.width.in = 15
bean.pdf.height.in = 6

################################################################################
# Read the data file that has phenotype computation columns added.  Double-check
# each column's name and type.
################################################################################

dfRoots = read.xlsx2(processedDataPath, sheetIndex=1, header=TRUE,
    stringsAsFactors=FALSE, colClasses=processedDataColClasses)

if (ncol(dfRoots) != length(processedDataColClasses))
    stop("Expected a different number of data columns")
for (i in 1:ncol(dfRoots))
    {
    if (colnames(dfRoots)[i] != names(processedDataColClasses)[i])
        stop("Unexpected column name mismatch in processedDataColClasses, column ", colnames(dfRoots)[i], "\n")
    if (class(dfRoots[,i]) != processedDataColClasses[i])
        stop("Wrong class of data column, column ", colnames(dfRoots)[i], "\n")
    }

################################################################################
# Plot the roots on each plate, one plate per plot page.
################################################################################

# Given a 1-based row and column number, return the corresponding 1-based seed
# number, paying attention to the current settings of "sided" and "interleave".
getSeedNumFromRowCol = function(row, col)
    {
    if (sided)
        seedNum = (row-1)*colsOneSide+(col-1)%%colsOneSide+1
    else if (!interleave)
        seedNum = (row-1)*seedCols+col
    else # (!sided && interleave)
        seedNum = floor((row-1)/2)*seedCols+col
    return(seedNum)
    }

# Given a 1-based row and column number and data frame df containing all seeds
# on a plate, return the seed type of the seed at location (row, col), paying
# attention to the current settings of "sided" and "interleave", and using the
# type of the most upper-left seed, i.e. of seed 1 on the left side, to decide
# the seed type alternation.  If there is no such seed, assume the upper-left
# seed would have type types[1].  If seed types are not used, return NA.
getSeedTypeFromRowCol = function(row, col, df)
    {
    if (is.na(seedTypes[1]))
        return(NA)
    # Get the type and side of the seed in the upper-left position.  That seed
    # is the one with seed number 1.  There will be two of them if "sided" is
    # TRUE, and in that case use the one with side == "L".
    if (!sided)
        idxUL = which(df$seed == 1)
    else
        idxUL = which(df$seed == 1 & df$side == "L")
    if (length(idxUL) != 1)
        idxUL = 1 # ASSUME first seed in df is upper-left seed if there is a problem.
    typeOfSeedUL = df$type[idxUL]
    # Get the type that is the opposite of typeOfSeedUL.
    if (typeOfSeedUL == seedTypes[1])
        otherType = seedTypes[length(seedTypes)]
    else
        otherType = seedTypes[1]
    # For non-sided non-interleave, type remains constant, it never changes on a plate.
    # Use the type of the upper-left seed.
    if (!sided && !interleave)
        type = typeOfSeedUL
    # For sided non-interleave, the type depends only on the side, which is determined
    # by the column.
    else if (sided && !interleave)
        {
        if (col <= colsOneSide)
            type = typeOfSeedUL # left side
        else
            type = otherType    # right side
        }
    # For non-sided interleave, the type alternates row-by-row, depending only on row modulo 2.
    else if (!sided && interleave)
        {
        if (row %% 2 == 1)
            type = typeOfSeedUL # rows 1,3,5...
        else
            type = otherType    # rows 2,4,6...
        }
    # For sided interleave, the type alternates row-by-row and side-by-side.
    else
        {
        if ((row %% 2 == 1) == (col <= colsOneSide))
            type = typeOfSeedUL
        else
            type = otherType
        }
    return(type)
    }

# Define a template string for producing error messages as plates are plotted.
templateStr = "%genotype%_%experiment%_%plateLetter%%seed%"

# Plot each plate on a separate page.  For the plates that are located BELOW
# another plate in the image, the y coordinate will be large, and we need to
# subtract off the offset of one whole plate from those y-coordinates.  We
# also must handle the upside-down y-coordinate system of the y-coords in the
# data files, by subtracting them from the plate height in cm, after they have
# been converted to cm.  Note that the plotting coordinates were defined in
# plate cm, so if we calculate plotting dimensions in plate cm, they are
# automatically in user coordinates for plotting.
pdf(platePlotFile, width=plate.pdf.width.in, height=plate.pdf.height.in)
par(mai=mai, omi=omi, mfrow=mfrow)

plateIDs = unique(dfRoots$plateID)
cmPerPixel = mean(dfRoots$cM/dfRoots$scale)
for (plateID in plateIDs)
    {
    cat("Plotting plate", plateID, "\n")
    df = dfRoots[dfRoots$plateID == plateID,]
    if (requireAllSeeds)
        {
        if (nrow(df) != numSeedsPerPlate)
            cat("Expected ", numSeedsPerPlate, "roots on plate ", plateID, " but it has", nrow(df), "\n")
        }
    else
        {
        if (nrow(df) == 0)
            cat("Expected at least one root on plate ", plateID, " but it has none\n")
        }

    # Get the plate rectangle position and size, in cm.
    plateRect = as.integer(unlist(strsplit(df$plateRect[1], ",", fixed=TRUE)))
    plate.offset.x.cm = plateRect[1] * cmPerPixel
    plate.offset.y.cm = plateRect[2] * cmPerPixel
    plate.width.cm = plateRect[3] * cmPerPixel
    plate.height.cm = plateRect[4] * cmPerPixel

    # Retrieve the plate basic data.
    plateExperiment = df$experiment[1]
    plateGenotype = df$genotype[1]
    plateReplicate = df$replicate[1]
    plateLetter = df$plateLetter[1]

    # Generate the plot title string by making replacements.
    title = replaceTemplateString(plotTitle, df[1,])

    # Start the plot.
    plot(NA, type="n", xlim=xlim, ylim=ylim, axes=FALSE, frame.plot=FALSE,
        xlab="", ylab="", cex.main=title.cex, main=title)
    # Plot a rectangle to represent the plate.
    rect(plateLowerLeft.usr[1], plateLowerLeft.usr[2], plateUpperRight.usr[1], plateUpperRight.usr[2])
    # If sided, put a vertical line down the middle.
    if (sided)
        {
        mid.plate = (plateLowerLeft.usr[1]+plateUpperRight.usr[1])/2
        lines(rep(mid.plate, 2), c(plateLowerLeft.usr[2], plateUpperRight.usr[2]), col=addAlpha("red", 0.3), lwd=6)
        }

    # seed.x and seed.y are the expected plate position of next seed.
    seed.x = plateLowerLeft.usr[1] + seed1X_L.cm
    seed.y = plateUpperRight.usr[2] - seed1Y.cm

    # Now loop for each row of seeds and plot them.
    for (row in 1:seedRows)
        {
        seedRowOffset = (row-1)*3

        # If requested, plot the data labels column.
        if (plotPhenotypeDataPctWidth > 0)
            {
            label.x = labelOffsetX.usr + labelWidthX.usr/2
            label.y = plateSize.cm - (seed1Y.cm+(row-1)*seedSpaceY.cm)
            ty = label.y
            if (seedNumLabel)
                {
                ty = ty - dataHeight.usr
                text(label.x, ty, "seedNum", cex=data.cex, adj=c(0.5,0), font=2)
                }
            for (trait in plotPhenotypeCols)
                {
                ty = ty - dataHeight.usr
                text(label.x, ty, trait, cex=data.cex, adj=c(0.5, 0))
                }
            }

        # Loop for each seed on this row and plot the seeds and their data.
        plotColor = plotColors[1]
        for (col in 1:seedCols)
            {
            seedNum = getSeedNumFromRowCol(row, col)
            seedType = getSeedTypeFromRowCol(row, col, df)
            if (!is.na(seedType))
                dfSeed = df[df$seed == seedNum & df$type == seedType,]
            else
                {
                dfSeed = df[df$seed == seedNum,]
                seedType = "(types not used)"
                }
            if (requireAllSeeds)
                {
                if (nrow(dfSeed) != 1)
                    cat("Expected 1 root for seed ", seedNum, " of type ",
                        seedType, " but instead there were ", nrow(dfSeed), ", ",
                        getFileSeedStr(dfSeed, 1, templateStr), " and ", getFileSeedStr(dfSeed, 2, templateStr), "\n")
                }
            else
                {
                if (nrow(dfSeed) > 1)
                    cat("Expected at most 1 root for seed ", seedNum, " of type ",
                        seedType, " but instead there were ", nrow(dfSeed), ", ",
                        getFileSeedStr(dfSeed, 1, templateStr), " and ", getFileSeedStr(dfSeed, 2, templateStr), "\n")
                }
            if (nrow(dfSeed) == 0)
                for (colNum in ncol(dfSeed))
                    dfSeed[1,colNum] = NA
            dfSeed = dfSeed[1,]

            # If requested, plot each root's phenotype data values to the left
            # of the plate.
            if (plotPhenotypeDataPctWidth > 0)
                {
                data.x = dataOffsetLeftX.usr + (col-1)*dataColWidth.usr
                if (sided && col > colsOneSide)
                    data.x = dataOffsetRightX.usr + (col-colsOneSide-1)*dataColWidth.usr
                data.y = label.y
                if (seedNumLabel)
                    {
                    text(data.x, data.y-1*dataHeight.usr, seedNum, cex=data.cex, adj=c(0,0), font=2)
                    data.y = data.y - dataHeight.usr
                    }
                # Note: Use paste() to convert NA to "NA" string.
                for (trait in plotPhenotypeCols)
                    {
                    data.y = data.y - dataHeight.usr
                    text(data.x, data.y, paste(dfSeed[, trait]), cex=data.cex, adj=c(0,0))
                    }
                }

            # Plot seed number label.
            if (!is.na(seedNum.dy))
                text(seed.x, seed.y+seedNum.dy, seedNum, adj=c(0.5,0.5), cex=seedNum.cex)

            # Now plot the roots themselves.
            xy = dfSeed$pts
            if (!is.na(seedFlags.dy))
                txt.y = seed.y+seedFlags.dy # Position for text above presumed position of seed.
            if (is.na(xy))
                {
                # Plot "N/A" above presumed position of seed.
                if (!is.na(seedFlags.dy))
                    text(seed.x, txt.y, "NA", adj=c(0.5,0.5), cex=seedFlags.cex)
                }
            else
                {
                # Plot the root tracing polyline.
                xy = unlist(strsplit(xy, ";", fixed=TRUE))
                x = sapply(xy, function(P) unlist(strsplit(P, ",", fixed=TRUE))[1])
                y = sapply(xy, function(P) unlist(strsplit(P, ",", fixed=TRUE))[2])
                scale = dfSeed$cM/dfSeed$scale
                x = as.numeric(x) * scale
                y = as.numeric(y) * scale
                xt = cumsum(x)
                yt = cumsum(y)
                xt = xt - plate.offset.x.cm
                yt = yt - plate.offset.y.cm
                xt = plateLowerLeft.usr[1] + xt
                yt = plateUpperRight.usr[2] - yt
                lines(xt, yt, col=plotColor)
                i = which(plotColor == plotColors)
                Ncol = length(plotColors)
                i = (i %% Ncol) + 1
                plotColor = plotColors[i]
                # Above the seed position, plot [CA] with "C" if "collide" flag
                # is TRUE, "A" if "along" flag is TRUE.
                if (!is.na(seedFlags.dy))
                    if (dfSeed$collide || dfSeed$along)
                        text(seed.x, txt.y, adj=c(0.5,0.5), cex=seedFlags.cex,
                            paste("[", ifelse(dfSeed$collide, "C", ""), ifelse(dfSeed$along, "A", ""), "]", sep=""))
                }

            # Advance seed.x and seed.y to position of next seed.
            if (col < seedCols)
                seed.x = seed.x + seedSpaceX.cm
            else
                {
                seed.x = plateLowerLeft.usr[1] + seed1X_L.cm
                seed.y = seed.y - seedSpaceY.cm
                }
            }
        }
    }
dev.off()

################################################################################
# Make bar plots or bean plots of the data for each trait, subsetting by each
# co-variate given by beanSubsetCols.
################################################################################

# Loop for each trait and make a bar plot or bean plot for each one.
{
pdf(beanPlotFile, height=bean.pdf.height.in, width=bean.pdf.width.in)
cat("Bean plots\n")
for (t in beanTraits)
	{
	cat("  Trait: ", t, "\n")

    for (covar in beanSubsetCols)
        {
        cat("    Co-variate: ", covar, "\n")

        # For SIGN OF ANGLE traits, just make a bar plot.
        if (grepl("Sign$", t))
            {
            V = dfRoots[, t]
            V2 = rep("0", length(V))
            V2[V == -1] = "NEG"
            V2[V == +1] = "POS"
            x = tapply(factor(V2, levels=c("0", "NEG", "POS")), dfRoots[, covar], table)
            mtx = do.call(rbind, x)
            mtx = mtx[, c("NEG", "0", "POS")]
            # Sort by reverse total of NEG, "0", and POS.
            mtx = mtx[order(apply(mtx, 1, sum), decreasing=TRUE),,drop=FALSE]
            txt = c(NEG="Root grows left", `0`="Root grows straight down", POS="Root grows right")
            txt = txt[colnames(mtx)]
            barplot(t(mtx), beside=TRUE, col=c("blue", "black", "red"), las=2, cex.names=0.7,
                main=paste("Trait:", t, "Co-variate:", covar),
                xlab=covar, ylab="Number of Roots", legend.text=txt)
            }

        # For numeric traits, make a bean plot.
        else
            {
            # Get the mean value of the trait in each level of the co-variate (all
            # are categoric) and sort the levels by mean value, then refactor the
            # co-variate in sorted order.
            mean.values = tapply(dfRoots[, t], dfRoots[, covar], mean)
            mean.values = sort(mean.values)
            dfRoots[,covar] = factor(dfRoots[,covar], names(mean.values))
            covar.levels = names(mean.values)
            N = length(covar.levels)

            # Make some text size magnification estimates.
            cex = 60/N # A weird guess...
            dx = 0.5
            # Here's a correction for when there are very few beans.
            if (cex > 0.8)
                {
                cex = 0.8
                dx = -4/N
                }

            # Make a bean plot.
            formula = as.formula(paste(t, " ~ ", covar, sep=""))
            beanplot(formula, data=dfRoots, show.names=FALSE, font.axis=2, grownage=8, ll=0.08, method="jitter",
                border="black", col=c("white", "black", "black", "blue"), frame.plot=FALSE, beanlinewd=1, maxwidth=0.98,
                main=paste("\n\n\n\n", "Trait:", t, "Co-variate:", covar), xlab=covar, ylab=t, log="")
            xvals = 1:N
            beanLabels = covar.levels
            mtext(beanLabels, 1, line=0.5, at=xvals, cex=cex, font=2, las=3)

            # Add annotation of number of samples per experiment to beans.
            ymin = par("usr")[3]
            numSamps = tapply(dfRoots[,t], dfRoots[,covar], length)
            text(xvals, ymin, numSamps[beanLabels], cex=0.5, font=2, srt=90, pos=4, offset=0, col="black")
            }
        }
	}
dev.off()
}

