################################################################################
# This file contains R code to test possible linear models to be fit to root
# phenotype data from the Brady Lab root RNAseq experiment using tomato
# introgression lines (ILs) with RNAseq data first obtained in June 2013.
# This code reads measured and pre-processed phenotype data, including root
# lengths and various measures of root angle, and fits various linear models
# to it to discover which terms have most effect on the phenotypes.
#
# EDIT FILE Include_setAnalysisAndDate.R BEFORE RUNNING THIS CODE.
################################################################################

# Set working directory.
setwd("~/Documents/UCDavis/BradyLab/GrowthExperiments/Tomato/RootRNAseqJun2013_Phenotypes")

################################################################################
# Include files.
################################################################################

source("Include_setAnalysisAndDate.R")
source("Include_general.R")
source("Include_xlsx.R")
source("Include_dataColumnTypes.R")

if (analysisVariant != "Orig")
    stop("This file has not been updated to handle any analysisVariant other than Orig")

################################################################################
# Load libraries used by this.
################################################################################

library("lme4")

################################################################################
# Set basic constants controlling the analysis.
################################################################################

# Minimum R^2, in percent, of an effect term in order to include it in the model.
minR2pct = 3

# Output directory name.
outputFolder = "ModelTesting"

# Set output filename prefix, which prefixes all output files, and includes the
# output folder at the beginning of it.
rootOutputPrefix = paste(outputFolder, paste(outputFolder, "_", sep=""), sep=PATHSEP)

# Input data filename.  The entire input data file is copied to the output folder.
inputFilename = paste(dataFilePrefix, processedDataFile, sep="")

# Input file pathname.
processedDataPath = paste(dataDir, inputFilename, sep=PATHSEP)

# Set this TRUE to remove all roots whose "collide" value is TRUE (root collided
# with another root).
rmvCollideRoots = FALSE

# Set this TRUE to remove all roots whose "along" value is TRUE (root grew
# alongside another root).
rmvAlongRoots = TRUE

# Set criteria for data filtering.
minPtsPerGenotypePerPlateDate = 3

# Synthetic data columns can be created by concatenating data in other columns.
# Specify such columns here.  This is particularly useful for nested factors.
# If column B is nested within column A, then creating a new column A.B that
# joins the two column values (with a "." separator) allows specification of
# A.B within a model, resulting in a nested effect (lmer() manual recommends
# this approach).
# For each synthetic data column, specify a template string here, with data
# column names surrounded by % characters.  These will be replaced by the value
# in those columns to form a new string that is placed in a new column whose
# name is the name of the string in this vector. #####
synColTemplates = c(
    date.gt="%plateDate%.%genotype%",                   # This is interaction term plateDate:genotype, needed sometimes.
    date.gt.pl="%plateDate%.%genotype%.%plateLetter%",  # Plates are nested within plateDates and genotypes.
    date.pp="%plateDate%.%platePos%",                   # Plate positions are nested within plateDates.
    date.gd="%plateDate%.%germDay%",                    # Germination days are nested within plateDates.
    gt.col="%genotype%.%collide%",                      # Collide flag nested within genotype (Dan K).
    gt.root="%genotype%.%platingAge%",                  # Plating age nested within genotype.
    gt.germ="%genotype%.%germAge%",                     # Germination age nested within genotype.
    NULL) # NULL is not included in a vector.

# Additional synthetic columns might be created by applying a mathematical
# relationship, represented by an R formula, to other columns.  Specify the
# formulas in this list, with the formula element names being the new column
# names.  Use I() to surround arithmetic expressions.
formulaCols = list(
    # pg.age="I(platingAge - germAge)"
    )

################################################################################
# Get traits that were measured.
################################################################################

# Get traits.
dfTraits = getTraits()
traits = rownames(dfTraits)

# Save the dfTraits data frame in the output folder.
filename = paste(rootOutputPrefix, "traits.tsv", sep="")
write.table(dfTraits, filename, sep="\t", row.names=TRUE, col.names=TRUE)
# dfTraits = read.table(filename, header=TRUE, row.names=1, sep="\t", stringsAsFactors=FALSE)

# Remove the "Sign" traits, we won't deal with those.
#traits = traits[!grepl("Sign", traits)]

# Get number of traits.
ntraits = length(traits)

################################################################################
# Create log file.
################################################################################

# Create the output folder if it doesn't exist.
if (!file.exists(outputFolder))
    dir.create(outputFolder)

# Make an empty log file to start logging our activity into.
logfile = paste(rootOutputPrefix, "logfile.txt", sep="")
setDefaultLogFile(logfile)
printLog("RootRNAseqJune2013 phenotype modelling analysis log file", logfile, "\n", append=FALSE)
printLog("There are", ntraits, "traits in total:\n", getPrint(traits, "    "), "\n")

################################################################################
# Read the input data file.  This file has already been thoroughly vetted for
# correctness by an earlier pre-processing stage.  It should never be edited.
# If something needs changed in it, change the raw input data or change the
# pre-processing code that makes it.
# If columns are changed, colClasses will need to be changed too.
################################################################################

dfRoots = read.xlsx2(processedDataPath, sheetIndex=1, header=TRUE,
    stringsAsFactors=FALSE, colClasses=processedDataColClasses)

################################################################################
# Write the data frame to the output folder and show a summary.
################################################################################

# Write the data frame to the output folder.
outputFilePath = paste(outputFolder, inputFilename, sep=PATHSEP)
make.xlsx(dfRoots, outputFilePath, freezeTopRow=TRUE, halignDefault="CENTER")

# Show summary of data.
printLog("Data column names:\n")
printLog(colnames(dfRoots), "\n")
printLog("Data summary:\n")
printLog(getPrint(summary(dfRoots)))

################################################################################
# Get genotypes and plateDates.
################################################################################

# Get genotypes and number of genotypes.
genotypes = unique(dfRoots$genotype)
ngenotypes = length(genotypes) #how long is the vector (140 #genotypes)
printLog("There are", ngenotypes, "genotypes in total:\n")
printLog(genotypes, "\n", fill=70, labels="   ")

# Get plateDate names and number of plateDates.
plateDates = unique(dfRoots$plateDate)
nplateDates = length(plateDates)
printLog("There are", nplateDates, "plateDates in total: ", paste(plateDates, collapse=","), "\n")

################################################################################
# Create synthetic columns using the defined template strings and formulas.
################################################################################

for (col in names(synColTemplates))
    dfRoots[,col] = replaceTemplateString(synColTemplates[col], dfRoots)

for (col in names(formulaCols))
    dfRoots[,col] = eval(parse(text=formulaCols[[col]]), dfRoots)

################################################################################
# The data was obtained in multiple growth experiments, as indicated by the date
# in the plateDate column.  Samples from different experiments were divvied up
# into four biological replicates numbered REP2 through REP5 (REP1 was a
# practice rep).  Each biorep might have samples from more than one growth
# experiment.  The last one centimeter of most roots was harvested and RNA
# extracted for an RNAseq experiment.  Just prior to harvest, the plate was
# imaged.  Each root that grew, didn't hit plate walls, and wasn't obscured
# by labels was measured by tracing it and recording the centerline coordinates.
# These coordinates were analyzed to generate the traits given in traits[].
# There are no NA entries in the trait data, all roots in dfRoots have a value
# for each trait.
# In some cases, some ILs may have insufficient roots to be analyzed.  While the
# insufficient data could be passed to the modelling code, we prefer to remove
# those ILs here and list them up front so we know they have been excluded.
# Later, analyses using subsets of data may do the same thing, so we define a
# function here to subset the data.
################################################################################

# Make reduced data set from the data frame df.
# Arguments:
#   minPts: remove genotypes for which there are fewer than minPts roots in each
#       plateDate, and at least one plateDate with >= minPts roots.  Any data
#       points in a plateDate with fewer than "minPts" data points are discarded.
#       Any genotypes without at least one plateDate remaining are discarded.
#   rmvCollide: if TRUE, remove roots whose "collide" column is TRUE (root
#       collided with another).
#   rmvAlong: if TRUE, remove roots whose "along" column is TRUE (root grew
#       alongside another).
# Return NULL if all data is removed, else return a list with these members:
#       data: data frame containing the remaining data, with several columns
#           converted to factors.  Certain columns are removed because we don't
#           need them here and they are in the way: filename, fileDate, comment,
#           plateRect, pts.
#       numRoots: matrix with genotypes (ILs) as row names, plateDates as column
#           names, and entries the total number of roots for that IL in that
#           plateDate AFTER removing cases with inadequate data, with all ILs
#           and plateDates present in the matrix.
#       genotypes: the genotypes (ILs) remaining.
#       ngenotypes: number of ILs in data.
#       plateDates: the plateDates in data.
#       nplateDates: the number of plateDates in data.
reduceData = function(df, minPts, rmvCollide, rmvAlong, verbose=TRUE)
    {
    # Remove roots with collide=TRUE.
    if (rmvCollide)
        {
        N = nrow(df)
        Nc = sum(df$collide)
        Nr = N-Nc
        if (Nc > 0)
            {
            printLog("Out of", N, "total roots,", Nc, "were removed because they collide with another one, which leaves", Nr, "roots\n")
            df = df[!df$collide,]
            }
        }

    # Remove roots with along=TRUE.
    if (rmvAlong)
        {
        N = nrow(df)
        Na = sum(df$along)
        Nr = N-Na
        if (Na > 0)
            {
            printLog("Out of", N, "total roots,", Na, "were removed because they grew alongside another one, which leaves", Nr, "roots\n")
            df = df[!df$along,]
            }
        }

    # Remove unneeded columns.
    df = df[, setdiff(colnames(df), c("filename", "fileDate", "comment", "plateRect", "pts"))]

    # Make list for returning data.
    rdata = list()

    # Save original genotypes and plateDates.
	old.genotypes = unique(df$genotype)
	old.ngenotypes = length(old.genotypes)
	old.plateDates = unique(df$plateDate)
	old.nplateDates = length(old.plateDates)

    # Count number of roots of each genotype in each plateDate.
	numRoots = tapply(1:nrow(df), list(df$genotype, df$plateDate), length)
	numRoots[is.na(numRoots)] = 0
    # Remove rows not meeting the minPts requirement.
	df = df[apply(df, 1, function(d) numRoots[d["genotype"],d["plateDate"]] >= minPts),]
	# Check for no data left.
	if (nrow(df) == 0)
		{
		if (verbose)
    		printLog("No samples that satisfy the minimal data criteria!!!\n")
		return(NULL)
		}

	# Convert columns likely to be used in linear modelling to factors.
	df$replicate = factor(df$replicate)
	df$genotype = factor(df$genotype)
	df$platePos = factor(df$platePos)
	df$plateLetter = factor(df$plateLetter)
	df$germDay = factor(as.character(df$germDay))
	df$collide = factor(df$collide)
	df$plateDate = factor(df$plateDate)
	# Convert all synthetic columns too.
	for (col in names(synColTemplates))
    	df[,col] = factor(df[,col])

	# Set m82 as reference level.
	if (!any(df$genotype == "M82"))
		{
		if (verbose)
    		printLog("No M82 samples that satisfy the minimal data criteria!!!\n")
		return(NULL)
		}
	df$genotype = relevel(df$genotype, ref="M82")

	# Save the reduced data set in rdata list.
	rdata$data = df

    # If verbose, display number of roots over all genotypes.
    if (verbose)
        {
        numRootsInGenotype = tapply(1:nrow(df), df$genotype, length)
        numRootsInGenotype = numRootsInGenotype[!(names(numRootsInGenotype) %in% parents)]
        printLog("   Minimum number of roots for any IL:", min(numRootsInGenotype), "\n")
        printLog("   Maximum number of roots for any IL:", max(numRootsInGenotype), "\n")
        printLog("   Mean number of roots for all ILs:", mean(numRootsInGenotype), "\n")
        printLog("   SD for number of roots for all ILs:", sd(numRootsInGenotype), "\n")
        printLog("\n")
        }

	# Count number of roots remaining for each genotype and plateDate.
	rdata$numRoots = tapply(1:nrow(df), list(df$genotype, df$plateDate), length)
    rdata$numRoots[is.na(rdata$numRoots)] = 0

	# If verbose, display number of roots over all plateDates.
	if (verbose)
	    {
        numRoots = rdata$numRoots
        numRoots = numRoots[!rownames(numRoots) %in% parents,]
        printLog("   Min and max number of roots for ILs in each plateDate:\n")
        for (plateDate in colnames(numRoots))
            {
            x = numRoots[, plateDate]
            x = x[x != 0]
            printLog("    ", format(plateDate, width=6), " min:", format(min(x), width=3), " max:", format(max(x), width=3), "\n")
            }
        printLog("\n")
        }

    # Get remaining genotypes and plateDates and save in rdata.
	new.genotypes = levels(df$genotype)
	new.ngenotypes = length(new.genotypes)
	new.plateDates = levels(df$plateDate)
	new.nplateDates = length(new.plateDates)

	# Tell the user which genotypes were eliminated and which retained as a result of above.
	rmvd.genotypes = setdiff(old.genotypes, new.genotypes)
	rmvd.ngenotypes = length(rmvd.genotypes)
	printLog("   There are", new.ngenotypes, "genotypes remaining after removing those with insufficient data:\n")
	printLog(new.genotypes, "\n", fill=70, labels="   ")
	printLog("   There were", rmvd.ngenotypes, "genotypes removed.  They were:\n")
	printLog(rmvd.genotypes, "\n", fill=70, labels="   ")
	return(rdata)
	}

# Now reduce the overall data set, removing ILs with insufficient data.
printLog("Reducing data set to include only those genotypes with at least",
    minPtsPerGenotypePerPlateDate, "points in each plateDate.\n")
rdata = reduceData(dfRoots, minPts=minPtsPerGenotypePerPlateDate, rmvCollide=rmvCollideRoots, rmvAlong=rmvAlongRoots)
data.all = rdata$data

################################################################################
# Model testing 1.  Using lm(), simple fixed-effect tests to get an idea.
################################################################################

# Note: we could classify platingAge and germAge as traits also, but we won't.
# Instead, we'll test to see if those are correlated with other traits.

# Question 1: Is genotype significant?
printLog("Model testing: genotype effects\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "genotype", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# All traits are highly significant for genotype, with three stars for all
# traits except wholeAngSign, which has one star, and avgAngSign with two.

# Question 2: looking only at M82 and PENN, is measurer significant in interaction?
printLog("Model testing: measurer effects M82 and PENN\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "genotype + genotype:measurer", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all[data.all$genotype %in% c("M82","PENN"),]
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# I didn't think it would be, but it IS significant, for all traits except waviness
# and the Sign traits.
# Siobhan may have measured a lot more colliding roots than Livingstone.  If we
# remove those, is measurer still significant?
printLog("Model testing: measurer effects M82 and PENN when colliding roots removed\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "genotype + genotype:measurer", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all[data.all$genotype %in% c("M82","PENN"),]
    data.r = data.r[data.r$collide == "FALSE",]
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# That got rid of measurer significance EXCEPT with wholeAng, avgAbsAng,
# quarterAngSign, and avgAngSign.  avgAbsAng is our most important trait.
# Siobhan is actually 10 degrees more than Livingstone for avgAbsAng, and
# 12 degrees for wholeAng.  As a result of this, I had Siobhan measure
# several plates of PENN roots that Livingstone measured, and then analyzed
# the significance of measurer in separate files names 2/3/4x.*.  In that
# analysis, measurer showed no effect whatsoever.  So, I'm not going to worry
# about including it in the model.

# Question 3: looking only at M82 and PENN individually and in interaction,
# is collide significant?
printLog("Model testing: collide effects M82 and PENN\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "collide*genotype", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all[data.all$genotype %in% c("M82","PENN"),]
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# "collide" is significant in wholeAng and avgAbsAng.  The collide coefficient
# is quite significant in avgAbsAng at 0.0014.
# The interaction effect is significant in straightLen, waviness, and avgAbsAng.
# Test with all genotypes.
printLog("Model testing: collide effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "collide*genotype", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    lm.1.s$coefficients = lm.1.s$coefficients[grepl("collide", rownames(lm.1.s$coefficients)) & lm.1.s$coefficients$stars != "",]
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# "collide" is HIGHLY significant in all traits (single * in Sign traits).
# The interaction effect is not significant in any traits.  However, the
# interaction coefficient is highly significant for many genotypes in many
# traits.
# Roots are more likely to collide if they grow at a high angle.  Therefore,
# I'm concerned that a "collide" term would take away variance preferentially
# from angled genotypes in the angle traits.  In avgAbsAng, the collide
# coefficient is 10 degrees, but the interaction coefficients that are
# significant are generally negative values close to 10 degrees, in high-angle
# ILs, including IL7-4-1 and IL7-5-5 and Penn.  I don't understand the negative
# angle.  I think it means that when a collision occurs, the root tends to grow
# straighter.
# Actually, now that I think about it, the right thing to do is to include
# ONLY the interaction effect, since the degree of influence of collision
# depends greatly on the genotype.  Let's try it.
printLog("Model testing: collide interaction with genotype effects\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "genotype + collide:genotype", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    lm.1.s$coefficients = lm.1.s$coefficients[grepl("collide", rownames(lm.1.s$coefficients)) & lm.1.s$coefficients$stars != "",]
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Interaction effects are significant in rootLen, wholeAng, and avgAbsAng.  Some
# coefficients are very significant in all the traits.  I think this is the way
# to go, but only in those traits that need it.
# Dan Kliebenstein says not to do interaction effects if genotypes are different,
# which can be tested with a chi squared test.  He said to nest collide within
# genotype instead.  I don't quite understand this, I thought nested was the same
# as interaction, but I guess I'm wrong.  Maybe the thing is that when it is
# interaction, the interactions across all genotypes must follow some distribution,
# whereas nested keeps each genotype completely separate for the collide effect?
# Do the chi-squared test.
df = data.frame(collide=tapply(data.all$collide == TRUE, data.all$genotype, sum),
    no.collide=tapply(data.all$collide == FALSE, data.all$genotype, sum),
    stringsAsFactors=FALSE)
cstr = chisq.test(as.matrix(df))
printLog("chi-squared test for collide being different among genotypes:\n")
printLog(getPrint(cstr, "   "), "\n")
# 1e-15, highly significant.
# Let's retest collide using the gt.col column, which nests collide within
# genotype.  Will results be different??
printLog("Model testing: collide nested within genotype effects\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "genotype + gt.col", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    lm.1.s$coefficients = lm.1.s$coefficients[grepl("gt.col", rownames(lm.1.s$coefficients)) & lm.1.s$coefficients$stars != "",]
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# It seems to give exactly the same p-values, and the same ILs have sig. parameters,
# but the parameters are in a different order and the values are different.  However,
# the p-values of the parameters are also identical.  I think this is no different
# than a genotype:collide interaction.
# Now Dan is saying to use a collide main effect only.  Let's try that.
printLog("Model testing: collide effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "collide + genotype", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    lm.1.s$coefficients = lm.1.s$coefficients[grepl("collide", rownames(lm.1.s$coefficients)) & lm.1.s$coefficients$stars != "",]
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Well, it is significant in all traits and it subtracts 5 degrees off of avgAbsAng
# in all colliding roots.  How strongly is colliding roots correlated with avgAbsAng?
x = tapply(dfRoots$avgAbsAng, dfRoots$genotype, mean)
y = tapply(as.numeric(dfRoots$collide), dfRoots$genotype, mean)
plot(x, y, type="p", pch=20, xlim=c(0,50), ylim=c(0,1), xlab="avgAbsAng", ylab="collide freq")
# Not much, not nearly as much as I thought.  And Dan is right that the sum of squares
# for the collide main effect is large, but for the interaction effect is small, when
# both effects are included.

# Question 4: looking only at M82 and PENN, is plateDate significant?
printLog("Model testing: plateDate effects M82 and PENN\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate*genotype", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all[data.all$genotype %in% c("M82","PENN"),]
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# plateDate IS significant for all traits EXCEPT the Sign traits excluding
# avgAngSign (for which it is).  Although plateDate by genotype interaction is
# significant for some traits, it is NOT significant for the angle traits other
# than wholeAng and avgAbsAng.  Let's try all genotypes.  This takes a LONG time.
printLog("Model testing: plateDate effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate*genotype", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# This shows the plateDate, genotype, AND interaction terms significant for all
# traits, except Sign traits, where only genotype is significant, with the
# exception that the interaction term is NOT significant in quarterAng, tenthAng,
# and tipAng, and it IS significant in wholeAngSign.  I should try doing the
# interaction term as a random effect.

# Question 5: looking only at M82 and PENN, is plateLetter significant?
# Note that plate is nested within both plateDate and genotype (every plate
# was one genotype in one plateDate).  Therefore, we want to factor out the
# effects of plateDate and genotype first, then include plate as a factor.
# We do not need to do interaction effects with plate.  When we try to include
# plateDate:genotype interaction, it does not appear in results.  I think this
# is because it is subsumed by date.gt.pl, but I don't quite understand why,
# because we specify it first, and it should have fewer degrees of freedom.
# I make it work here by using the date.gt column.  This indicates I don't
# really know what I'm doing.
printLog("Model testing: plate effects M82 and PENN\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype + date.gt + date.gt.pl", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all[data.all$genotype %in% c("M82","PENN"),]
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Plate letter is not significant for any trait.  However, it (of course)
# factors away variance from the plateDate_genotype interaction term.  Let's
# check this using all genotypes.  Again this takes a LONG time.
printLog("Model testing: plate effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype + date.gt + date.gt.pl", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Plate letter is significant for all traits except waviness, quarterAng, and
# tipAng.  It looks like we should include it as a random effects term, but I'm
# worried about this one because it uses so many degrees of freedom.

# Question 6: We don't expect replicate to be significant, after factoring out
# plateDate (to some degree, replicate is equivalent to plateDate, but not
# completely).  Check it using all genotypes, leaving out the genotype by
# plateDate interaction.  Exclude UNKN replicates.
printLog("Model testing: replicate effects all genotypes, REP2-REP5 only\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype + replicate", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all[data.all$replicate %in% c("REP2", "REP3", "REP4", "REP5"),]
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    printLog(getPrint(lm.1.s$coefficients[lm.1.s$coefficients$stars != "" & grepl("replicate", rownames(lm.1.s$coefficients)),],
        "     "), "\n")
    }
# It is significant in rootLen(**), waviness(*), quarterAng(*), and avgAngSign(*).
# For the coefficients, a number of REP coefficients are quite significant.
# Looking only at coefficients with at least ** significance, there is only REP3
# and REP5 in rootLen.  Looking at avgAbsAng, REP3 and REP4 have one *.
# Since the significance is borderline, and here we've left out the
# genotype x plateDate interaction, I don't think we need to worry about this.

# Question 7: Is amount of time after plating when germination occurred (germAge)
# significant?  By itself and in interaction with genotype.
printLog("Model testing: germAge effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype*germAge", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# In general, germAge is significant and its interaction with genotype is not.
# Exceptions are that it is not significant for the Signs, it is the other way
# around with avgAbsAng, and the interaction term is sig. in the lengths,
# waviness, wholeAng, and avgAngSign.  Should we use the interaction term???

# Question 8: Is platingAge significant?  By itself and in interaction with genotype.
printLog("Model testing: platingAge effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype*platingAge", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Both terms are highly significant for length.  For other traits, only the
# interaction term is significant, if either is, and these traits are waviness
# and avgAbsAng (both **).  Should we use the interaction term???  Test ONLY
# that term.
printLog("Model testing: platingAge interaction effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype + genotype:platingAge", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Significant for all except tipAng, tenthAng, quarterAng and the Signs.  It
# sounds like we should have a platingAge term and its interaction for length
# traits, and interaction term for all traits except maybe tipAng, tenthAng,
# quarterAng, and signs.

# Question 9: Is rootAge (post-germination age, platingAge-germAge) significant?
printLog("Model testing: rootAge effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype*rootAge", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    #lm.1.s = summary(lm.1)
    #lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    #lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    #printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# It is highly significant for all traits except avgAbsAng and the Signs, like
# germAge.  The interaction term is only significant for avgAbsAng.  I guess this
# one doesn't add anything that would permit us to dispense with both germAge and
# platingAge.  How about if we were to include both germAge and platingAge, and try
# both orders?
printLog("Model testing: platingAge+germAge effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype + platingAge + germAge", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    }
# germAge is still coming out significant.  Try the other way.
printLog("Model testing: germAge+platingAge effects all genotypes\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "plateDate + genotype + germAge + platingAge", sep="~"))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = data.all
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    }
# platingAge is still sig. for the lengths plus quarterAng.  We should include a
# platingAge term in quarterAng and the lengths, and put germAge first.  Use a
# germAge term for all except avgAbsAng.

# That was model testing with lm().  Was that even right?  Should I instead be
# using lmer() and random effects?  Should I do it again with lmer()?
# Let's summarize the results we got.
#   1. genotype is our primary fixed-effect term
#   2. measurer is not a factor
#   3. collide should be a fixed effect.
#   4. plateDate must be a random effect.
#   5. Test genotype x plateDate interaction random effect.
#   6. plateLetter should probably be a random effect.  Since it is nested in plateDate
#           and genotype, its presence is equivalent to a genotype x plateDate interaction.
#   7. replicate is not a factor
#   8. germAge should be a fixed effect, I think, because we are exhausting its values.
#           However, Dan said to make it a random effect.  Also, we should test the
#           interaction with genotype (gt.germ).
#   9. platingAge is only needed as a random effect for length phenotypes and maybe
#           quarterAng.  Also, we should test the interaction with genotype (gt.root).

################################################################################
# Model testing 2.  Using lmer(), mixed-effects, compare models of increasing
# sophistication.
################################################################################

data.r = data.all
filename = paste(rootOutputPrefix, "lmer.txt", sep="")
printLog("Output to file '", filename, "'\n")
outFiles = c(NA, filename)
printLog("Model testing increasingly sophisticated models with lmer()\n", file=outFiles, append=FALSE)

term.date = " + (1|plateDate)"
term.collide = " + collide"
term.gt = " + genotype"
term.germ = " + (1|germAge)"
term.gtgerm = " + (1|gt.germ)"
term.age = " + (1|platingAge)"
term.gtage = " + (1|gt.root)"
term.dategt = " + (1|date.gt)"
term.plate = " + (1|date.gt.pl)"

form0 = term.date
form1 = paste(term.collide, term.date, sep="")
form2 = paste(term.collide, term.gt, term.date, sep="")
form3 = paste(term.collide, term.gt, term.date, term.germ, sep="")
form3x = paste(term.collide, term.gt, term.date, term.germ, term.gtgerm, sep="")
form4 = paste(term.collide, term.gt, term.date, term.germ, term.age, sep="")
form4x = paste(term.collide, term.gt, term.date, term.germ, term.age, term.gtage, sep="")
form5 = paste(term.collide, term.gt, term.date, term.germ, term.dategt, sep="")
form6 = paste(term.collide, term.gt, term.date, term.germ, term.plate, sep="")
form7 = paste(term.collide, term.gt, term.date, term.germ, term.age, term.dategt, sep="")
form8 = paste(term.collide, term.gt, term.date, term.germ, term.age, term.plate, sep="")
form9 = paste(term.collide, term.gt, term.date, term.germ, term.dategt, term.plate, sep="")
form10 = paste(term.collide, term.gt, term.date, term.germ, term.age, term.dategt, term.plate, sep="")

for (trait in traits)
    {
    printLog("\n*************** ", trait, "\n", file=outFiles)

    term0 = paste(trait, " ~ 1", sep="")
    formula0 = as.formula(paste(term0, form0, sep=""))
    formula1 = as.formula(paste(term0, form1, sep=""))
    formula2 = as.formula(paste(term0, form2, sep=""))
    formula3 = as.formula(paste(term0, form3, sep=""))
    formula3x = as.formula(paste(term0, form3x, sep=""))
    formula4 = as.formula(paste(term0, form4, sep=""))
    formula4x = as.formula(paste(term0, form4x, sep=""))
    formula5 = as.formula(paste(term0, form5, sep=""))
    formula6 = as.formula(paste(term0, form6, sep=""))
    formula7 = as.formula(paste(term0, form7, sep=""))
    formula8 = as.formula(paste(term0, form8, sep=""))
    formula9 = as.formula(paste(term0, form9, sep=""))
    formula10 = as.formula(paste(term0, form10, sep=""))

    fm0 = lmer(formula0, data=data.r, REML=FALSE)
    #printLog(getPrint(fm0, "   "), "\n", file=outFiles)
    fm1 = lmer(formula1, data=data.r, REML=FALSE)
    #printLog(getPrint(fm1, "   "), "\n", file=outFiles)
    fm2 = lmer(formula2, data=data.r, REML=FALSE)
    #printLog(getPrint(fm2, "   "), "\n", file=outFiles)
    fm3 = lmer(formula3, data=data.r, REML=FALSE)
    #printLog(getPrint(fm3, "   "), "\n", file=outFiles)
    fm3x = lmer(formula3x, data=data.r, REML=FALSE)
    #printLog(getPrint(fm3x, "   "), "\n", file=outFiles)
    fm4 = lmer(formula4, data=data.r, REML=FALSE)
    #printLog(getPrint(fm4, "   "), "\n", file=outFiles)
    fm4x = lmer(formula4x, data=data.r, REML=FALSE)
    #printLog(getPrint(fm4x, "   "), "\n", file=outFiles)
    fm5 = lmer(formula5, data=data.r, REML=FALSE)
    #printLog(getPrint(fm5, "   "), "\n", file=outFiles)
    fm6 = lmer(formula6, data=data.r, REML=FALSE)
    #printLog(getPrint(fm6, "   "), "\n", file=outFiles)
    fm7 = lmer(formula7, data=data.r, REML=FALSE)
    #printLog(getPrint(fm7, "   "), "\n", file=outFiles)
    fm8 = lmer(formula8, data=data.r, REML=FALSE)
    #printLog(getPrint(fm8, "   "), "\n", file=outFiles)
    fm9 = lmer(formula9, data=data.r, REML=FALSE)
    #printLog(getPrint(fm9, "   "), "\n", file=outFiles)
    fm10 = lmer(formula10, data=data.r, REML=FALSE)
    #printLog(getPrint(fm10, "   "), "\n", file=outFiles)
    an01 = anova(fm0, fm1)
    an12 = anova(fm1, fm2)
    an23 = anova(fm2, fm3)
    an33x = anova(fm3, fm3x)
    an34 = anova(fm3, fm4)
    an44x = anova(fm4, fm4x)
    an35 = anova(fm3, fm5)
    an36 = anova(fm3, fm6)
    an47 = anova(fm4, fm7)
    an48 = anova(fm4, fm8)
    an59 = anova(fm5, fm9)
    an910 = anova(fm9, fm10)
    printLog(" Add collide\n", file=outFiles)
    printLog(getPrint(an01, "    "), "\n", file=outFiles)
    printLog(" Add genotype\n", file=outFiles)
    printLog(getPrint(an12, "    "), "\n", file=outFiles)
    printLog(" Add germAge = base\n", file=outFiles)
    printLog(getPrint(an23, "    "), "\n", file=outFiles)
    printLog(" Add germAge interaction with genotype\n", file=outFiles)
    printLog(getPrint(an33x, "    "), "\n", file=outFiles)
    printLog(" Add RootAge to base => base2A\n", file=outFiles)
    printLog(getPrint(an34, "    "), "\n", file=outFiles)
    printLog(" Add RootAge interaction with genotype\n", file=outFiles)
    printLog(getPrint(an44x, "    "), "\n", file=outFiles)
    printLog(" Add plateDate:genotype to base => base2B\n", file=outFiles)
    printLog(getPrint(an35, "    "), "\n", file=outFiles)
    printLog(" Add plate to base\n", file=outFiles)
    printLog(getPrint(an36, "    "), "\n", file=outFiles)
    printLog(" Add plateDate:genotype to base2A\n", file=outFiles)
    printLog(getPrint(an47, "    "), "\n", file=outFiles)
    printLog(" Add plate to base2A\n", file=outFiles)
    printLog(getPrint(an48, "    "), "\n", file=outFiles)
    printLog(" Add plate to base2B => base3\n", file=outFiles)
    printLog(getPrint(an59, "    "), "\n", file=outFiles)
    printLog(" Add platingAge to base3 = full model\n", file=outFiles)
    printLog(getPrint(an910, "    "), "\n", file=outFiles)
    }
# Our terms were:
#   plateDate
#   collide
#   genotype
#   germAge
#   platingAge
#   plate = plateDate:genotype:plateLetter
#   genotype:germAge
#   genotype:platingAge
#   genotype:plateDate

# In most cases, each additional term is very significant, except:
#   1. germAge was not significant for waviness
#   2. platingAge is ONLY significant for straightLen and rootLen
#   3. plate was not significant for waviness or any angle trait except avgAbsAng
#   4. germAge and platingAge interactions with genotype are NEVER significant
#   5. plateDate interaction with genotype is ONLY sig. for straightLen and rootLen

# I think it is questionable whether to add BOTH date.gt and date.gt.pl.  

################################################################################
# Model testing 3.  Repeat above but remove non-significant terms for each trait
# before moving on to the next more sophisticated model.  Also remove terms
# whose R^2 isn't at least minR2pct.
################################################################################

# Effect terms we will include in formulas.
terms = c(
    term.date="(1|plateDate)",
    term.gt="genotype",
    term.collide="collide",
    term.germ="(1|germAge)",
    term.gtgerm="(1|gt.germ)",
    term.age="(1|platingAge)",
    term.gtage="(1|gt.root)",
    term.dategt="(1|date.gt)",
    term.plate="(1|date.gt.pl)")

# The corresponding effect names in the R2 object returned by compute.lmer.R2.
terms.R2 = c(
    term.date="re:plateDate",
    term.gt="fe:genotype",
    term.collide="fe:collide",
    term.germ="re:germAge",
    term.gtgerm="re:gt.germ",
    term.age="re:platingAge",
    term.gtage="re:gt.root",
    term.dategt="re:date.gt",
    term.plate="re:date.gt.pl")

# Make the same full-blown formulas as in previous section.  We will drop terms
# from these as we find them non-significant.
forms = list(
    form0=c("term.date"),
    form1=c("term.gt", "term.date"),
    form2=c("term.gt", "term.collide", "term.date"),
    form3=c("term.gt", "term.collide", "term.date", "term.germ"),
    form3x=c("term.gt", "term.collide", "term.date", "term.germ", "term.gtgerm"),
    form4=c("term.gt", "term.collide", "term.date", "term.germ", "term.age"),
    form4x=c("term.gt", "term.collide", "term.date", "term.germ", "term.age", "term.gtage"),
    form5=c("term.gt", "term.collide", "term.date", "term.germ", "term.dategt"),
    form6=c("term.gt", "term.collide", "term.date", "term.germ", "term.plate"),
    form7=c("term.gt", "term.collide", "term.date", "term.germ", "term.age", "term.dategt"),
    form8=c("term.gt", "term.collide", "term.date", "term.germ", "term.age", "term.plate"),
    form9=c("term.gt", "term.collide", "term.date", "term.germ", "term.dategt", "term.plate"),
    form10=c("term.gt", "term.collide", "term.date", "term.germ", "term.age", "term.dategt", "term.plate"))

# Which fit should each fit be compared to?
compareTo = c(form0=NA, form1="form0", form2="form1", form3="form2", form3x="form3", form4="form3",
    form4x="form4", form5="form3", form6="form3", form7="form4", form8="form4", form9="form5", form10="form9")

# Fit models above for each trait.
data.r = data.all
filename = paste(rootOutputPrefix, "lmer_dropTerms.txt", sep="")
printLog("Output to file '", filename, "'\n")
outFiles = c(NA, filename)
printLog("Model testing increasingly sophisticated models but dropping non-sig terms.\n", file=outFiles, append=FALSE)

for (trait in traits)
    {
    printLog("\n", file=outFiles)
    printLog("******************************************************************************\n", file=outFiles)
    printLog("*************** ", trait, "\n", file=outFiles)
    printLog("******************************************************************************\n", file=outFiles)

    term0 = paste(trait, " ~ 1", sep="")
    trait.forms = forms
    formulas = sapply(trait.forms, function(V) paste(term0, paste(terms[V], collapse=" + "), sep=" + "))

    # Loop through the formulas, fit the model, compare it to the less-sophisticated
    # model that it is to be compared to, and then:
    #   - if it is significant, great, just move on
    #   - if it is not significant, we want to remove the term(s) that is different
    #       in it than the model it was compared to, from ALL subsequent formulas
    #       and if doing so makes one of those formulas equal to another formula,
    #       the modified formula is removed from consideration by setting the
    #       formulas[] element to NA.  The formula that was non-significant is
    #       also removed from consideration the same way.  The NA means the
    #       formula will not be fit or compared later. 
    fits = list()
    for (form in names(formulas))
        {
        if (is.na(formulas[form]))
            next # Happens if we removed the formula because it now matches another formula.
        the.formula = as.formula(formulas[form])
        # Fit the model given by the formula to the data.
        fits[[form]] = lmer(the.formula, data=data.r, REML=FALSE)
        fm2 = fits[[form]]
        attr(fm2, "data") = data.r
        fm2 = compute.lmer.R2(fm2)
        # If a comparison is to be made, find the one to compare it to.
        compare.form = compareTo[form]
        if (!is.na(compare.form))
            {
            # If the one we are to compare to has been removed, look to its earlier
            # incarnations.
            while (is.na(formulas[compare.form]))
                {
                compare.form = compareTo[compare.form]
                if (is.na(compare.form))
                    break # First formula only.
                }
            }
        # If none to compare it to, display the fit and continue with next formula.
        if (is.na(compare.form))
            {
            printLog(" Terms so far: ", paste(terms[trait.forms[[form]]], collapse=", "), "\n", file=outFiles, sep="")
            # Display basic model fit results and anova results of the fit.
            printLog(getPrint(fm2, "   ", printFunc=print.lmerMod.abbrev), "\n", file=outFiles)
            printLog(getPrint(anova(fm2), "   "), "\n", file=outFiles)
            next # This one has NOTHING it can be compared to, it is currently the simplest.
            }
        # Get and display what is new in the new (form) formula.
        newTermNames = setdiff(trait.forms[[form]], trait.forms[[compare.form]])
        newTerms = terms[newTermNames]
        printLog("\n *** New term(s) added: ", paste(newTerms, collapse=", "), "\n", file=outFiles, sep="")
        # Display basic model fit results and anova results of the fit.
        printLog(getPrint(fm2, "   ", printFunc=print.lmerMod.abbrev), "\n", file=outFiles)
        printLog(getPrint(anova(fm2), "   "), "\n", file=outFiles)
        # Now do the comparison.
        fm1 = fits[[compare.form]]
        an = anova(fm1, fm2)
        # Display results.
        printLog(getPrint(an, "    "), "\n", file=outFiles, sep="")
        # Are the new terms significant?
        isSig = (an["fm2","Pr(>Chisq)"] < 0.05)
        # Do they have a sufficient R^2?
        R2 = attr(fm2, "R2")
        R2 = R2[terms.R2[newTermNames]]
        if (any(is.na(R2))) stop("R2 bad")
        isSolidR2 = any(round(100*R2) >= minR2pct)
        if (!isSig || !isSolidR2)
            {
            if (!isSig)
                printLog("   ******* ", paste(newTerms, collapse=", "),
                    " not significant, dropping from all formulas\n", file=outFiles, sep="")
            else
                printLog("   ******* ", paste(newTerms, collapse=", "), " has poor R2 < ", minR2pct,
                    "%, dropping from all formulas\n", file=outFiles, sep="")
            # Remove the formula itself from further consideration.
            formulas[form] = NA
            # Remove newTerms from later formulas, and if this makes them equal to
            # some other formula, remove it.  We need to remove newTermNames from
            # trait.forms too, since we use it.
            i = which(names(formulas) == form)
            for (other.form in names(formulas)[(i+1):length(formulas)])
                if (!is.na(formulas[other.form]))
                    {
                    for (term in newTerms)
                        formulas[other.form] = sub(paste(" + ", term, sep=""), "", formulas[other.form], fixed=TRUE)
                    trait.forms[[other.form]] = setdiff(trait.forms[[other.form]], newTermNames)
                    for (f in names(formulas))
                        if (f != other.form & !is.na(formulas[f]))
                            if (formulas[f] == formulas[other.form])
                                {
                                formulas[other.form] = NA
                                break
                                }
                    }
            }
        }
    }
# We ended up with these significant terms for each trait, all consistent with the
# previous model testing in the previous section.  However, the R^2 of some of the
# terms was very low so we dropped them even though they were significant.
#   trait           significant terms with high R2                                                  rejected low R2
#   -----------     -------------------------------------------------------------------------       ---------------------------
#   straightLen		genotype***, plateDate(R2=3%), germAge***, platingAge***, date.gt***, date.gt.pl** collide*
#   rootLen		    genotype***, plateDate(R2=3%), germAge***, platingAge***, date.gt, date.gt.pl      collide***
#   waviness		plateDate(R2=0%)                                                                collide***, genotype***, 
#   wholeAng		genotype***, collide*** (R2=11%), plateDate(R2=2%)                              germAge***
#   quarterAng		genotype***, plateDate(R2=3%), germAge***                                       collide***
#   tenthAng		genotype***, plateDate(R2=1%)                                                   collide***, germAge***
#   tipAng		    genotype***, plateDate(R2=1%)                                                   collide**, germAge***
#   avgAbsAng		genotype***, collide*** (R2=10%), plateDate(R2=2%)                              germAge***
#   wholeAngSign	genotype**, plateDate(R2=0%), date.gt.plate***                                  collide*, germAge***
#   quarterAngSign	genotype***, plateDate(R2=0%), date.gt.plate*                                   collide*, germAge***
#   tenthAngSign	genotype***, plateDate(R2=1%), date.gt.plate*                                   collide**, germAge***
#   tipAngSign		genotype***, plateDate(R2=2%), date.gt.plate*                                   collide**, germAge***
#   avgAngSign		plateDate(R2=0%), date.gt.plate***                                              collide*, genotype**, germAge***

# Meeting with Dan and Siobhan:
# 1. Ignore waviness and sign traits.  They are not of interest to us (and not very sig. for genotype).
# 2. Do not include date.gt and date.gt.plate as terms in the root length models.
# 3. Include collide as a term in the model, even if it doesn't account for much variation.  (w/o genotype nesting???)
# 4. Include germAge even if it doesn't account for much variation.
# 5. Switch the experiment term to plateDate.  (Done now in this file).  Include even if accounts for not much variation.
# 6. Include platingAge only in the length traits.
# 7. Use lme() instead of lmer(), as with previous paper, because getting p-values there was solved.
#
# This leaves these models:
#   trait           model formula
#   -----------     ------------------------------------------------------
#   straightLen		genotype + collide + (1|germAge) + (1|platingAge) + (1|plateDate)
#   rootLen		    genotype + collide + (1|germAge) + (1|platingAge) + (1|plateDate)
#   wholeAng		genotype + collide + (1|germAge) + (1|plateDate)
#   quarterAng		genotype + collide + (1|germAge) + (1|plateDate)
#   tenthAng		genotype + collide + (1|germAge) + (1|plateDate)
#   tipAng		    genotype + collide + (1|germAge) + (1|plateDate)
#   avgAbsAng		genotype + collide + (1|germAge) + (1|plateDate)
