################################################################################
# This file defines a vector of processed root data frame column names and their
# types, to be used when reading the data from a file, and it provides data
# frames containing trait information.
#
# This code is written in a way that makes them easy to tailor it for other
# projects.  This entire file can be copied and fairly simple changes made so
# that the file will serve for your project.
#
# Places to make changes are marked with XXXXX
################################################################################

#######################################################################################
# Names of parents and colors to use for them. XXXXX
#######################################################################################
parents = c("M82", "PENN")
parentColors = c(M82="red", PENN="springgreen4")
parentColorsLight = c(M82="red", PENN="springgreen3")
parentColorNames = c(M82="red", PENN="green")

#######################################################################################
# Names of columns in input data file, and their desired R type (i.e. class).
# NOTE: If the input file columns are changed, this must be changed! XXXXX
#######################################################################################
{
if (analysisVariant == "Orig")
    processedDataColClasses = c(
        filename="character", fileDate="character", plateDate="character", platingAge="integer",
        germAge="integer", rootAge="integer", measurer="character", replicate="character",
        genotype="character", platePos="character", plateLetter="character", seed="integer",
        germDay="integer", collide="logical", along="logical",
        comment="character", experiment="character", cM="numeric", scale="integer",
        plateRect="character", plateID="character", seedID="character",
        straightLen="numeric", rootLen="numeric", waviness="numeric", wholeAng="numeric",
        quarterAng="numeric", tenthAng="numeric", tipAng="numeric", avgAbsAng="numeric",
        wholeAngSign="integer", quarterAngSign="integer", tenthAngSign="integer",
        tipAngSign="integer", avgAngSign="integer", pts="character")
else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
    processedDataColClasses = c(
        filename="character", fileDate="character", plateDate="character", platingAge="integer",
        germAge="integer", rootAge="integer", measurer="character", replicate="character",
        genotype="character", platePos="character", plateLetter="character", seed="integer",
        germDay="integer", collide="logical", along="logical",
        comment="character", experiment="character", cM="numeric", scale="integer",
        plateRect="character", plateID="character", seedID="character",
        straightLen="numeric", rootLen="numeric", waviness="numeric", avgAbsAng="numeric",
        tipAng="numeric", first3rdAng="numeric", mid3rdAng="numeric", last3rdAng="numeric",
        avgAngSign="integer", tipAngSign="integer", first3rdAngSign="integer",
        mid3rdAngSign="integer", last3rdAngSign="integer", pts="character")
else
    stop("Bad analysisVariant value")
}

#######################################################################################
# Return a data frame with one row per trait, rownames are trait names, and these columns:
#   units: units of measurement of the trait
#   desc: description of the trait
#######################################################################################
getTraits = function()
    {
    # Trait names, units, and descriptions.  The trait names are equal to the names
    # of columns in the input data file containing the trait values. XXXXX
    if (analysisVariant == "Orig")
        mtx = matrix(ncol=3, byrow=TRUE, dimnames=list(NULL, c("trait", "units", "desc")), data=c(
        #   trait           units       description
            "straightLen",  "cm",       "Root straight-line length from top to tip",
            "rootLen",      "cm",       "Root length from top to tip",
            "waviness",     "",         "Measure of waviness, 0 is a straight line",
            "wholeAng",     "degrees",  "Angle of straight line from root top to tip",
            "quarterAng",   "degrees",  "Angle of straight line from last quarter to tip",
            "tenthAng",     "degrees",  "Angle of straight line from last tenth to tip",
            "tipAng",       "degrees",  "Angle of tip of root",
            "avgAbsAng",    "degrees",  "Average of absolute value of angle along entire root",
            "wholeAngSign", "-1=L,0=D,+1=R",  "wholeAng line is on left (-1), down(0), or right (+1)",
            "quarterAngSign","-1=L,0=D,+1=R", "quarterAng line is on left (-1), down(0), or right (+1)",
            "tenthAngSign", "-1=L,0=D,+1=R",  "tenthAng line is on left (-1), down(0), or right (+1)",
            "tipAngSign",   "-1=L,0=D,+1=R",  "Tip of root growing left (-1), down(0), or right (+1)",
            "avgAngSign",   "-1=L,0=D,+1=R",  "Average of root growth on left (-1), down(0), or right (+1)"))
    else if (analysisVariant == "thirdsAngs" || analysisVariant == "thirdsAngB")
        mtx = matrix(ncol=3, byrow=TRUE, dimnames=list(NULL, c("trait", "units", "desc")), data=c(
        #   trait           units       description
            "straightLen",  "cm",       "Root straight-line length from top to tip",
            "rootLen",      "cm",       "Root length from top to tip",
            "waviness",     "",         "Measure of waviness, 0 is a straight line",
            "tipAng",       "degrees",  "Angle of tip of root",
            "tipAngSign",   "-1=L,0=D,+1=R",  "Tip of root growing left (-1), down(0), or right (+1)",
            "avgAbsAng",    "degrees",  "Average of absolute value of angle along entire root",
            "avgAngSign",   "-1=L,0=D,+1=R",  "Average of root growth on left (-1), down(0), or right (+1)",
            "first3rdAng",  "degrees",  "Average of absolute value of angle along first third root",
            "first3rdAngSign","-1=L,0=D,+1=R","Average of first third of root growth on left (-1), down(0), or right (+1)",
            "mid3rdAng",    "degrees",  "Average of absolute value of angle along middle third root",
            "mid3rdAngSign","-1=L,0=D,+1=R","Average of middle third of root growth on left (-1), down(0), or right (+1)",
            "last3rdAng",   "degrees",  "Average of absolute value of angle along last third root",
            "last3rdAngSign","-1=L,0=D,+1=R","Average of last third of root growth on left (-1), down(0), or right (+1)"))
    else
        stop("Bad analysisVariant value")

    dfTraits = as.data.frame(mtx[,2:ncol(mtx)], row.names=mtx[,1], stringsAsFactors=FALSE)
    return(dfTraits)
    }

#######################################################################################
# Return a data frame of trait groups, with two columns: "group" and "trait".  Each
# trait is a member of one trait group of similar traits.
#######################################################################################
getTraitGroups = function()
    {
    # Define groups of similar or related traits.  These trait groups might be used to
    # group together genes D.E. in any IL sig. different than M82 for any trait in the
    # group, in order to analyze the genes for enrichments or coexpression that would
    # provide clues to genes driving the traits.

    # Trait groups are defined with a special formula.  The description of the formula
    # "language" is located in the comments for the function getILsInGroups() in file
    # Include_phenoAnalysis.R.

    # Although the formula language permits great expressiveness in defining groups,
    # each group should be defined according to some expected biological significance.

    # Group names MAY be the same as trait names.

    # Define groups by defining a vector whose element names are the group names, and
    # whose element values are the group formulas.

    if (analysisVariant == "Orig")
        traitGroups = c(
            avgAbsAng = "avgAbsAng",
            anglesCommon = "wholeAng & quarterAng & tenthAng & tipAng & avgAbsAng",
            avgAbsAngNotTipAng = "avgAbsAng & !tipAng",
            tipAngNotAvgAbsAng = "tipAng & !avgAbsAng",
            lengthsCommon = "straightLen & rootLen",
            NULL) # NULL doesn't get entered into the vector.
    else if (analysisVariant == "thirdsAngs")
        traitGroups = c(
            tipAng = "tipAng",
            avgAbsAng = "avgAbsAng",
            first3rdAngOnly = "first3rdAng & !mid3rdAng & !last3rdAng",
            mid3rdAngOnly = "mid3rdAng & !first3rdAng & !last3rdAng",
            last3rdAngOnly = "last3rdAng & !first3rdAng & !mid3rdAng",
            anglesCommon = "tipAng & avgAbsAng & first3rdAng & mid3rdAng & last3rdAng",
            avgAbsAngNotTipAng = "avgAbsAng & !tipAng",
            tipAngNotAvgAbsAng = "tipAng & !avgAbsAng",
            lengthsCommon = "straightLen & rootLen",
            NULL) # NULL doesn't get entered into the vector.
    else if (analysisVariant == "thirdsAngB")
        traitGroups = c(
            tipAng = "tipAng",
            avgAbsAng = "avgAbsAng",
            first3rdAngOnly = "first3rdAng & !mid3rdAng & !last3rdAng",
            mid3rdAngOnly = "mid3rdAng & !first3rdAng & !last3rdAng",
            last3rdAngOnly = "last3rdAng & !first3rdAng & !mid3rdAng",
            anglesCommon = "tipAng & avgAbsAng & first3rdAng & mid3rdAng & last3rdAng",
            avgAbsAngNotTipAng = "avgAbsAng & !tipAng",
            tipAngNotAvgAbsAng = "tipAng & !avgAbsAng",
            first3rdAngNotTipAng = "first3rdAng & !tipAng",
            tipAngNotFirst3rdAng = "tipAng & !first3rdAng",
            lengthsCommon = "straightLen & rootLen",
            NULL) # NULL doesn't get entered into the vector.
    else
        stop("Bad analysisVariant value")

    dfGroups = data.frame(group=names(traitGroups), formula=traitGroups, stringsAsFactors=FALSE)
    return(dfGroups)
    }

################################################################################
# End of file.
################################################################################
cat("Include_dataColumnTypes.R included\n")
