#######################################################################################
# This file contains general-purpose R definitions and functions.
#
# Revision history:
# Author    Date        Description
# --------  -------     ---------------------------------------------------------------
# Ted Toal  09Mar2014   Create file.
#######################################################################################

#######################################################################################
# General-purpose constants.
#######################################################################################
# Get file path separator character.
PATHSEP = substr(dir(pattern="^.*\\.*$", full.names=TRUE)[1], 2, 2)

# Degrees per radian.
degPerRad = 180/pi

# Centimeters per inch.
cmPerInch = 2.54

################################################################################
# dirSuffix
#
# Suffix for dataDir and analysisDir.
################################################################################
dirSuffix = analysisVariant

################################################################################
# dataDir
#
# Name of directory under the working directory where raw input file is located
# and where processed data output files are to be stored.
################################################################################
dataDir = paste("PhenotypeData_", fileDate, "_", dirSuffix, sep="")

################################################################################
# analysisDir
#
# Name of directory under the working directory where model analysis files are
# placed.
################################################################################
analysisDir = paste("Analysis_", fileDate, "_", dirSuffix, sep="")

################################################################################
# rawInputDir
#
# Name of directory under the working directory that contains the raw input data
# file(s).
################################################################################
rawInputDir = "ImagesAndRawData"

#######################################################################################
# File name prefixes.
#######################################################################################
# Filename prefix for data output files.
dataFilePrefix = "Results_"

# Pathname prefix for data output files, which includes the output directory and
# prefix portion of all raw and processed data output files.
dataPathPrefix = paste(dataDir, dataFilePrefix, sep=PATHSEP)

# Filename prefix for analysis output files, which includes the output directory
# and prefix portion of all analysis output files.
analysisOutputPrefix = paste(analysisDir, paste(fileDate, "_", sep=""), sep=PATHSEP)

#######################################################################################
# File names and paths.
#######################################################################################
# Data output files.
cleanedRawDataFile = "AllRawData.xlsx"
processedDataFile = "Processed.xlsx"

# Data output paths.
cleanedRawDataPath = paste(dataPathPrefix, cleanedRawDataFile, sep="")
processedDataPath = paste(dataPathPrefix, processedDataFile, sep="")

#######################################################################################
# Define ifelse() that works better.  The usual R ifelse() forces the yes and no values
# to be of the same "shape" as the test argument.  We make some reasonable assumptions
# here, although if the lengths of the args vary one to another, who knows what is right?
#######################################################################################
myIfElse = function(test, yes, no)
    {
    if (length(test) == 1)
        {
        if (test)
            return(yes);
        return(no);
        }
    if (length(test) == length(yes))
        result = yes
    else
        result = rep(yes, length.out=length(test))
    if (length(test) == length(no))
        result[!test] = no[!test]
    else
        result[!test] = rep(no, length.out=length(test))[!test]
    return(result)
    }

#######################################################################################
# Define a function to change a vector V of character strings to each have at least
# "N" characters by filling with leading character "fillCh" if they are shorter.
#######################################################################################
fillVec = function(V, N, fillCh)
    {
    V = as.character(V)
    Nch = nchar(V)
    while (any(Nch < N))
        {
        short = (Nch < N)
        V[short] = paste(fillCh, V[short], sep="")
        Nch = nchar(V)
        }
    return(V)
    }

#######################################################################################
# Capture and return the strings generated by the default print function for object x.
#
# Use this to print R objects that don't print correctly when you use "cat".  E.g. if
# cat(x, "\n") fails, use:   cat(getPrint(x), "\n")
#
# To include space before each line of the output, specify spaces for the prefix argument.
#
# To include a different prefix on the first line, specify it with "firstLinePfx".
#
# The current console width is used unless "width" is specified.
#
# If noLeadingIndexes is TRUE, then the vector indexes of the form "[1]" or [23]" are
# removed from the strings.  If noDoubleQuotes is TRUE, then " characters are removed
# from the strings.
#
# Normally you want to use the default print function for object x, but if you want to
# use a different function, supply it as argument "printFunc".
#######################################################################################
getPrint = function(x, prefix="", width=NULL, firstLinePfx=prefix,
    noLeadingIndexes=TRUE, noDoubleQuotes=TRUE, printFunc=print)
    {
    tempFile = tempfile("TempFile", fileext=".txt")
    sink(tempFile)
    if (is.null(width))
        width = getOption("width")
    saveWidth = options(width=width-nchar(prefix))
    printFunc(x)
    options(saveWidth)
    sink()
    x = readLines(tempFile)
    if (noLeadingIndexes)
        {
        x = sub("^ *\\[[0-9]+\\] ", "", x)
        x = gsub("\n *\\[[0-9]+\\] ", "", x)
        }
    if (noDoubleQuotes)
        x = gsub('"', "", x)
    x = sub(" $", "", x)
    x = paste(prefix, x, sep="", collapse="\n")
    if (firstLinePfx != prefix)
        x = sub(paste("^", prefix, sep=""), firstLinePfx, x)
    return(x)
    }

#######################################################################################
# Change colors to add alpha channel to them.  Alpha channel allows the color to be
# partially transparent.  When alpha=1, the color is solid; when 0, it is fully
# transparent; when 0.5, it is half-transparent.
# Arguments:
#   colors: vector of colors to which to add alpha channel.
#   alpha: vector of alpha channel values to use, recycled to length of "color" vector.
# Returns: vector of colors with alpha added.
#######################################################################################
addAlpha = function(colors, alpha)
    {
    alpha = rep(alpha, length.out=length(colors))
    col = col2rgb(colors)
    col = rgb(col["red",], col["green",], col["blue",], alpha*255, maxColorValue=255)
    names(col) = names(colors)
    return(col)
    }

#######################################################################################
# Call assign() using globalenv() for the envir argument, to assign "value" to a global
# variable with specified "name".
#
# When you assign a variable in a function, it disappears after you return from the
# function.  You may just want to return the variable from the function, e.g.
# "return(x)", or you may want it to be assigned globally so you can access it later
# from the R command line, by doing this:   assignGlobal("x", x)
#######################################################################################
assignGlobal = function(name, value)
    {
    assign(name, value, envir=globalenv())
    }

#######################################################################################
# Set the name of an output log file used by the printLog() function below.  If you
# don't set it, the default log file name is "defaultLogFile.txt".
#######################################################################################
outLogFile = "defaultLogFile.txt"
setDefaultLogFile = function(filename)
	{
	assignGlobal("outLogFile", filename)
	}

#######################################################################################
# Write the arguments to BOTH the console and the log file.  Arguments are like the
# cat() function arguments.  Use "file=filename" to specify a particular log file,
# otherwise the default log file set above is used.  You can also write the same output
# to multiple log files by using a vector of filenames for "file", and using NA in that
# vector causes output to the default log file set above.  To create a new log file and
# delete the old one, make the first call to this function use "append=FALSE".
# Note: the 'file' argument can be a vector of files, and if so, the output is sent to
# ALL files in the vector!  Also, output to the console can be suppressed by specifying
# toConsole=FALSE.  Also, if you set file="" (empty string), nothing is written to the
# log file.
#######################################################################################
printLog = function(..., sep=" ", append=TRUE, file=NA, toConsole=TRUE)
	{
	file[is.na(file)] = outLogFile
	if (toConsole)
	    {
	    cat(..., sep=sep)
	    flush.console()
	    }
	for (aFile in file)
	    if (aFile != "")
    	    cat(..., sep=sep, file=aFile, append=append)
	}

#######################################################################################
# Replace template strings with their actual data from data frame df.  A template
# string has within it substrings of the form %column_name%, and these are replaced
# by the contents of that df column.
# Returns a vector of length equal to number of df rows, each a templateString
# with %column_name% values replaced by corresponding row/column of df.
#######################################################################################
replaceTemplateString = function(templateString, df)
    {
    S = rep(templateString, nrow(df))
    for (col in colnames(df))
        {
        pctCol = paste("%", col, "%", sep="")
        if (grepl(pctCol, templateString))
            S = sapply(1:nrow(df), function(i) sub(pctCol, df[i, col], S[i], fixed=TRUE))
        }
    return(S)
    }

#######################################################################################
# Display an error message constructed from data in a rawData or dfRoots data frame, a
# row index vector Vidx, and a template string templateStr that identifies a data frame
# row by for example producing a seed ID string.
# Returns a string giving the file name and seed ID within that file, of the first
# indexed element.  The df argument can be either rawData or dfRoots.  The Vidx vector
# can be either numeric or logical indexes.
#######################################################################################
getFileSeedStr = function(df, Vidx, templateStr)
    {
    if (is.logical(Vidx))
        idx = which(Vidx)[1]
    else
        idx = Vidx[1]
    seedID = replaceTemplateString(templateStr, df[idx,])
    S = paste("file ", df$filename[idx], " ", seedID, sep="")
    return(S)
    }

#######################################################################################
# Find permutation order of objects to sort them in increasing order.  This function is
# like the function "order", and calls it with the ... arguments, returning its value.  
#
# Arguments:
#   ...: one or more vectors, all of same length, either character or convertible to
#       character, defining the desired sorting order.  Sorting occurs on the first
#       argument, with ties broken by the second argument, 2nd-level ties by third,
#       etc.  These arguments may be named, and must be named if the "lead0" argument
#       is used (argument names, not names of elements of the vectors).
#   lead0: vector of names of one or more of the ... arguments, which are to be
#       analyzed for strings of digits that are converted to longer strings by
#       adding leading 0's, so that all digit strings in that argument are the same
#       length.  If NULL, use first ... argument (use empty vector for none).
#   first: a single vector or list of vectors, each vector giving the values within
#       the ... argument vectors that are to appear FIRST in sorted order.  If a
#       vector is specified for "first", its elements apply to the first ... argument,
#       while if a list is specified, each of its vector elements must have a name
#       that matches the name of the ... argument to which the vector is to be applied.
#       The order of values in a vector determine their sorting order.
#   last: like "first", but this determines values in ... vectors that are to appear
#       LAST in order.
# Returns: The permutation order, the same as order() returns.
#
# This function differs from order() in that it does special sorting on digits,
# the motivation being to sort a list of IL name strings such that e.g. "IL10-1" sorts
# AFTER "IL9-1-5".  It adds leading zeroes to each string of 1 or more digits surrounded
# by non-digits so that all digit strings are the same length (the length of the longest
# digit string in any of the arguments), and sorting occurs on the modified values.
# Thus, this can be used to sort many types of data, not just a list of ILs.
# E.g. to sort the IL-rows of a data frame df by "IL" (with leading zeroes added), then
# columns "chr" then "start":
#   ord = order.special(rownames(df), df$chr, df$start)
#   df = df[ord,]
# As with order(), there can be more than one argument.  By default, only the first
# argument is modified to add leading zeroes to number strings.  The argument "lead0"
# can be specified to give the names of ... arguments to be modified with leading
# zeroes.
# E.g. to sort the rows of a data frame df by columns "trait" then "IL" (with leading
# zeroes added) then "pos":
#   ord = order.special(df$trait, ILs=df$IL, df$pos, lead0="ILs")
#   df = df[ord,]
# Use the "first" and "last" arguments to list particular values which should sort to
# the start or end.  These arguments each are either a list or vector.  A vector is
# applied to the first ... argument and gives the values which are to be first or last
# in the sorted order.  A list value for "first" or "last" has as its elements vectors
# whose names are ... argument names naming the arguments to which the vectors apply,
# and the vectors contain the values within those arguments to be first or last in
# sorted order.
# E.g. to sort the rows of a data frame df by IL (with leading zeroes added for sorting
# purposes and with "M82" first and "PENN" last) and then by chr then start:
#   ord = order.special(rownames(df), df$chr, df$start, first="M82", last="PENN")
#   df = df[ord,]
# or to sort by columns "trait" then "IL" (with leading zeroes added and both M82 and
# PENN appearing first) then "pos":
#   ord = order.special(df$trait, ILs=df$IL, df$pos, lead0="ILs", first=list(ILs=c("M82", "PENN")))
#   df = df[ord,]
#######################################################################################
order.special = function(..., lead0=NULL, first=NULL, last=NULL)
    {
    args = list(...)
    # Provide default for lead0.
    if (is.null(lead0))
        lead0 = 1
    # Make first and last be either NULL or lists.
    if (!is.null(first) && !is.list(first))
        {
        first = list(first)
        names(first) = names(args)[1]
        if (is.null(names(first)))
            names(first) = 1
        }
    if (!is.null(last) && !is.list(last))
        {
        last = list(last)
        names(last) = names(args)[1]
        if (is.null(names(last)))
            names(last) = 1
        }
    # Make sure first and last lists have names.
    if (is.list(first) && is.null(names(first))) stop("order.special error: expected 'first' arg to have names")
    if (is.list(last) && is.null(names(last))) stop("order.special error: expected 'last' arg to have names")

    # To sort first or last, we will modify any vector requiring such sorting so
    # that its elements start with either "A", "B", or "C" depending on whether
    # they are to sort first (A), middle (B), or last (C).  Furthermore, while
    # middle-sorting elements consist of "C" followed by the element value,
    # elements sorting first or last consist of "A" or "B" followed by a 9-digit
    # number that is the index of the element value in the "first" or "last"
    # vector, filled with leading 0's.  This ensures that these elements will
    # sort in the same order as they occur in "first" or "last".
    isModified = rep(FALSE, length(args)) # Track which args vectors are modified, for use when doing "last".
    names(isModified) = names(args)
    for (argName in names(first))
        {
        if (grepl("^[0-9]+$", argName))
            idx = as.integer(argName)
        else
            idx = which(argName == names(args))
        if (length(idx) != 1) stop("order.special error: expected 'first' arg list element name to match an argument name")
        V = as.character(args[[idx]])
        matches = match(V, first[[argName]])
        V[!is.na(matches)] = sprintf("A%09d", matches[!is.na(matches)])
        V[is.na(matches)] = paste("B", V[is.na(matches)], sep="")
        args[[idx]] = V
        isModified[idx] = TRUE
        }
    # Now do "last".
    for (argName in names(last))
        {
        idx = which(argName == names(args))
        if (length(idx) != 1) stop("order.special error: expected 'last' arg list element name to match an argument name")
        V = as.character(args[[idx]])
        # If we already modified this vector when doing "first" above, then we
        # won't find the "last" elements, but rather, those with "B" prepended.
        # (If "A" is prepended, the value appears in both "first" and "last"
        # vectors, let the "first" vector have precedence.)
        V.last = last[[argName]]
        if (isModified[idx])
            V.last = paste("B", V.last, sep="")
        matches = match(V, V.last)
        V[!is.na(matches)] = sprintf("C%09d", matches[!is.na(matches)])
        # If this args vector was already modified, we don't need to prepend "B".
        if (!isModified[idx])
            V[is.na(matches)] = paste("B", V[is.na(matches)], sep="")
        args[[idx]] = V
        }

    # For each argument name in lead0 (or index 1 if lead0 wasn't specified), add
    # leading zeroes to number strings in the argument.
    for (idx in lead0)
        {
        if (grepl("^[0-9]+$", idx))
            idx = as.integer(idx)
        V = args[[idx]]
        # Find length of longest digit string so we know the number of digits including
        # leading 0's that each digit substring will be set to.  Construct a regular expression and
        # its replacement, for adding leading zeroes.
        digitStrs = strsplit(as.character(args[[1]]), "[^0-9]+")
        NdigitsMax = max(sapply(digitStrs, function(V) max(c(0,nchar(V)))))
        if (NdigitsMax > 1)
            {
            RE = paste("(^|[^0-9])([0-9]{1,", NdigitsMax-1, "})([^0-9]|$)", sep="")
            repRE = "\\10\\2\\3"
            # Keep applying the regular expression until no further changes occur.
            while (TRUE)
                {
                Vnew = gsub(RE, repRE, V)
                if (all(Vnew == V, na.rm=TRUE))
                    break
                V = Vnew
                }
            # Replace the argument by the modified strings with leading zeroes.
            args[[idx]] = V
            }
        }
    # Now call order with the arguments, and return its return value.
    return(do.call(order, args))
    }

################################################################################
# Call order.special with arguments appropriate for sorting IL names.
################################################################################
sortILs = function(ILs)
    {
    return(order.special(ILs, first=c("M82", "PENN")))
    }

################################################################################
# R^2 values from linear models are useful because they give a measure of how
# well the model fits the data.  Mixed models have problems with R^2 and lme4
# does not provide R^2 values.  A simple method to compute a reasonable R^2
# from a mixed-model fit is given in:
#   Nakagawa, Shinichi, and Holger Schielzeth. "A general and simple method
#   for obtaining R2 from generalized linear mixed‐effects models." Methods
#   in Ecology and Evolution 4.2 (2013): 133-142.
# Two R^2 values are defined there, marginal and conditional.  The marginal
# R^2 is the R^2 due to fixed effects, while the conditional R^2 is that due
# to both fixed and random effects.
#       marginal R^2 = (variance of fixed-effect predictions for the data) /
#           (numerator + sum of random effects variances + residual variance)
#       conditional R^2 = (marginal R^2 numerator + sum of random effects variances) / (marginal R^2 denominator)
# By "fixed effect predictions applied to the data", I mean the sum of the fixed
# effect coefficients multiplied by their corresponding dependent variable value
# in the data.  For categorical dependent variables, the dependent variable value
# is 1, right?  E.g. if "genotype" is a categorical dependent variable, and a
# genotype value of "PENN" has coefficient value 48.9, then for each row of
# the data table whose genotype value is PENN, the fixed-effect prediction is
# 48.9 * 1 = 48.9.
# The definitions above provide an obvious extension to calculating an R^2 type
# of value to compare effect sizes of the different fixed and random effects.
# Let's call that R^2 a "fixed effect R^2" or "random effect R^2", and define
# these as:
#       fixed effect R^2 = (variance of that fixed-effect's prediction for the data) / (marginal R^2 denominator)
#       random effect R^2 = (variance of that random effect variable) / (marginal R^2 denominator)
# The residuals also have an obvious R^2 value:
#       residual R^2 = (variance of the residuals) / (marginal R^2 denominator)
# The code below computes all these R^2 values from an lmer() model fit.
# Arguments:
#   fm: lmer() model fit as returned by lmer().
#   df: the data frame of data to which the model was fit, preferably only with
#       columns that participate in fixed or random effects.  May be NULL if
#       the "data" attribute of fm contains the data frame.
# This assumes that nested effects were implemented explicitly as a df column
# whose values reflect the nesting, rather than via implicit nesting.
# Returns: Object fm with the following new attributes:
#   marginal.R2: the marginal R^2 value
#   conditional.R2: the conditional R^2 value
#   residual.R2: the residual R^2 value.
#   fixed.effect.R2: vector of fixed effect R^2 values, names are effect names, e.g. "age" or "age:collide".
#   random.effect.R2: vector of random effect R^2 values, names are effect names.
#   R2: named vector combining ALL of the above R2 values.  The names are:
#       Marginal
#       Conditional
#       Residual
#       fe:(fixed-effect name)
#       re:(random-effect name)
################################################################################

compute.lmer.R2 = function(fm, df=NULL)
    {
    # Make sure we have df.
    if (is.null(df))
        df = attr(fm, "data")
    if (is.null(df)) stop("compute.lmer.R2 requires a data frame")

    # Define a local var() function that returns 0 if all elements are identical.
    myvar = function(V) { if (all(V == V[1])) return(0); return(var(V)) }

    # Get fixed-effect coefficients with their names.  It is possible there are none.
    fe.coefs = fixef(fm)

    # Get rid of the intercept term if any, we don't need it.
    fe.coefs = fe.coefs[names(fe.coefs) != "(Intercept)"]

    # Extract the names of each fixed effect from the coefficient names, prefixing
    # each name with "fe:".  For example, if the coefficient name is genotypeIL8-3-1,
    # then the fixed effect name is "fe:genotype" (genotype = column name in df).
    # Or, if the coefficient name is genotypeIL8-3-1:collideTRUE:germAge, the
    # fixed effect name is fe:genotype:collide:germAge (all column names in df).
    # We will do this by removing everything following a df column name up to ":"
    # or end of string.
    if (is.null(fe.coefs) || length(fe.coefs) == 0)
        {
        unique.fe.names = c()
        fe.col.names = list()
        }
    else
        {
        tmp = names(fe.coefs)
        for (col in colnames(df))
            tmp = sub(paste("(", col, ")([^:]*)(:|$)", sep=""), "\\1\\3", tmp)
        unique.fe.names = paste("fe", unique(tmp), sep=":")
        # Break the fixed-effect names up into their df column components.
        fe.col.names = strsplit(sub("^fe:", "", unique.fe.names, fixed=FALSE), ":", fixed=TRUE)
        names(fe.col.names) = unique.fe.names
        }

    # Get random effect variances and residual variances.  There must be at least one.
    var.df = as.data.frame(VarCorr(fm))
    var.df = var.df[is.na(var.df$var2),] # If not NA, it is a correlation.
    resid.var = var.df$sdcor[var.df$grp == "Residual"]
    var.df = var.df[var.df$grp != "Residual",,drop=FALSE]
    re.var = var.df$sdcor

    # Construct names for random effects from the "grp" and "var1" columns of var.df.
    # Use ":" to separate group from var1, and prefix it with "re:", and assign
    # these as the names of re.var.  Don't use "(Intercept)" though in the name.
    re.names = paste("re", var.df$grp, var.df$var1, sep=":")
    re.names = sub(":(Intercept)", "", re.names, fixed=TRUE)
    names(re.var) = re.names

    # I believe lmer() removes rows containing NA for any of the fixed or random
    # effects.  Remove these rows from df.  First we must extract from re.names
    # and unique.fe.names the individual column names.
    eff.col.names = unlist(strsplit(sub("^fe:", "", unique.fe.names, fixed=FALSE), ":", fixed=TRUE))
    eff.col.names = c(eff.col.names, unlist(strsplit(sub("^re:", "", re.names), ":", fixed=TRUE)))
    eff.col.names = unique(eff.col.names)
    #eff.col.names
    # Make sure df actually has these column names.
    badCols = !eff.col.names %in% colnames(df)
    if (any(badCols)) stop("compute.lmer.R2: data frame does not have columns for effect terms: ",
        paste(eff.col.names[badCols], collapse=","))
    #dim(df)
    df = df[apply(df[, eff.col.names, drop=FALSE], 1, function(V) all(!is.na(V))),,drop=FALSE]
    #dim(df)

    # If there are no fixed effects, skip computing predictions.
    if (is.null(fe.coefs) || length(fe.coefs) == 0)
        {
        fe.var = c()
        fe.var.total = 0
        }
    else
        {
        # Initialize a vector to hold fixed-effect prediction variances, giving the
        # fixed-effect names to the elements.
        fe.var = rep(NA, length(unique.fe.names))
        names(fe.var) = unique.fe.names

        # Compute predictions for each fixed-effect term, save their variances for
        # computing fixed.effect.R2, and sum them to get total fixed-effect prediction.
        predictions.sum = rep(0, nrow(df))
        for (fe.name in unique.fe.names)
            {
            # Get the df columns for the components of this fixed effect.
            col.names = fe.col.names[[fe.name]]

            # Make a regular expression that will match all fe.coefs names of
            # coefficients for this fixed effect, capturing the column value.
            RE = paste(paste(col.names, collapse="([^:]*):"), "(.*)$", sep="")

            # Replacement expression to paste together the captured values.
            RepRE = paste("\\", 1:length(col.names), sep="", collapse=":")

            # Get coefficients for this fixed effect.  Use strsplit to split out
            # the values that are separated by ":"'s.  There will only be values
            # for categorical columns, not for numeric ones.
            which.coefs = grepl(RE, names(fe.coefs))
            the.coefs = fe.coefs[which.coefs]
            which.values = strsplit(sub(RE, RepRE, names(the.coefs)), ":", fixed=TRUE)

            # Compute the predictions.  The easiest way to do this is going to be
            # to loop for each coefficient, I'm afraid.
            df$predictions = 0 # Initialize predictions for this effect.
            for (i in 1:length(the.coefs))
                {
                # Make a logical vector for indexing df, to extract the relevant rows.
                # Only categorical columns are used to extract relevant rows.
                which.rows = rep(TRUE, nrow(df))
                k = 1
                for (col in col.names)
                    if (!is.numeric(df[,col]))
                        {
                        which.rows = which.rows & (df[,col] == which.values[[i]][k])
                        k = k + 1
                        }

                # Calculate the prediction.  The coefficient is multiplied by all
                # numeric columns, or 1 if none.
                df$predictions[which.rows] = the.coefs[i]
                for (col in col.names)
                    if (is.numeric(df[,col]))
                        df$predictions[which.rows] = df$predictions[which.rows] * df[which.rows, col]
                }

            # Compute the prediction variances and save in fe.var, and add them to
            # the cumulative sum.
            fe.var[fe.name] = myvar(df$predictions)
            predictions.sum = predictions.sum + df$predictions  
            }

        # Compute total fixed-effect variance.
        fe.var.total = myvar(predictions.sum)
        }

    # Compute sum of random-effect variances.
    re.var.sum = sum(re.var)

    # Sum all the variances to get the denominator term.
    var.sum = fe.var.total + re.var.sum + resid.var
    if (var.sum == 0) stop("Error in compute.lmer.R2, total variance is 0")

    # Compute marginal and conditional R^2.
    marginal.R2 = fe.var.total/var.sum
    conditional.R2 = (fe.var.total + re.var.sum)/var.sum

    # Compute fixed and random effect R^2 values and residual R^2.
    fixed.effect.R2 = fe.var/var.sum
    random.effect.R2 = re.var/var.sum
    residual.R2 = resid.var/var.sum

    # Add the R^2 values as attributes to fm and return it.
    attr(fm, "marginal.R2") = marginal.R2
    attr(fm, "conditional.R2") = conditional.R2
    attr(fm, "residual.R2") = residual.R2
    attr(fm, "fixed.effect.R2") = fixed.effect.R2
    attr(fm, "random.effect.R2") = random.effect.R2
    attr(fm, "R2") = c(Marginal=marginal.R2, Conditional=conditional.R2,
        Residual=residual.R2, fixed.effect.R2, random.effect.R2)
    return(fm)
    }

################################################################################
# The lme4 package uses objects such as "lmerMod" for linear model fitting.
# When such an object is printed with print(lmerModObj), the output contains
# all the fixed effect coefficients.  The following function reconstructs the
# same output without those coefficients, but adding output obtained by calling
# compute.lmer.R2 if the "R2" attribute is not already present in fm.
# Also, I cannot find the "Data:" name.
# Arguments:
#   fm: mixed linear model fit returned by lmer(), an lmerMod object.
#   df: data frame containing data to which the model was fit.  May be NULL if
#       the "data" attribute of fm contains the data frame.
# Returns: fm (invisibly), with attributes added by compute.lmer.R2.
################################################################################

print.lmerMod.abbrev = function(fm, df=NULL)
    {
    fm.sum = summary(fm)
    if (is.null(df))
        df = attr(fm, "data")
    if (is.null(df))
        stop("print.lmerMod.abbrev requires data frame")
    if (is.null(attr(fm, "R2")))
        fm = compute.lmer.R2(fm, df)
    cat(fm.sum$methTitle, "  ['", fm.sum$objClass[1], "']\n", sep="")
    cat("Formula: ", sub("  +", " ", Reduce(paste, deparse(formula(fm)))), "\n", sep="")
    cat(getPrint(fm.sum$AICtab), "\n", sep="")
    cat("Random effects:\n")
    cat(getPrint(VarCorr(fm)), "\n", sep="")
    ng = ngrps(fm)
    cat("Number of obs: ", nobs(fm), ", groups: ", paste(names(ng), ng, sep=": ", collapse=", "), "\n", sep="")
    cat("R^2 estimates: (Marginal=all fixed effects, Conditional=fixed and random effects)\n")
    cat(sprintf("  %-30s%-15s%s\n", "Name", "R^2 estimate", "Percent"))
    R2 = attr(fm, "R2")
    for (i in 1:length(R2))
        cat(sprintf("  %-30s%-15g%d %%\n", names(R2)[i], signif(R2[i], 5), round(100*R2[i])))
    return(invisible(fm))
    }

#######################################################################################
# End of file.
#######################################################################################
cat("Include_general.R included\n")
