#######################################################################################
# This file contains R definitions and functions for creating Excel spreadsheet files.
#
# Revision history:
# Author    Date        Description
# --------  -------     ---------------------------------------------------------------
# Ted Toal  28Dec2013   Create file.
#######################################################################################

# Load required packages.
library("xlsx")
library("gdata")

###########################################################################
# Write an Excel spreadsheet .xlsx file containing data from a data frame,
# placed onto one or more sheets.
# Arguments:
#   df: data frame containing data to write to the spreadsheet file.
#   xlsxFile: name of .xlsx file.
#   sheetCol: name of df column used to split the data frame into separate
#       sheets, or NULL to place the entire data frame onto one sheet.  This
#       column's unique contents appear as the sheet names and do not appear
#       as data within a sheet.
#   autosizeCols: TRUE to automatically size the columns, FALSE to not, or
#       a vector of names of df columns to auto-size.
#   freezeTopRow: TRUE to freeze top row of all sheets, FALSE to not.
#   colNames: TRUE to put column names at the top of each sheet, FALSE to not,
#       or a vector of names to put at top of each sheet.
#   boldCols: TRUE to use boldface for the column names at top of sheet,
#       FALSE to not, or a vector of df columns to be bolded.
#   halignDefault: default cell horizontal alignment, "LEFT", "RIGHT", "CENTER",
#       or NULL to accept default Excel alignment.
#   valignDefault: default cell vertical alignment, "TOP", "BOTTOM", "CENTER",
#       or NULL to accept default Excel alignment.
#   halignCols: vector of df column names to which to apply halignAlternate.
#   halignAlternate: alternate cell horizontal alignment, "LEFT", "RIGHT", "CENTER",
#       or NULL to accept default Excel alignment.
#   valignCols: vector of df column names to which to apply valignAlternate.
#   valignAlternate: alternate cell vertical alignment, "TOP", "BOTTOM", "CENTER",
#       or NULL to accept default Excel alignment.
#   bicolorCol: name of df column used to split the data frame into rows of
#       alternating cell colors, or NULL to use a single cell color for all data.
#       Whenever the value in this column changes from the previous row's value,
#       the background color is toggled.
#   rmvBicolorCol: TRUE to remove the data in bicolorCol so it doesn't appear
#       within the sheet, FALSE to leave the data so it appears in the sheet.
#   bkgdColor1: The background color of the sheets, NULL to use default background
#       background color.  If bicolorCol is not NULL, this is the first of two
#       such colors.
#   bkgdColor2: The alternate background color of the sheets, used when bicolorCol
#       is not NULL.
# Test with:
# df = data.frame(A=1:10, B=rnorm(10), C=letters[1:10], D=c(1,2,7,7,1,2,1,1,2,2), stringsAsFactors=FALSE)
# make.xlsx(df, "test1.xlsx")
# make.xlsx(df, "test2.xlsx", sheetCol="D")
# make.xlsx(df, "test3.xlsx", autosizeCols=FALSE)
# make.xlsx(df, "test4.xlsx", autosizeCols = c("A", "C"))
# make.xlsx(df, "test5.xlsx", freezeTopRow=TRUE)
# make.xlsx(df, "test6.xlsx", colNames=FALSE)
# make.xlsx(df, "test7.xlsx", colNames=c("FIRST", "2nd", "3rd", "fourth"))
# make.xlsx(df, "test8.xlsx", boldCols=FALSE)
# make.xlsx(df, "test9.xlsx", boldCols=c("B", "C"))
# make.xlsx(df, "test10.xlsx", halignDefault="CENTER")
# make.xlsx(df, "test11.xlsx", valignDefault="BOTTOM")
# make.xlsx(df, "test12.xlsx", halignCols="B", halignAlternate="CENTER")
# make.xlsx(df, "test13.xlsx", valignCols="B", valignAlternate="BOTTOM")
###########################################################################
make.xlsx = function(df, xlsxFile=NULL, sheetNames=NULL, sheetCol=NULL, sizeCols=0,
    freezeTopRow=TRUE, colNames=TRUE, boldCols=TRUE, halignDefault=NULL, valignDefault=NULL,
    halignCols=c(), halignAlternate=NULL, valignCols=c(), valignAlternate=NULL,
    borderCol="black", bicolorCols=NULL, rmvBicolorCols=FALSE,
    bkgdColors=myIfElse(is.null(bicolorCols), NULL, c("lavender","lightblue")), wb=NULL)
    {
    # Get the df column names (use numbers if none), and get the column names
    # that will appear in the spreadsheet (remove column sheetCol).
    df.colnames = colnames(df)
    if (is.null(df.colnames))
        df.colnames = 1:ncol(df)
    df.colnames_No_sheetCol = df.colnames
    if (!is.null(sheetCol))
        df.colnames_No_sheetCol = setdiff(df.colnames, sheetCol)
    # If sizeCols has no names, set the first name to "default".
    if (!is.null(sizeCols) && is.null(names(sizeCols)))
        names(sizeCols)[1] = "default"
    # Get the column names to be used in the first spreadsheet row, if any.
    if (is.logical(colNames))
        colNames = myIfElse(colNames, df.colnames_No_sheetCol, c())
    # Get the df column names to be boldfaced, if any.
    if (is.logical(boldCols))
        boldCols = myIfElse(boldCols, df.colnames_No_sheetCol, c())
    # Create the workbook if wb is NULL.
    if (is.null(wb))
        wb = createWorkbook()
    # Get sheet names.
    if (is.null(sheetNames))
        {
        if (is.null(sheetCol))
            sheetNames = "sheet1"
        else
            sheetNames = as.character(unique(df[, sheetCol]))
        }
    # Background fill color.
    if (is.null(bkgdColors))
        bicolorCols = NULL
    else
        {
        fill1 = Fill(foregroundColor=bkgdColors[1], backgroundColor=bkgdColors[1])
        if (length(bkgdColors) < 2)
            bicolorCols = NULL
        if (!is.null(bicolorCols))
            fill2 = Fill(foregroundColor=bkgdColors[2], backgroundColor=bkgdColors[2])
        }
    # Loop and create each sheet.
    for (sheetName in sheetNames)
        {
        # Get the data subset for the sheet.
        if (is.null(sheetCol))
            df2 = df
        else
            df2 = df[df[, sheetCol] == sheetName, df.colnames_No_sheetCol]
        # Create the sheet.
        sheet = createSheet(wb, sheetName=sheetName)
        # Create a block of cells, including an extra row for a header if requested.
        numHeaderRows = ifelse(length(colNames) > 0, 1, 0)
        cb = CellBlock(sheet, 1, 1, numHeaderRows+nrow(df2), ncol(df2))
        # Add the data column by column.
        for (i in 1:ncol(df2))
            {
            #cat("i =", i, colnames(df2)[i], "\n")
            col = df.colnames_No_sheetCol[i] # If df had no column names, this can be numbers.  df2 columns may have one number missing so numbered columns are renumbered in df2.
            data = df2[,i]
            # Determine the cell style for the column data.
            cs.data = CellStyle(wb)
            # Horizontal alignment.
            if (!is.null(halignAlternate) && (col %in% halignCols))
                cs.data = cs.data + Alignment(h=paste("ALIGN", halignAlternate, sep="_"))
            else if (!is.null(halignDefault))
                cs.data = cs.data + Alignment(h=paste("ALIGN", halignDefault, sep="_"))
            # Vertical alignment.
            if (!is.null(valignAlternate) && (col %in% valignCols))
                cs.data = cs.data + Alignment(v=paste("VERTICAL", valignAlternate, sep="_"))
            else if (!is.null(valignDefault))
                cs.data = cs.data + Alignment(v=paste("VERTICAL", valignDefault, sep="_"))
            # Cell borders.
            if (!is.null(borderCol))
                cs.data = cs.data + Border(color=borderCol,
                    position=c("TOP", "BOTTOM", "LEFT", "RIGHT"), pen="BORDER_THIN")
            # Set the header row cell if there is a header row.
            if (numHeaderRows > 0)
                {
                # Determine the cell style for the header row cell.
                cs.header = cs.data
                # Boldface.
                if (col %in% boldCols)
                    cs.header = cs.header + Font(wb, isBold=TRUE)
                # Set the header cell contents and style.
                CB.setColData(cb, colNames[i], i, rowOffset=0, colStyle=cs.header)
                }
            # Set the data cells for this column if there is any data.
            if (length(data) > 0)
                {
                cs.data = cs.data + DataFormat("text")
                # CB.setColData cannot handle logical data.
                if (class(data) == "logical")
                    data = as.character(data)
                CB.setColData(cb, data, i, rowOffset=numHeaderRows, colStyle=cs.data)
                }
            # Background fill color.
            if (!is.null(bkgdColors))
                {
                df2Rows = 1:nrow(df2)
                colRep = rep(i, nrow(df2))
                if (is.null(bicolorCols))
                    CB.setFill(cb, fill1, numHeaderRows + df2Rows, colRep)
                else
                    {
                    dfBicolor = df2[, bicolorCols[1]]
                    for (col in bicolorCols[-1])
                        dfBicolor = paste(dfBicolor, df2[, col], sep="_")
                    rowColor = c(0, cumsum(dfBicolor[-1] != dfBicolor[-nrow(df2)]) %% 2)
                    if (rmvBicolorCols)
                        df2 = df2[, !df.colnames_No_sheetCol %in% bicolorCols]
                    if (all(rowColor == 0))
                        CB.setFill(cb, fill1, numHeaderRows + df2Rows, colRep)
                    else if (all(rowColor == 1))
                        CB.setFill(cb, fill2, numHeaderRows + df2Rows, colRep)
                    else
                        {
                        CB.setFill(cb, fill1, numHeaderRows + df2Rows[rowColor == 0], rep(i, sum(rowColor == 0)))
                        CB.setFill(cb, fill2, numHeaderRows + df2Rows[rowColor == 1], rep(i, sum(rowColor == 1)))
                        }
                    }
                }
            }
        # Size columns as requested.
        defaultSize = -1 # -1 means no change to default column size.
        defaultCols = df.colnames_No_sheetCol # Columns to be set to default column size.
        if (!is.null(sizeCols))
            for (col in names(sizeCols))
                {
                if (col == "default")
                    defaultSize = sizeCols[col]
                else if (any(col == df.colnames_No_sheetCol))
                    {
                    defaultCols = setdiff(defaultCols, col)
                    if (sizeCols[col] == 0)
                        autoSizeColumn(sheet, which(col == df.colnames_No_sheetCol))
                    else
                        setColumnWidth(sheet, which(col == df.colnames_No_sheetCol), sizeCols[col])
                    }
                }
        if (defaultSize != -1)
            for (col in defaultCols)
                {
                if (defaultSize == 0)
                    autoSizeColumn(sheet, which(col == df.colnames_No_sheetCol))
                else
                    setColumnWidth(sheet, which(col == df.colnames_No_sheetCol), defaultSize)
                }
        # Freeze top row if requested.
        if (freezeTopRow)
            createFreezePane(sheet, 2, 1, 2, 0)
        }
    # Save the workbook to the output file if the filename isn't NULL.
    if (!is.null(xlsxFile))
        saveWorkbook(wb, xlsxFile)
    return(invisible(wb))
    }

#######################################################################################
# End of file.
#######################################################################################
cat("Include_xlsx.R included\n")
