#######################################################################################
# This file contains R definitions and functions for general purpose use.  It may be
# freely shared.
# Author: Ted Toal
# Lab: Brady
#######################################################################################

cat("Including Include_UtilityFunctions.R\n")

#######################################################################################
# Make sure Java memory is high.
#######################################################################################

options(java.parameters=c("-Djava.awt.headless=true", "-Xmx3500m"))

#######################################################################################
# Partial matching of "$" operator is SO dangerous, and you can't turn it off!!!
#######################################################################################

options(warnPartialMatchDollar=TRUE)

#######################################################################################
# Define some universal R constants.
#######################################################################################

# Rectangle, margin text, regular text, and axis positions.
BOTTOM = 1
LEFT = 2
TOP = 3
RIGHT = 4

# Point types.
pchSquare = 0
pchCircle = 1
pchTriangleOpen = 2
pchPlusSign = 3
pchCross = 4
pchDiamondOpen = 5
pchDelOpen = 6
pchSquareCross = 7
pchStar = 8
pchPlusDiamond = 9
pchPlusCircle = 10
pchJewishStar = 11
pchPlusSquare = 12
pchCrossCircleSquare = 13
pchDelSquare = 14
pchBox = 15
pchMediumDot = 16
pchTriangle = 17
pchDiamond = 18
pchBigDot = 19
pchDot = 20
pchBigDotBg = 21
pchBoxBg = 22
pchDiamondBg = 23
pchTriangleBg = 24
pchDelBg = 25

# Line types.
ltyNone = 0
ltySolid = 1
ltyDashed = 2
ltyDotted = 3
ltyDotDash = 4
ltyLongDash = 5
ltyTwoDash = 6

#######################################################################################
# Sample standard error of mean.
#######################################################################################
sem = function(V) sd(V)/sqrt(length(V))

#######################################################################################
# Capture and return the strings generated by the default print function for object x.
#
# Use this to print R objects that don't print correctly when you use "cat".  E.g. if
# cat(x, "\n") fails, use:   cat(getPrint(x), "\n")
#
# To include space before each line of the output, specify spaces for the prefix argument.
#
# To include a different prefix on the first line, specify it with "firstLinePfx".
#
# The current console width is used unless "width" is specified.
#
# If noLeadingIndexes is TRUE, then the vector indexes of the form "[1]" or [23]" are
# removed from the strings.  If noDoubleQuotes is TRUE, then " characters are removed
# from the strings.
#
# Normally you want to use the default print function for object x, but if you want to
# use a different function, supply it as argument "printFunc".
#######################################################################################
getPrint = function(x, prefix="", width=NULL, firstLinePfx=prefix,
    noLeadingIndexes=TRUE, noDoubleQuotes=TRUE, printFunc=print)
    {
    tempFile = tempfile("TempFile", fileext=".txt")
    sink(tempFile)
    if (is.null(width))
        width = getOption("width")
    saveWidth = options(width=width-nchar(prefix))
    printFunc(x)
    options(saveWidth)
    sink()
    x = readLines(tempFile)
    if (noLeadingIndexes)
        {
        x = sub("^ *\\[[0-9]+\\] ", "", x)
        x = gsub("\n *\\[[0-9]+\\] ", "", x)
        }
    if (noDoubleQuotes)
        x = gsub('"', "", x)
    x = sub(" $", "", x)
    x = paste(prefix, x, sep="", collapse="\n")
    if (firstLinePfx != prefix)
        x = sub(paste("^", prefix, sep=""), firstLinePfx, x)
    return(x)
    }

#######################################################################################
# Convert a number of bytes into a human-readable number with the specified number
# of significant digits and specified separator between number and units suffix.
#######################################################################################
numBytes.to.human = function(numBytes, digits=2, sep=" ")
    {
    if (numBytes < 1e3)
        suffix = "bytes"
    else if (numBytes < 1e6)
        {
        numBytes = numBytes / 1e3
        suffix = "KB"
        }
    else if (numBytes < 1e9)
        {
        numBytes = numBytes / 1e6
        suffix = "MB"
        }
    else if (numBytes < 1e12)
        {
        numBytes = numBytes / 1e9
        suffix = "GB"
        }
    else if (numBytes < 1e15)
        {
        numBytes = numBytes / 1e12
        suffix = "TB"
        }
    else if (numBytes < 1e18)
        {
        numBytes = numBytes / 1e15
        suffix = "PB"
        }
    else
        {
        numBytes = numBytes / 1e18
        suffix = "EB"
        }
    numBytes = signif(numBytes, digits=digits)
    return(paste(numBytes, suffix, sep=sep))
    }

#######################################################################################
# Get first few values of a vector as a string, with "..." if more than maxValues.
# Include names or indexes if withNames is TRUE.
#######################################################################################
getvals = function(v, maxValues=8, withNames=TRUE, nullValue="(NULL)")
    {
    if (is.null(v))
        return(nullValue)
    if (!is.vector(v))
        return("showv argument is not a vector")
    L = length(v)
    if (L == 0)
        return("(empty)")
    if (L > maxValues)
        v = v[1:maxValues]
    if (is.numeric(v))
        v = signif(v, 3)
    if (!withNames)
        v = as.character(v)
    else
        {
        indexes = paste("[", 1:length(v), "]", sep="")
        if (!is.null(names(v)))
            indexes[names(v) != ""] = names(v)[names(v) != ""]
        v = paste(indexes, "=", v, sep="")
        }
    v = paste(v, collapse=" ")
    if (L > maxValues)
        v = paste(v, "...")
    return(v)
    }

#######################################################################################
# Analyze argument(s) and compute and return object summary info.  An array of strings,
# one per argument object, is returned.
#######################################################################################
obsum = function(...)
    {
    # To add:
    #   matrix/data frame:
    #       show values if not too many, else just first few if too many
    #       show both row and col names
    # Better job with complex objects.
    argValues = list(...)
    L = substitute(list(...)) # L is a language object whose length is 1 more than # of args
    L = L[-1]
    result = character()
    if (length(L) > 0)
        for (i in 1:length(L))
            {
            s = deparse(L[[i]]) # deparse returns string form of argument i
            v = argValues[[i]]
            s1 = class(v)
            s2 = paste("Size=", numBytes.to.human(object.size(v), sep="_"), sep="")
            s3 = switch(s1,
                NULL="",
                list=paste("Len=", length(v), "  Vals: ", getvals(v), sep=""),
                numeric=paste("Len=", length(v), "  Vals: ", getvals(v), sep=""),
                integer=paste("Len=", length(v), "  Vals: ", getvals(v), sep=""),
                character=paste("Len=", length(v), "  Vals: ", getvals(v), sep=""),
                logical=paste("Len=", length(v), "  Vals: ", getvals(v), sep=""),
                matrix=paste("Dim=", paste(dim(v), collapse=","), "  Type=", mode(v), sep=""),
                data.frame=
                    {
                    st = paste("Dim=", paste(dim(v), collapse=","), sep="")
                    if (ncol(v) > 0)
                        {
                        colTypes = character()
                        for (i in 1:ncol(v))
                            colTypes = c(colTypes, mode(v[,i]))
                        names(colTypes) = paste("$", colnames(v), sep="")
                        st = paste(st, "  Types: ", getvals(colTypes), sep="")
                        }
                    st
                    },
                # Unknown type.
                    {
                    if (!is.null(names(v)))
                        {
                        nms = names(v)
                        st = paste("Unknown but has names:", getvals(nms), sep="")
                        }
                    else
                        st = "Not a class that an() can decipher!"
                    st
                    }
                 )
            s = paste(s, "=", s1, " ", s2, " ", s3, sep="")
            result = c(result, s)
            }
    return(result)
    }

#######################################################################################
# Call assign() using globalenv() for the envir argument, to assign "value" to a global
# variable with specified "name".
#
# When you assign a variable in a function, it disappears after you return from the
# function.  You may just want to return the variable from the function, e.g.
# "return(x)", or you may want it to be assigned globally so you can access it later
# from the R command line, by doing this:   assignGlobal("x", x)
#######################################################################################
assignGlobal = function(name, value)
    {
    assign(name, value, envir=globalenv())
    }

#######################################################################################
# Install a package if it isn't already installed.
#######################################################################################
installPackage = function(name, repos = getOption("repos"))
    {
    if (!(name %in% installed.packages()[,"Package"]))
        install.packages(name)
    }

#######################################################################################
# Add the specified library, and if it isn't installed, install it.
#######################################################################################
addThisLibrary = function(lib, repos = getOption("repos"))
    {
    cat("Getting R library", lib, "\n")
    if (!require(lib, character.only=TRUE))
        installPackage(lib, repos)
    }

#######################################################################################
# Get biocLite(), make sure it is updated.
#######################################################################################
get_biocLite = function()
    {
    if (!exists("biocLiteUpdated"))
        {
        if (exists("biocLite"))
            rm(biocLite, envir=globalenv())
        source("http://bioconductor.org/biocLite.R")
        assignGlobal("biocLiteUpdated", TRUE)
        }
    }

#######################################################################################
# Install a Bioconductor package if it isn't already installed.
#######################################################################################
installBioPackage = function(name)
    {
    if (!(name %in% installed.packages()[,"Package"]))
        {
        get_biocLite()
        biocLite(name)
        }
    }

#######################################################################################
# Add the specified Bioconductor library, and if it isn't installed, install it.
#######################################################################################
addThisBioLibrary = function(lib)
    {
    cat("Getting Bioconductor library", lib, "\n")
    if (!require(lib, character.only=TRUE))
        installBioPackage(lib)
    }

#######################################################################################
# Add common libraries we use.
#######################################################################################
addLibraries = function()
    {
    options(java.parameters=c("-Djava.awt.headless=true", "-Xmx2500m"))
    addThisLibrary("RCurl")
    addThisLibrary("rjson")
    addThisLibrary("graph")
    addThisLibrary("grid")
    addThisLibrary("hash")
    addThisLibrary("lattice")
    addThisLibrary("latticeExtra")
    addThisLibrary("cluster")
    addThisLibrary("multcomp")
    addThisLibrary("nlme")
    # Following must be commented out when using VarCorr in the nlme package (or use nlme::VarCorr)
    addThisLibrary("lme4") # lmer is here, which I use to fit random effect linear models for heritability
    addThisLibrary("ggm")
    addThisLibrary("sandwich")
    addThisLibrary("contrast")
    addThisLibrary("gdata")
    addThisLibrary("gmodels")
    addThisLibrary("gplots")
    addThisLibrary("ggplot2")
    addThisLibrary("plyr")
    addThisLibrary("dplyr")
    addThisLibrary("reshape")
    addThisLibrary("reshape2")
    addThisLibrary("kohonen")
    addThisLibrary("annotate")
    addThisLibrary("hwriter")
    addThisLibrary("plotrix")
    addThisLibrary("animation")
    addThisLibrary("VennDiagram")
    addThisLibrary("beanplot")
    addThisLibrary("XML")
    addThisLibrary("xlsx")
    addThisBioLibrary("Biobase")
    addThisBioLibrary("RColorBrewer")
    #addThisBioLibrary("Rgraphviz")
    #addThisBioLibrary("ShortRead")
    addThisBioLibrary("edgeR")
    addThisBioLibrary("eisa")
    addThisBioLibrary("hopach")
    addThisBioLibrary("seqinr")
    addThisBioLibrary("seqLogo")
    addThisBioLibrary("RBGL")
    addThisBioLibrary("GEOquery")
    addThisBioLibrary("limma")
    addThisBioLibrary("affy")
    addThisBioLibrary("affxparser")
    addThisBioLibrary("goseq")
    addThisBioLibrary("goTools")
    #addThisBioLibrary("GGtools") # Interferes with anova() used on lmer() object
    addThisBioLibrary("GO.db")
    addThisBioLibrary("biomaRt")
    addThisBioLibrary("Category")
    addThisBioLibrary("ath1121501.db")
    # addThisBioLibrary("Biostrings") # conflict with spacejam, function simplify()
    }

#######################################################################################
# Update Bioconductor packages.
#######################################################################################
updateBioPackages = function()
    {
    source("http://bioconductor.org/biocLite.R")
    biocLite("BiocUpgrade")
    }

#######################################################################################
# Test if a global variable exists.
#   globalVar: global variable name.
# Return TRUE iff the global variable exists.
#######################################################################################
globalVarExists = function(globalVar)
    {
    return(exists(globalVar))
    }

#######################################################################################
# Delete global variable.
#   globalVar: global variable name.
#######################################################################################
deleteGlobalVar = function(globalVar)
    {
    if (exists(globalVar))
        {
        # Sometimes there is an error message about the environment being locked.
        # The following line is an attempt to eliminate that error, but it doesn't work.
        #unlockBinding(globalVar, globalenv())
        rm(list=globalVar, inherits=TRUE)
        }
    }

#######################################################################################
# Save data in a global variable.
#   data: the data to be saved in the global variable.
#   globalVar: global variable name.
#######################################################################################
cacheData = function(data, globalVar)
    {
    assignGlobal(globalVar, data)
    }

#######################################################################################
# Retrieve the value of a global variable.
#   globalVar: global variable name.
# Return the data value that is stored in the global variable, or NULL if the global
# variable doesn't exist.
# Note: avoid using this function as it generally will return a copy of a very large
# object.  Instead, refer to the global variable directly by its name.  This is only
# used when the name is not known to the caller except at run time via a variable
# containing the name.
#######################################################################################
getCachedData = function(globalVar)
    {
    if (!exists(globalVar))
        return(NULL)
    return(get(globalVar))
    }

#######################################################################################
# Read a file into a data frame and then save the data frame in a global variable.
# If the global variable already exists, do nothing under the assumption that the
# file has already been read and saved in it.
#   filename: name of file to read.
#   globalVar: name of global variable in which to save the data frame.
#   rownameColumn: If not NA, this must be the name of a column to add as the first
#       column of the data frame, with values for the column taken from the row names
#       of the data frame (the ... argument would have argument row.names in it).
#   header: for read.table.
#   stringsAsFactors: for read.table.
#   ...: additional arguments for read.table()
#######################################################################################
readAndCacheFile = function(filename, globalVar, rownameColumn=NA, header=TRUE,
    stringsAsFactors=FALSE, ...)
    {
    if (!exists(globalVar))
        {
        df = read.table(filename, header=header, stringsAsFactors=stringsAsFactors, ...)
        if (!is.na(rownameColumn))
            {
            df = data.frame(xxx=rownames(df), df, stringsAsFactors=stringsAsFactors)
            colnames(df)[1] = rownameColumn
            }
        assignGlobal(globalVar, df)
        }
    }

#######################################################################################
# Return the current value of a specified global variable, or if it doesn't exist, call
# readAndCacheFile() with the specified arguments to read data from the file and store
# it in the global variable.
#   filename: name of file to read.
#   globalVar: name of the global variable in which to store the file data.
#   rownameColumn: see readAndCacheFile().
#   header: for read.table.
#   stringsAsFactors: for read.table.
#   ...: additional arguments for read.table()
# Return the global variable's value, which is the data read from the file.
# Note: avoid using this function as it generally will return a copy of a very large
# object.  Instead, refer to the global variable directly by its name.  This is only
# used when the name is not known to the caller except at run time via a variable
# containing the name.
#######################################################################################
readAndGetCachedData = function(filename, globalVar, rownameColumn=NA, header=TRUE,
    stringsAsFactors=FALSE, ...)
    {
    if (!exists(globalVar))
        readAndCacheFile(filename, globalVar, rownameColumn, header, stringsAsFactors, ...)
    return(get(globalVar))
    }

#######################################################################################
# Show any objects whose size is greater than N.  This is useful when you've made giant
# objects with R and wish to get rid of some unnecessary ones without closing R.
#######################################################################################
showBigObjs = function(N=10000000)
    {
    # This is not as easy as I thought.  We need to look in the local environments of
    # all stack frames on the calling stack, and look in all environments starting at
    # the global environment and moving up the parent list.
    sizes = c()
    frames = sys.frames()
    for (env in frames)
        {
        envSizes = sapply(ls(envir=env), function(x) object.size(get(x, envir=env)))
        envSizes = envSizes[envSizes >= N]
        sizes = c(sizes, envSizes)
        }
    env = environment()
    while (environmentName(env) != "R_EmptyEnv")
        {
        envSizes = sapply(ls(envir=env), function(x) object.size(get(x, envir=env)))
        envSizes = envSizes[envSizes >= N]
        sizes = c(sizes, envSizes)
        env = parent.env(env)
        }
    sizes = sort(unlist(sizes))
    print(as.data.frame(sizes))
    }

#######################################################################################
# Look for a command line argument of the form "name=", and return the value following
# "name=".  If there is no such argument, return defValue.  If asFUN is not null, it
# is assumed to be an "as" function (e.g. as.numeric) which is applied to the value to
# generate the return value (else the return type is character).
# Example: R is invoked with this command line:  R beta=6
# Then getCommandLineArg("beta", 10, as.numeric) returns 6.
#######################################################################################
getCommandLineArg = function(name, defValue = NULL, asFUN = NULL)
    {
    which = charmatch(name, commandArgs())
    if (any(is.na(which)))
        return(defValue)
    value = substring(commandArgs()[which], nchar(name)+2)
    if (!is.null(asFUN))
        {
        value = asFUN(value)
        if (any(is.na(value)))
            return(defValue)
        }
    return(value)
    }

#######################################################################################
# Process argument list to look for options.
# This can be used at the start of an R program that is invoked from the command line,
# to examine the arguments for options of the form "-optname value".
#
# Arguments:
#   args: vector of arguments to examine, could be obtained from commandArgs(TRUE).
#   options: list of available options, see Details.
#   initDefaults: TRUE to initialize option variables to default values, FALSE if not.
#   assignValues: TRUE to assign option variables to option value, FALSE if not.
#
# Returns: either a character vector whose value is an error string describing an
# error, or a list with these members:
#   found: a sublist whose member names are names of options that were found, and
#       whose values are the values found for those options.
#   args: args vector modified to remove all options.
#
# Details:
#
# The 'options' argument is a list describing all available options.  Each list
# element is a sublist of information for one option.  The name of each sublist
# is the option name, that is, the string that must occur preceded by "-" on the
# command line to specify the option.  The members of the sublist are:
#   var: the variable name to hold the option value.
#   type: the R type to which the user-specified option is to be converted, e.g.
#       "numeric", "character", "integer", "logical"
#   default: default value for the option.  If not specified, NULL is used.
# The sublist may also contain these optional members:
#   required: TRUE if the option is required, not optional.
#   minval: the minimum allowed value for a numeric option value.
#   maxval: the maximum allowed value for a numeric option value.
#   values: a vector of allowed values for the option.
#
# When "type" is "logical", the user may specify "TRUE", "FALSE", "T", "F", "1",
# or "0" as the value, or omit the value (in which case it is taken as TRUE).
#
# Example options definition and call:
# Options = list(
#     cmax=list(var="cmax", type="numeric", default=30.0, minval=2.0, maxval=100),
#     location=list(var="location", type="character", default="inner", values=c("inner", "outer")),
#     class=list(var="geneClass", type="character", default="TF"),
#     N=list(var="loopCount", type="integer", default=20, required=TRUE),
#     comp=list(var="compress", type="logical", default=FALSE)
#     )
# args = commandArgs(TRUE)
# ret = parseOptions(args, Options)
# if (is.character(ret)) stop(ret)
# args = ret$args
# if (length(args) != 1) stop("Usage: Rscript pgm [options] <filename>")
# cat("loopCount is: ", loopCount, "\n")
#######################################################################################
parseOptions = function(args, options, initDefaults=TRUE, assignValues=TRUE)
    {
    err = function(S) stop("parseOptions: ", S)

    if (!is.character(args)) err("Expected 'args' to be character")
    if (!is.list(options)) err("Expected 'options' to be a list")
    if (is.null(names(options))) err("Expected 'options' members to be named")
    sapply(options, function(V) if (!is.list(V)) err("Expected all 'options' members to be sublists"))
    sapply(options, function(L) if (is.null(L$var)) err("Expected all 'options' sublists to have a 'var' member"))
    sapply(options, function(L) if (is.null(L$type)) err("Expected all 'options' sublists to have a 'type' member"))

    # Assign default values initially.
    if (initDefaults)
        for (L in Options)
            assignGlobal(L$var, L$default)

    # n counts up as we skip non-option arguments.
    n = 1

    # Set up return list.
    ret = list(found=list())

    # Parse each argument.
    while (n <= length(args))
        {
        # If argument doesn't start with '-' it is not an option.
        if (!grepl("^-", args[n]))
            n = n + 1
        else
            {
            # Get the option name and remove it from args.
            opt = sub("^-", "", args[n])
            args = args[-n]

            # Look up the option name in 'options' list.
            idx = which(opt == names(options))
            if (length(idx) != 1)
                return(paste("Unknown option: ", args[n], sep=""))
            L = options[[idx]]

            # The option must have a value as the next argument, unless "logical".
            if (length(args) < n)
                {
                if (L$type == "logical")
                    value = TRUE
                else
                    return(paste("Missing value for option '", args[n], "'", sep=""))
                }
            else
                {
                value = as(args[n], L$type)
                args = args[-n]
                # Check the value to see if it is valid.
                if (is.na(value))
                    return(paste("Option '", args[n], "' value is not recognized as of type ", L$type, sep=""))
                if (!is.null(L$minval) && value < L$minval)
                    return(paste("Option '", args[n], "' value must be >= ", L$minval, " but is ", value, sep=""))
                if (!is.null(L$maxval) && value > L$maxval)
                    return(paste("Option '", args[n], "' value must be <= ", L$maxval, " but is ", value, sep=""))
                if (!is.null(L$values) && !value %in% L$values)
                    return(paste("Option '", args[n], "' has an invalid value: ", value, sep=""))
                }

            # Save the value in "found".
            ret$found[[opt]] = value

            # Assign the value to the variable.
            if (assignValues)
                assignGlobal(L$var, value)
            }
        }

    # Save the modified args vector.
    ret$args = args

    # Check to see if all required options were specified.
    notFound = setdiff(names(options), names(ret$found))
    required = sapply(notFound, function(opt) options[[opt]]$required)
    notSpecified = sapply(required, function(req) !is.null(req) && req)
    if (any(notSpecified))
        return(paste("Required option(s) not specified: ", paste(names(notSpecified)[notSpecified], collapse=","), sep=""))

    # Finished.
    return(ret)
    }

#######################################################################################
# Replace special substrings within a string with values given by a list of lists.
#
# Arguments:
#   SS: vector of one or more substrings to undergo replacement.
#   LL: list of lists, with sublists having names and elements in sublists also named.
#   left: character(s) marking start of a replacement field.
#   right: character(s) marking end of a replacement field.
#   sep: character occurring in SS between 'left' and 'right' that separates an LL
#       sublist name from a sublist element name.
#   name.subnameRE: R regular expression string matching anything that might occur
#       in SS between 'left' and 'right', including sep.
#
# Returns: SS with replacements made.
#
# Details:
#######################################################################################
replaceSubstrings = function(SS, LL, left="%%", right="", sep="$", name.subnameRE = "[A-Za-z0-9_.$]+")
    {
    RE = paste("^(.*)", left, "(", name.subnameRE, ")", right, "(.*)$", sep="")
    hasSubstring = grepl(RE, SS)
    while (any(hasSubstring))
        {
        ss.left = sub(RE, "\\1", SS[hasSubstring])
        ss.ctr = sub(RE, "\\2", SS[hasSubstring])
        ss.right = sub(RE, "\\3", SS[hasSubstring])
        ss.names = strsplit(ss.ctr, sep, fixed=TRUE)
        badLens = sapply(ss.names, function(V) length(V) != 2)
        if (any(badLens))
            stop("replaceSubstrings: expected two strings separated by ", sep,
                " but not found in ", paste(ss.ctr[badLens], collapse=" : "))
        names.1 = sapply(ss.names, '[', 1)
        names.2 = sapply(ss.names, '[', 2)
        sublists = LL[names.1]
        notFound = sapply(sublists, is.null)
        if (any(notFound))
            stop("replaceSubstrings: unknown sublist name(s): ", paste(names.1[notFound], collapse=" : "))
        subelementValues = sapply(1:length(names.1), function(i) sublists[[i]][names.2[i]])
        notFound = sapply(subelementValues, is.null)
        if (any(notFound))
            stop("replaceSubstrings: unknown sublist element name(s): ", paste(names.2[notFound], collapse=" : "))
        SS[hasSubstring] = paste(ss.left, subelementValues, ss.right, sep="")
        hasSubstring = grepl(RE, SS)
        }
    return(SS)
    }

#######################################################################################
# Use this to measure how long it takes R to do something, in seconds, see example below.
#
# Get the current proc.time() and subtract a previous proc.time() from it, saving both
# the current and delta CPU time (sum of user and system time) in a list.  Accumulate
# delta time in a specified time slot.  Optionally convert the cumulative values in the
# delta time slots to units of minutes.
#
# Arguments:
#   L: A list to which elements are added if not present.  The list is returned, and
#       must be specified as the L argument in the next call to this function.  On the
#       first call this argument may be left off to default to an empty list.
#   i: Index (or name) of a time slot into which to add the delta CPU time for this call.
#       Each call to this function can specify a different index, see example below.
#       Use NULL to not update any cumulative time slot.
#   cvt: If TRUE, the return list contains an updated cumDeltaCPUdifftime member.
#
# Returns: list with these members:
#       last: the proc.time() value found this call to the function.
#       delta: the difference in total CPU time (user+system) between the current
#           time and the previous time this function was called with the same list.
#           When this function is called with an empty list as in the default case
#           normally used on the first call, delta will be 0.
#       cum: a vector of cumulative delta times, indexed by index values provided by
#           argument i.
#       cvt: cum converted to a difftime object, if cvt is not FALSE.
#
# Example:
# L = getDeltaCpuTime()
# for (k in bigVector)
#   {
#   doSomething()
#   L = getDeltaCpuTime(L, "doSomething")
#   doSomethingElse()
#   L = getDeltaCpuTime(L, "doSomethingElse")
#   }
# L = getDeltaCpuTime(L, cvt=TRUE)
# for (n in names(L$cvt))
#   cat("Time for", n, ":", L$cvt[n], "secs\n")
#######################################################################################
getDeltaCpuTime = function(L=list(), i=NULL, cvt=FALSE)
    {
    cur = proc.time()
    if (!is.list(L))
        L = list()
    if (is.null(L$last) || any(is.na(L$last)))
        L$last = cur
    if (is.null(L$cum) || any(is.na(L$cum)))
        L$cum = numeric()
    t = cur - L$last
    L$last = cur
    L$delta = as.numeric(t[1]) + as.numeric(t[2]) # User + system CPU time
    if (!is.null(i))
        {
        if (is.null(L$cum[i]) || any(is.na(L$cum[i])))
            L$cum[i] = L$delta
        else
            L$cum[i] = L$cum[i] + L$delta
        }
    if (!cvt)
        {
        L$cvt = c()
        for (idx in 1:length(L$cum))
            L$cvt[idx] = as.difftime(L$cum[idx], units="secs")
        names(L$cvt) = names(L$cum)
        }
    return(L)
    }

#######################################################################################
# Get the current proc.time() and subtract a previous proc.time() from it.  Convert
# the result to a CPU processor time as a difftime in units of minutes and return it.
# Use this to measure how long it takes R to do something, like this:
#  prevTime = proc.time()
#  (now do some time-consuming stuff)
#  minutes = getDiffCpuTime(prevTime)
#  cat("It took", minutes, "minutes to do it.\n")
#######################################################################################
getDiffCpuTime = function(prevTime)
    {
    curTime = proc.time()
    t = curTime - prevTime
    cpu = as.numeric(t[1]) + as.numeric(t[2]) # User + system CPU time, don't use elapsed time!
    t = as.difftime(cpu, units="secs")
    units(t) = "mins"
    return(t)
    }

#######################################################################################
# Allow "y" and "yes" and "t" and "true" for TRUE.  Allow "NA" and "N/A" and "" for NA.
# This is written to work properly when o is a vector.
# This works in parallel with asLogical, returning TRUE when asLogical recognizes a
# valid true or false value, NA when asLogical recognizes a valid NA value, and FALSE
# when asLogical does not recognize the value and so returns its default value of NA.
#######################################################################################
isLogical = function(o)
    {
    if (is.vector(o) && length(o) > 1) return(sapply(o, isLogical, USE.NAMES=FALSE))
    if (is.logical(o)) return(o != "X") # Returns TRUE or NA
    if (!is.character(o) && as.logical(o)) return(TRUE)
    s = tolower(as.character(o))
    if (s == "y" || s == "yes" || s == "t" || s == "true") return(TRUE)
    if (s == "n" || s == "no" || s == "f" || s == "false") return(TRUE)
    if (s == "na" | s == "n/a" | s == "") return(NA)
    return(FALSE)
    }

#######################################################################################
# Allow "y" and "yes" and "t" and "true" for TRUE.  Allow "NA" and "N/A" and "" for NA.
# This is written to work properly when o is a vector.
#######################################################################################
asLogical = function(o)
    {
    if (is.vector(o) && length(o) > 1) return(sapply(o, asLogical, USE.NAMES=FALSE))
    if (is.logical(o)) return(o)
    if (!is.character(o) && as.logical(o)) return(TRUE)
    s = tolower(as.character(o))
    if (s == "y" || s == "yes" || s == "t" || s == "true") return(TRUE)
    if (s == "n" || s == "no" || s == "f" || s == "false") return(FALSE)
    if (s == "na" | s == "n/a" | s == "") return(NA)
    return(NA)
    }

#######################################################################################
# Define ifelse() that works better.  The usual R ifelse() forces the yes and no values
# to be of the same "shape" as the test argument.  We make some reasonable assumptions
# here, although if the lengths of the args vary one to another, who knows what is right?
# Also, we provide an optional fourth argument which is assigned as the result when the
# test value is NA.
#######################################################################################
myIfElse = function(test, yes, no, na=NA)
    {
    # First handle case where test is an array (matrix or higher dimension).
    if (is.array(test))
        {
        if (any(is.null(yes)) || any(is.null(no)))
            stop("myIfElse requires non-NULL for arrays")
        if (!is.array(yes))
            yes = array(rep(yes, length.out=length(test)), dim=dim(test))
        else if (length(dim(test)) != length(dim(yes)) || !all(dim(test) == dim(yes)))
            stop("myIfElse requires test and yes array args to be of same dimension")
        if (!is.array(no))
            no = array(rep(no, length.out=length(test)), dim=dim(test))
        else if (length(dim(test)) != length(dim(no)) || !all(dim(test) == dim(no)))
            stop("myIfElse requires test and no array args to be of same dimension")
        if (!is.array(na))
            na = array(rep(na, length.out=length(test)), dim=dim(test))
        else if (length(dim(test)) != length(dim(na)) || !all(dim(test) == dim(na)))
            stop("myIfElse requires test and na array args to be of same dimension")
        result = yes
        result[!is.na(test) & !test] = no[!is.na(test) & !test]
        result[is.na(test)] = na[is.na(test)]
        return(result)
        }
    # Next handle case where test is a simple scalar.
    if (length(test) == 1)
        {
        if (is.na(test))
            return(na)
        if (test)
            return(yes);
        return(no);
        }
    # Assume test is a vector.
    if (any(is.null(yes)) || any(is.null(no)))
        stop("myIfElse requires non-NULL for vectors")
    if (length(test) != length(yes))
        yes = rep(yes, length.out=length(test))
    if (length(test) != length(no))
        no = rep(no, length.out=length(test))
    if (length(test) != length(na))
        na = rep(na, length.out=length(test))
    result = yes
    result[!is.na(test) & !test] = no[!is.na(test) & !test]
    result[is.na(test)] = na[is.na(test)]
    return(result)
    }

################################################################################
# Conservative 8-color palette adapted for color blindness, with first color = "black".
#
# Wong, Bang. "Points of view: Color blindness." nature methods 8.6 (2011): 441-441.
################################################################################
colorBlind.8 = c(
    black="#000000",
    bluegreen="#009E73",
    blue="#0072B2",
    vermillion="#D55E00",
    reddishpurple="#CC79A7",
    orange="#E69F00",
    skyblue="#56B4E9",
    yellow="#F0E442")

# plot(NA, xlim=0:1, ylim=0:1, axes=FALSE, xlab="", ylab="")
# rect((0:7)/8, 0, (1:8)/8, 1, border=NA, col=colorBlind.8)

################################################################################
# I wanted a few more colors and tried to find three more that for me were different.
################################################################################
colorBlind.11 = c(colorBlind.8, olivegreen="#607030", plum="#C0A0D0", salmon="#E0A070")

#######################################################################################
# Like which.min() and which.max() except the NAME of the first maximum index is returned.
#######################################################################################
which.name.min = function(v)
    {
    return(names(v)[which.min(v)])
    }

which.name.max = function(v)
    {
    return(names(v)[which.max(v)])
    }

#######################################################################################
# Return the greatest common divisor of the integers in vector V.
#######################################################################################
GCD = function(V)
    {
    if (!is.numeric(V)) stop("GCD: non-numeric argument")
    V = V[!is.na(V)]
    if (length(V) < 2) stop("GCD: need at least two non-NA values")
    IV = as.integer(V)
    if (any(IV != V)) stop("GCD: some arguments are not integers")
    if (any(IV < 1)) stop("GCD: some arguments are < 1")

    # Handle length(IV) > 2
    N = length(IV)
    while (N > 2)
        {
        IV = c(IV[1:(N-2)], GCD.fast(IV[(N-1):N]))
        N = N - 1
        }

    return(GCD.fast(IV))
    }

# Compute gcd of IV[1] and IV[2], no error checking.
GCD.fast = function(IV)
    {
    while (IV[1] > 0)
        IV = c(IV[2] %% IV[1], IV[1]);
    return(IV[2]);
    }

#######################################################################################
# Return the least common multiple of the integers in vector V.
#######################################################################################
LCM = function(V)
    {
    if (!is.numeric(V)) stop("LCM: non-numeric argument")
    V = V[!is.na(V)]
    if (length(V) < 2) stop("LCM: need at least two non-NA values")
    IV = as.integer(V)
    if (any(IV != V)) stop("LCM: some arguments are not integers")
    if (any(IV < 1)) stop("LCM: some arguments are < 1")

    # Handle length(IV) > 2
    if (length(IV) > 2)
        IV = c(IV[1], LCM.fast(IV[-1]))

    # Compute lcm of IV[1] and IV[2]
    return(LCM.fast(IV))
    }

# Compute lcm of IV[1] and IV[2], no error checking.
LCM.fast = function(IV)
    {
    return(IV[1]*IV[2]/GCD(IV));
    }

#######################################################################################
# Add a vector n of names to a vector v and return the result.
#######################################################################################
addNamesToVector = function(v, n)
    {
    names(v) = n
    return(v)
    }

#######################################################################################
# Create a vector whose names are given by vector n and whose values are equal to v,
# which is recycled to length(n).
#######################################################################################
makeNamedVector = function(n, v=0)
    {
    V = rep(v, length.out=length(n))
    names(V) = n
    return(V)
    }

#######################################################################################
# Extract a column of a data frame as a vector with names equal to the data frame row names.
#######################################################################################
getDataframeColumnAsNamedVector = function(df, col=1)
    {
    v = df[,col]
    names(v) = rownames(df)
    return(v)
    }
df.col.as.vector = function(df, col=1) getDataframeColumnAsNamedVector(df, col)

#######################################################################################
# Extract a row of a data frame as a vector with names equal to the data frame column names.
#######################################################################################
getDataframeRowAsNamedVector = function(df, row=1)
    {
    v = as.vector(as.matrix(df[row,]))
    names(v) = colnames(df)
    return(v)
    }
df.row.as.vector = function(df, row=1) getDataframeRowAsNamedVector(df, row)

#######################################################################################
# Prepend object o to the start of vector v and return the resulting vector.
#######################################################################################
prepend = function(v, o)
    {
    return(append(v, o, after=0))
    }

#######################################################################################
# Insert one or more columns into data frame "df, preserving its row names.
#
# Arguments:
#   df: a data frame, possibly with 0 rows, or NULL.
#   newColumns: several possibilities:
#       - a data frame: all of its columns are inserted into df.  If df is not
#           NULL and doesn't have 0 rows, then newColumns must have the same number
#           of rows as df, or it can have 0 rows, in which case its columns are
#           added with NA values in the rows.  The column names of newColumns, if
#           any, become the names of the new columns of df if colNames argument is
#           NULL.  The row names of NewColumns, if any, become the new row names
#           of df if rowNames argument is an integer index of this newColumns
#           element.
#       - a scalar (vector of length 1): this value is inserted into every row of a
#           new column of df.  If df is NULL or has 0 rows, the new data frame has
#           one row with one column added with the scalar as its contents.  The
#           colNames argument should be specified to provide a name for the new
#           column.  The names of the vector, if any, become the names of the new
#           rows of df if rowNames argument is an integer index of this newColumns
#           element
#       - a vector (of length > 1): this is inserted as a new column into df.  If
#           df is not NULL and doesn't have 0 rows, then the length of newColumns
#           must equal the number of rows of df.  If df is NULL or has 0 rows, the
#           new data frame has one column added with the vector as its contents.
#           The colNames argument should be specified to provide a name for the new
#           column.  The names of the vector, if any, become the names of the new rows
#           of df if rowNames argument is an integer index of this newColumns element.
#       - a list: each element of the list must be either a data frame or vector and
#           is handled as described above for the other two newColumns possibilities.
#           If the list has names, those names become the names of new columns of df
#           if colNames argument is NULL and the named argument is a vector.
#       - NULL: this is similar to a value of NA (vector of length 1 containing NA),
#           except that if df is NULL or has 0 rows, the returned data frame will have
#           0 rows and will have a new column whose name is determined by the colNames
#           argument.  Use this to add a column to a data frame without adding any data
#           yet.
#   colNames: a vector of names for the new columns that are added to df, used in the
#       order the columns appear in the newColumns argument.  If NULL, column names
#       are taken from the newColumns argument when possible.
#   rowNames: several possibilities:
#       - a vector of names for the rows of df, of the same length as the number of
#           rows in df, or if df is NULL or has 0 rows, of the same length as the new
#           data provided in the newColumns argument.
#       - an integer.  In this case, the integer identifies which of the newColumns
#           arguments should be used to supply row names.  If newColumns is not a list,
#           the integer would be 1, but if it is a list, the integer indexes the list.
#       - NA: Row names of the returned data frame are set to NULL.
#       - NULL: The existing row names of df are not changed.
#   beforeCol: the column name or number before which to insert the new column(s).
#       If NULL, the argument is ignored.
#   afterCol: the column name or number after which to insert the new column(s), if
#       beforeCol is NULL.  If both this and beforeCol are NULL, the new columns are
#       added after the existing columns of df.
#   stringsAsFactors: passed along to any function used that has this argument.
#
# Returns: the new data frame with new columns added.
#
# Examples:
#   1. To add to df two new columns named "clusters" and "degree" containing NA,
#       after existing column "ILs":
#          df = insertDfColumns(df, newColumns=list(clusters=NA, degree=NA), afterCol="ILs")
#######################################################################################
insertDfColumns = function(df, newColumns=NA, colNames=NULL, rowNames=NULL,
    beforeCol=NULL, afterCol=NULL, stringsAsFactors=FALSE)
    {
    # Make sure df is a data frame or NULL.
    if (!is.null(df) && !is.data.frame(df)) stop("insertDfColumns: df argument is not a data frame or NULL")

    # If newColumns is not a list, make it a list with one element.
    if (!is.list(newColumns))
        newColumns = list(newColumns)

    # Record the initial number of columns in df.  New columns are initially added
    # after these, and will be moved to the correct position later.  If df is NULL,
    # use 0.
    NcolsInitial = ncol(df)
    if (is.null(df))
        NcolsInitial = 0

    # Process each element of the newColumns list.
    for (i in 1:length(newColumns))
        {
        newColName = names(newColumns)[i]
        newCol = newColumns[[i]]

        # Record the current number of columns in df, before columns are added.
        NcolsBeforeAdd = ncol(df)
        if (is.null(df))
            NcolsBeforeAdd = 0

        # Case 1: newColumns is a data frame.
        if (is.data.frame(newCol))
            {
            if (is.null(df))
                df = newCol
            else if (nrow(df) == 0 || nrow(newCol) == 0 || nrow(df) == nrow(newCol))
                {
                if (nrow(df) == 0 && nrow(newCol) > 0)
                    df[1:nrow(newCol),] = NA
                else if (nrow(df) > 0 && nrow(newCol) == 0)
                    newCol[1:nrow(df),] = NA
                df = cbind(df, newCol, stringsAsFactors=stringsAsFactors)
                }
            else stop("insertDfColumns: nrow(newColumns)=", nrow(newCol), " not equal to nrow(df)=", nrow(df))
            }

        # Case 2/3: newColumns is a vector, of length 1 or > 1.
        else if (is.vector(newCol))
            {
            if (length(newCol) == 0) stop("insertDfColumns: newColumns argument is vector of length 0")
            if (is.null(df))
                df = data.frame(newCol, stringsAsFactors=stringsAsFactors)
            else if (nrow(df) == 0 || nrow(df) == length(newCol))
                {
                if (nrow(df) == 0)
                    df[1:length(newCol),] = NA
                df = cbind(df, newCol, stringsAsFactors=stringsAsFactors)
                }
            else stop("insertDfColumns: length(newColumns)=", length(newCol), " not equal to nrow(df)=", nrow(df))

            # Add new column name = newColName if it isn't NULL and if colNames is NULL.
            if (is.null(colNames) && !is.null(newColName))
                colnames(df)[ncol(df)] = newColName
            }

        # Case 4: newColumns in NULL.
        else if (is.null(newCol))
            {
            newCol = as.data.frame(matrix(NA, nrow=0, ncol=1), stringsAsFactors=stringsAsFactors)
            if (is.null(df))
                df = newCol
            else if (nrow(df) == 0)
                df = cbind(df, newCol, stringsAsFactors=stringsAsFactors)
            else
                df = cbind(df, NA, stringsAsFactors=stringsAsFactors)
            }

        # Else newColumns is invalid.
        else stop("insertDfColumns: invalid newColumns argument class: ", class(newCol))

        # Do some sanity checks.
        if (is.null(df)) stop("insertDfColumns: programming error, unexpected NULL df")
        NcolsAfterAdd = ncol(df)
        numNewCols = NcolsAfterAdd - NcolsBeforeAdd
        if (numNewCols < 1) stop("insertDfColumns: programming error, expected new columns added")

        # Add column names to the new columns from colNames if it isn't NULL.
        if (!is.null(colNames))
            {
            if (!is.character(colNames)) stop("insertDfColumns: colNames argument is not character class")
            if (length(colNames) < numNewCols) stop("insertDfColumns: colNames argument length is too small")
            newColNames = colNames[1:numNewCols]
            colNames = colNames[-(1:numNewCols)]
            colnames(df)[(NcolsBeforeAdd+1):ncol(df)] = newColNames
            }
        }

    # Add row names to df from rowNames if it isn't NULL.
    if (is.character(rowNames))
        {
        if (length(rowNames) != nrow(df))
            stop("insertDfColumns: rowNames argument length=", length(rowNames), " not equal to nrow(df)=", nrow(df))
        rownames(df) = rowNames
        }
    else if (is.numeric(rowNames) && length(rowNames) == 1)
        {
        idx = as.integer(rowNames)
        if (idx < 1 || idx > length(newColumns))
            stop("insertDfColumns: invalid value for numeric rowNames argument: ", rowNames)
        newCol = newColumns[[idx]]
        newRowNames = c()
        if (is.data.frame(newCol))
            newRowNames = rownames(newCol)
        else if (is.vector(newCol))
            newRowNames = names(newCol)
        if (length(newRowNames) != nrow(df))
            stop("insertDfColumns: rowNames argument indexes nColumns argument with ",
                length(newRowNames), " row names not equal to nrow(df)=", nrow(df))
        rownames(df) = newRowNames
        }
    else if (length(rowNames) == 1 && is.na(rowNames))
        rownames(df) = NULL
    else if (!is.null(rowNames))
        stop("insertDfColumns: invalid rowNames argument class: ", class(rowNames), " or length: ", length(rowNames))

    # Move the new columns to the specified position.
    moveBefore = NULL # Numeric column index.  NULL means to not move the columns, leave them at the end of df.
    if (!is.null(beforeCol))
        {
        if (!is.numeric(beforeCol))
            idx = which(beforeCol == colnames(df))[1]
        if (is.numeric(idx) && length(idx) == 1 && !is.na(idx))
            moveBefore = as.integer(idx)
        else
            stop("insertDfColumns: invalid beforeCol argument: ", beforeCol, " No such column.")
        }
    else if (!is.null(afterCol))
        {
        if (!is.numeric(afterCol))
            idx = which(afterCol == colnames(df))[1]
        if (is.numeric(idx) && length(idx) == 1 && !is.na(idx))
            moveBefore = as.integer(idx) + 1
        else
            stop("insertDfColumns: invalid afterCol argument: ", afterCol, " No such column.")
        }
    if (!is.null(moveBefore) && (moveBefore > NcolsInitial))
            moveBefore = NULL
    if (!is.null(moveBefore))
        {
        NcolsFinal = ncol(df)
        NnewCols = NcolsFinal - NcolsInitial
        if (NnewCols < 1)
            stop("insertDfColumns: programming error, expected some new columns")
        oldColIdxs = 1:NcolsInitial
        newColIdxs = (NcolsInitial+1):NcolsFinal
        if (moveBefore <= 1)
            df = df[, c(newColIdxs, oldColIdxs)]
        else
            df = df[, c(1:(moveBefore-1), newColIdxs, moveBefore:NcolsInitial)]
        }
    return(df)
    }

#######################################################################################
# Append columns of data to data frame "df, preserving its row names.  Careful: this
# needs to work even if the data frame has no rows!!!!
#######################################################################################
appendColumns = function(df, columns)
    {
    return(insertDfColumns(df, columns))
    }

#######################################################################################
# Append rows of data to data frame "df, preserving its column names.  Careful: this
# needs to work even if the data frame has no columns!!!!
#######################################################################################
appendRows = function(df, rows)
    {
    if (ncol(df) == 0)
        {
        df = as.data.frame(matrix(NA, nrow=nrow(df)+1, ncol=0,
            dimnames=list(c(rownames(df), rownames(rows)), c())))
        }
    else
        {
        if (is.vector(rows))
            rows = as.matrix(rows, nrow=1)
        colnames(rows) = colnames(df)
        if (nrow(df) == 0)
            df = as.data.frame(rows, stringsAsFactors=FALSE)
        else
            df = rbind(df, rows)
        }
    return(df)
    }

#######################################################################################
# Move specified columns of matrix or data frame to specified new position.
#
# Arguments:
#   x: the data frame or matrix.
#   cols: names or numbers of columns to move, in the order that they are
#       to appear after being moved.
#   afterCol: name or number of column to move the columns after.
#   beforeCol: name or number of column to move the columns before.
#       (Either afterCol or beforeCol must be NULL and the other non-NULL).
#
# Returns: modified data frame or matrix.
#######################################################################################
moveColumns = function(x, cols, afterCol=NULL, beforeCol=NULL)
    {
    # Check cols argument.
    if (!is.numeric(cols))
        {
        missingCols = cols[!cols %in% colnames(x)]
        if (length(missingCols) > 0)
            stop("moveColumns: No such columns: ", paste(missingCols, collapse=" "))
        cols = match(cols, colnames(x))
        }

    # Check afterCol and beforeCol arguments.
    if (is.null(afterCol) && is.null(beforeCol))
        stop("moveColumns: Both afterCol and beforeCol are NULL")
    if (!is.null(afterCol) && !is.null(beforeCol))
        stop("moveColumns: Neither afterCol nor beforeCol is NULL")
    before = FALSE
    col = afterCol
    if (!is.null(beforeCol))
        {
        before = TRUE
        col = beforeCol
        }
    if (length(col) != 1)
        stop("moveColumns: expected exactly one column for afterCol/beforeCol, got: ", length(col))
    if (!is.numeric(col))
        {
        if (!any(colnames(x) == col))
            stop("moveColumns: No such column: ", col)
        col = which(colnames(x) == col)[1]
        }
    if (before)
        col = col-1

    # Now do the work.
    x.before = NULL
    if (col > 0)
        x.before = x[, setdiff(1:col, cols), drop=FALSE]
    x.after = NULL
    if (col < ncol(x))
        x.after = x[, setdiff((col+1):ncol(x), cols), drop=FALSE]
    x = x[, cols, drop=FALSE]
    # Why doesn't cbind accept NULL args like rbind?
    if (is.null(x.before) && is.null(x.after)) return(x)
    if (is.null(x.before) && !is.null(x.after)) return(cbind(x, x.after))
    if (!is.null(x.before) && is.null(x.after)) return(cbind(x.before, x))
    return(cbind(x.before, x, x.after))
    }

#######################################################################################
# Remove columns specified by vector cols from matrix or data frame mdf and return result.
#######################################################################################
removeColumns = function(mdf, cols)
    {
    # Do this in a way that allows cols to be numbers or column names.
    nCol = ncol(mdf)
    deleteCol = rep(FALSE, nCol)
    names(deleteCol) = colnames(mdf)
    deleteCol[cols] = TRUE
    for (i in seq(nCol,1))
        if (deleteCol[i])
            mdf = mdf[,-i, drop=FALSE]
    return(mdf)
    }

#######################################################################################
# Remove rows specified by vector rows from matrix or data frame mdf and return result.
#######################################################################################
removeRows = function(mdf, rows)
    {
    # Do this in a way that allows rows to be numbers or row names.
    nRow = nrow(mdf)
    deleteRow = rep(FALSE, nRow)
    names(deleteRow) = rownames(mdf)
    deleteRow[rows] = TRUE
    for (i in seq(nRow,1))
        if (deleteRow[i])
            mdf = mdf[-i, , drop=FALSE]
    return(mdf)
    }

###########################################################################
# Perform rbind on two data frames after first checking to make sure they
# have the same column names.  NULL values are simply passed to rbind.
###########################################################################
safe.rbind = function(df1, df2)
    {
    if (is.null(df1) || is.null(df2) || nrow(df1) == 0 || nrow(df2) == 0)
        return(rbind(df1, df2))
    if (ncol(df1) != ncol(df2) || any(colnames(df1) != colnames(df2)))
        stop("columns differ:\n", getPrint(colnames(df1), "   "), "\nand:\n",
            getPrint(colnames(df2), "   "), "\n")
    return(rbind(df1, df2))
    }

###########################################################################
# Perform cbind on two or more vectors or data frames.  If any are NULL or
# of length 0, they are skipped. (cbind gives an error in that case).  If
# you want one or more arguments to cause a return of NULL if the argument
# is NULL or of length 0, name the argument with a name that starts with
# "NULLif0".
###########################################################################
smart.cbind = function(...)
    {
    args = list(...)

    # Collect all non-empty args in L.  The names of empty or NULL arguments
    # are checked to see if they start with "NULLif0", and if so, NULL is
    # returned.
    L = list()
    while (length(args) > 0)
        {
        arg = args[[1]]
        argName = names(args)[1]
        args = args[-1]
        if (is.null(arg) ||
            (is.data.frame(arg) && nrow(arg) == 0) ||
            (!is.data.frame(arg) && length(arg) == 0))
            {
            if (length(argName) == 1 && grepl("^NULLif0", argName))
                return(NULL)
            next
            }
        L = c(L, list(arg))
        }
    if (length(L) == 0)
        return(NULL)

    # Do cbind on L.
    return(do.call(cbind, L))
    }

################################################################################
# An alternative to rbind() that is far, far faster.
#
# Arguments:
#   df1: First data frame to be bound together row-wise, or a list previously
#       returned by this function, or NULL to indicate an empty data frame.
#   df2: Second data frame to be bound together row-wise, or a list previously
#       returned by this function, or NULL to indicate an empty data frame.
#
# Returns: a list containing the data in df1 and df2.  To convert this list to
#   a data frame, use rbind.fast.finish().  The returned list has an attribute
#   named "nrow", containing the number of rows in total.  Do not add to or
#   subtract from the returned lists yourself.
#
# Note: When df1 or df2 is a data frame, if the first row name is not composed
# entirely of digits, the row names are saved in sublist element ".rownames",
# and are also accumulated when each new data frame is added.  Both data frames
# must have row names if they are present at all, and the row names must all be
# unique.  No checking is done here for that.
#
# Example of speed:
#
# N = 10000
# Na = 1:N
# Nb = (N+1):(2*N)
#
# f1 = function()
#   {
#   df1 = NULL
#   for (i in 1:100)
#       df1 = rbind(df1, data.frame(a=Na, b=Nb, row.names=paste("A", i, "_", Na, sep="")))
#   return(df1)
#   }
#
# f2 = function()
#   {
#   df2 = NULL
#   for (i in 1:100)
#       df2 = rbind.fast(df2, data.frame(a=Na, b=Nb, row.names=paste("A", i, "_", Na, sep="")))
#   df2 = rbind.fast.finish(df2)
#   return(df2)
#   }
#
# system.time(df1 <- f1())
#    user  system elapsed
#   7.901   3.395  11.460
# system.time(df2 <- f2())
#    user  system elapsed
#   0.056   0.004   0.070
# identical(df1, df2)
# [1] TRUE
################################################################################
rbind.fast = function(df1, df2)
    {
    if (!(is.null(df1) || is.data.frame(df1) || (is.list(df1) && is.list(df1[[1]]))) ||
        !(is.null(df2) || is.data.frame(df2) || (is.list(df2) && is.list(df2[[1]]))))
        stop("rbind.fast only works with NULL or data frame or list of lists arguments")

    # Convert df1 from data frame to list if not already done.  Each list element
    # is itself a list.
    if (is.data.frame(df1))
        {
        nr = nrow(df1)
        R = rownames(df1)
        df1 = as.list(df1)
        for (n in names(df1))
            df1[[n]] = list(df1[[n]])
        if (length(R) > 0 && !grepl("^[0-9]+$", R[1]))
            df1$.rownames = R
        attr(df1, "nrow") = nr
        }

    # Convert df2 from data frame to list if not already done.  Each list element
    # is itself a list.
    if (is.data.frame(df2))
        {
        nr = nrow(df2)
        R = rownames(df2)
        df2 = as.list(df2)
        for (n in names(df2))
            df2[[n]] = list(df2[[n]])
        if (length(R) > 0 && !grepl("^[0-9]+$", R[1]))
            df2$.rownames = R
        attr(df2, "nrow") = nr
        }

    # Handle NULL.
    if (is.null(df1))
        return(df2)
    if (is.null(df2))
        return(df1)

    # rbind df2 to df1 by extending each sublist of df1 by the corresponding
    # sublist of df2.
    if (any(names(df1) == ".rownames") != any(names(df2) == ".rownames"))
        stop("rbind.fast: one but not both data frames have row names")
    for (n in names(df2)) # Use df2 in case df1 is NULL
        df1[[n]] = c(df1[[n]], df2[[n]])
    attr(df1, "nrow") = attr(df1, "nrow") + attr(df2, "nrow")
    return(df1)
    }

################################################################################
# Convert the list returned by rbind.fast back into the appropriate data frame.
#
# Arguments:
#   df: List returned by rbind.fast(), or a data frame.
#   stringsAsFactors: passed to as.data.frame().
#
# Returns: a data frame corresponding to df.  If df is already a data frame, it
# is returned, else the list is collapsed back into a data frame.
#
# Note: if df is a list and has element ".rownames", these are made into row
# names of the returned data frame.
#
# Note: see rbind.fast().
################################################################################
rbind.fast.finish = function(df, stringsAsFactors=FALSE, verbose=FALSE)
    {
    if (is.list(df))
        {
        R = NULL
        if (any(names(df) == ".rownames"))
            {
            R = df$.rownames
            df$.rownames = NULL
            }
        for (n in names(df))
            {
            if (verbose)
                cat("Doing column '", n, "'\n", sep="")
            df[[n]] = unlist(df[[n]], use.names=FALSE)
            }
        df = as.data.frame(df, row.names=R, stringsAsFactors=FALSE)
        }
    return(df)
    }

################################################################################
# Does do.call(rbind, <data frame list>) using rbind.fast to vastly speed it up.
#
# Arguments:
#   L: List of data frames to be rbound.
#   stringsAsFactors: passed to as.data.frame().
#
# Returns: a data frame containing all data frames in list L, bound together
# one below the other, i.e. the number of columns stays constant.
################################################################################
do.call.rbind.fast = function(L, stringsAsFactors=FALSE)
    {
    dfAll = NULL
    for (df in L)
        dfAll = rbind.fast(dfAll, df)
    return(rbind.fast.finish(dfAll, stringsAsFactors))
    }

########################################
# If possible, shorten a vector S of strings so that none is longer than N characters,
# by removing terminal string characters until either all strings are N or fewer
# characters or until removal of a character would make two strings identical.
# Shortened names have * placed at their end, implying that at least two characters
# were removed.
########################################

shortenStrings = function(S, N)
    {
    if (length(S) == 1)
        return(S)
    origLens = nchar(S)
    origNmax = max(origLens)
    # See how short we can make the longest string and still have all strings be
    # unique.  However, no need to go shorter than Nmax-1.  We need to go to that
    # rather than just to Nmax, because all shortened strings will have "*" added
    # to the end.
    S2 = S
    Nmax = origNmax
    while (!any(duplicated(S2)) && Nmax > N-2)
        {
        Nmax = Nmax - 1
        S2 = substr(S2, 1, Nmax)
        }
    # Nmax is equal to the length at which duplicated strings appear.  That means
    # means that there were no duplicated strings at Nmax+1 (unless there were
    # some in the original list).  Truncate at Nmax+1 and add * to truncated
    # strings IF Nmax+2 is < origNmax.  The truncated strings will have length
    # Nmax+2.  To ensure that we try to get this to be <= N, we make the above
    # loop continue (if possible) until Nmax == N-2.
    if (Nmax+2 < origNmax)
        {
        S = substr(S, 1, Nmax+1)
        shortened = (nchar(S) < origLens)
        S[shortened] = paste(S[shortened], "*", sep="")
        }
    return(S)
    }

#######################################################################################
# For each row of matrix or data frame or vector mdfv, find the row of matrix or data
# frame or vector mdf2 that has a column keyCol2 entry that matches the column keyCol
# entry of that row of mdfv.  When a vector is given, keyCol or keyCol2 must be 1.
# Return an integer vector of length equal to the number of rows in mdfv, which has the
# row number in mdfv2 where a matching keyCol/keyCol2 was found, or NA if not found.
#######################################################################################
whichRowsInOtherDatasetHaveKeyMember = function(mdfv, keyCol, mdfv2, keyCol2)
    {
    mdfvWasVector = is.vector(mdfv)
    if (mdfvWasVector)
        {
        mdfvNames = names(mdfv)
        mdfv = matrix(mdfv, ncol=1)
        }
    if (is.vector(mdfv2))
        mdfv2 = matrix(mdfv2, ncol=1)
    return(match(mdfv[,keyCol], mdfv2[,keyCol2]))
    }

#######################################################################################
# Append object o to the end of list l and return the resulting list.
# Note: R can't seem to have an array containing a single element that is an empty array,
# but it has no problem with a list containing a single element that is an empty array.
# To get around this, use list() rather than c().
#######################################################################################
listAppend = function(l, o)
    {
    l[[length(l)+1]] = o;
    return(l)
    }

#######################################################################################
# Prepend object o to the start of list l and return the resulting list.
#######################################################################################
listPrepend = function(l, o)
    {
    l = listAppend(rev(l), o)
    return(rev(l))
    }

#######################################################################################
# Given a data frame df with a character column x containing strings separated by the
# string sep, expand the data frame with additional rows so that column x contains
# single strings not joined by sep.  I.e. if a row contains the string "A;B;C" in
# column x and sep is ";", the new data frame will have 3 rows in place of that one
# row, all three being identical except that column x contains either "A", "B", or "C",
# one row for each.
# Tested with:
#   df = data.frame(x=c("WOX", "PAN", "WOX;CLV", "REV;ABC;XYZ", "CLV;PAN"), y=1:5, stringsAsFactors=FALSE)
#   expandDfStrings(df, 1)
#######################################################################################
expandDfStrings = function(df, x, sep=";")
    {
    a = sapply(df[,x], function(s) strsplit(s, sep, fixed=TRUE), USE.NAMES=FALSE)
    n = sapply(a, length)
    if (length(n) == 0)
        return(NULL)
    sq = unlist(sapply(1:length(n), function(i) rep(i, n[i])))
    result = df[sq,]
    rownames(result) = NULL
    result[, x] = unlist(a)
    return(result)
    }

#######################################################################################
# Given a data frame df with a character column x containing strings separated by the
# string sep.in and another column y that can be used as a factor, compress the data
# frame by combining each set of rows having the same value in column y into a single
# row whose values are taken from the first row of the set, except that column x is the
# concatenation of all unique strings from column x of the set (separated by sep.out).
# I.e. if for two rows with the same value in column y, column x has the strings
# "A;B;C" and "C;D" and sep.in is ";", the new data frame will have a single row in
# place of those two, identical to the first of those rows except that column x is
# "A;B;C;D" (sep.out also ";").  If sep.in is "", column x is assumed to contain one
# string rather than multiple concatenated strings.
# Tested with:
#   df = data.frame(x=c("WOX", "PAN", "WOX;CLV", "REV;ABC;XYZ", "CLV;PAN"), y=c(1,2,1,2,1), stringsAsFactors=FALSE)
#   compressDfStrings(df, 1, 2)
#######################################################################################
compressDfStrings = function(df, x, y, sep.in=";", sep.out=";")
    {
    if (sep.in != "")
        df = expandDfStrings(df, x, sep.in)
    if (nrow(df) == 0)
        return(NULL)
    L = tapply(1:nrow(df), df[,y], function(ii)
        {
        d = df[ii[1],]
        d[,x] = paste(unique(df[ii,x]), collapse=sep.out)
        return(d)
        })
    result = do.call(rbind, L)
    return(result)
    }

#######################################################################################
# Given a data frame df with two character columns x and y, make a new data frame with
# two character columns v and w, with column v giving all the unique strings in column
# x, and column w being the concatenation (with a separator string) of all the strings
# in column y whose corresponding column x is the column v value.  Both columns x and
# y may contain concatenations of strings, and these are split before applying the
# algorithm.
# Arguments:
#   df: the data frame containing the data to work with.
#   x: first column of df to work with.
#   y: second column of df to work with.
#   v: name of first new data frame column.
#   w: name of second new data frame column.
#   x.sep: separator string to split column x, or empty string for no splitting.
#   y.sep: separator string to split column y, or empty string for no splitting.
#   result.sep: separator string for joining multiple strings in column w.
#######################################################################################
findPartnersOfUniqueStrings = function(df, x, y, v, w, x.sep="", y.sep="", result.sep=";")
    {
    if (x.sep != "")
        df = expandDfStrings(df, x, x.sep)
    if (y.sep != "")
        df = expandDfStrings(df, y, y.sep)
    uniqueStrings = unique(df[,x])
    # If no duplicates/splits either way, it is simple:
    # result = names(x)
    # names(result) = x
    # return(result)
    # Handling duplicates in x requires concatenating the corresponding names(x):
    # a = tapply(names(x), x, function(nms) paste(nms, collapse=result.sep))
    # result = as.vector(a)
    # names(result) = names(a)
    # But we also must handle splitting of x and this is much more complicated:
    if (x.sep != "")
        x = strsplit(x, x.sep, fixed=TRUE)
    a = sapply(names(x), function(nm) sapply(x[[nm]], function(L, nm2) return(list(nm2, L)), nm2=nm))
    a = as.data.frame(matrix(unlist(a, use.names=FALSE), ncol=2, byrow=TRUE), stringsAsFactors=FALSE)
    a = tapply(a[,1], a[,2], function(nms) paste(nms, collapse=result.sep))
    result = as.vector(a)
    names(result) = names(a)
    return(result)
    }

#######################################################################################
# Randomize each column of a data frame.  Return the randomized data frame.
#######################################################################################
randomizeByColumn = function(df)
    {
    return(apply(df, 2, sample))
    }

#######################################################################################
# Find permutation order of objects to sort them in increasing order.  This function is
# like the function "order", and calls it with the ... arguments, returning its value.
# This function differs from order() in that it does special sorting on digits, the
# motivation being to sort a list of strings such that e.g. "chr10" sorts AFTER "chr9".
#
# Arguments:
#   ...: one or more vectors, all of same length, either character or convertible to
#       character, defining the desired sorting order.  Sorting occurs on the first
#       argument, with ties broken by the second argument, 2nd-level ties by third,
#       etc.  These arguments may be named, and must be named if the "lead0" argument
#       is used (argument names, not names of elements of the vectors).
#   lead0: vector of names of one or more of the ... arguments, which are to be
#       analyzed for strings of digits that are converted to longer strings by
#       adding leading 0's, so that all digit strings in that argument are the same
#       length.  If NULL, use first ... argument (use empty vector for none).
#   first: a single vector or list of vectors, each vector giving the values within
#       the ... argument vectors that are to appear FIRST in sorted order.  If a
#       vector is specified for "first", its elements apply to the first ... argument,
#       while if a list is specified, each of its vector elements must have a name
#       that matches the name of the ... argument to which the vector is to be applied.
#       The order of values in a vector determine their sorting order.
#   last: like "first", but this determines values in ... vectors that are to appear
#       LAST in order.
# Returns: The permutation order, the same as order() returns.
#
# This function differs from order() in that it does special sorting on digits,
# the motivation being to sort a list of IL name strings such that e.g. "IL10-1" sorts
# AFTER "IL9-1-5".  It adds leading zeroes to each string of 1 or more digits surrounded
# by non-digits so that all digit strings are the same length (the length of the longest
# digit string in any of the arguments), and sorting occurs on the modified values.
# Thus, this can be used to sort many types of data, not just a list of ILs.
# E.g. to sort the IL-rows of a data frame df by "IL" (with leading zeroes added), then
# columns "chr" then "start":
#   ord = order.special(rownames(df), df$chr, df$start)
#   df = df[ord,]
# As with order(), there can be more than one argument.  By default, only the first
# argument is modified to add leading zeroes to number strings.  The argument "lead0"
# can be specified to give the names of ... arguments to be modified with leading
# zeroes.
# E.g. to sort the rows of a data frame df by columns "trait" then "IL" (with leading
# zeroes added) then "pos":
#   ord = order.special(df$trait, ILs=df$IL, df$pos, lead0="ILs")
#   df = df[ord,]
# Use the "first" and "last" arguments to list particular values which should sort to
# the start or end.  These arguments each are either a list or vector.  A vector is
# applied to the first ... argument and gives the values which are to be first or last
# in the sorted order.  A list value for "first" or "last" has as its elements vectors
# whose names are ... argument names naming the arguments to which the vectors apply,
# and the vectors contain the values within those arguments to be first or last in
# sorted order.
# E.g. to sort the rows of a data frame df by IL (with leading zeroes added for sorting
# purposes and with "M82" first and "PENN" last) and then by chr then start:
#   ord = order.special(rownames(df), df$chr, df$start, first="M82", last="PENN")
#   df = df[ord,]
# or to sort by columns "trait" then "IL" (with leading zeroes added and both M82 and
# PENN appearing first) then "pos":
#   ord = order.special(df$trait, ILs=df$IL, df$pos, lead0="ILs", first=list(ILs=c("M82", "PENN")))
#   df = df[ord,]
#######################################################################################
order.special = function(..., lead0=NULL, first=NULL, last=NULL)
    {
    args = list(...)
    if (length(args) == 0) stop("order.special error: must give at least one ... arg")
    for (arg in args[-1])
    if (length(arg) != length(args[[1]])) stop("order.special error: all ... args must be same length")
    if (length(args[[1]]) == 0)
        return(integer(0))

    # Provide default for lead0.
    if (is.null(lead0))
        lead0 = 1
    # Make first and last be either NULL or lists.
    if (!is.null(first) && !is.list(first))
        {
        first = list(first)
        names(first) = names(args)[1]
        if (is.null(names(first)))
            names(first) = 1
        }
    if (!is.null(last) && !is.list(last))
        {
        last = list(last)
        names(last) = names(args)[1]
        if (is.null(names(last)))
            names(last) = 1
        }
    # Make sure first and last lists have names.
    if (is.list(first) && is.null(names(first))) stop("order.special error: expected 'first' arg to have names")
    if (is.list(last) && is.null(names(last))) stop("order.special error: expected 'last' arg to have names")

    # To sort first or last, we will modify any vector requiring such sorting so
    # that its elements start with either "A", "B", or "C" depending on whether
    # they are to sort first (A), middle (B), or last (C).  Furthermore, while
    # middle-sorting elements consist of "C" followed by the element value,
    # elements sorting first or last consist of "A" or "B" followed by a 9-digit
    # number that is the index of the element value in the "first" or "last"
    # vector, filled with leading 0's.  This ensures that these elements will
    # sort in the same order as they occur in "first" or "last".
    isModified = rep(FALSE, length(args)) # Track which args vectors are modified, for use when doing "last".
    names(isModified) = names(args)
    for (argName in names(first))
        {
        if (grepl("^[0-9]+$", argName))
            idx = as.integer(argName)
        else
            idx = which(argName == names(args))
        if (length(idx) != 1) stop("order.special error: expected 'first' arg list element name to match an argument name")
        V = as.character(args[[idx]])
        matches = match(V, first[[argName]])
        V[!is.na(matches)] = sprintf("A%09d", matches[!is.na(matches)])
        V[is.na(matches)] = paste("B", V[is.na(matches)], sep="")
        args[[idx]] = V
        isModified[idx] = TRUE
        }
    # Now do "last".
    for (argName in names(last))
        {
        idx = which(argName == names(args))
        if (length(idx) != 1) stop("order.special error: expected 'last' arg list element name to match an argument name")
        V = as.character(args[[idx]])
        # If we already modified this vector when doing "first" above, then we
        # won't find the "last" elements, but rather, those with "B" prepended.
        # (If "A" is prepended, the value appears in both "first" and "last"
        # vectors, let the "first" vector have precedence.)
        V.last = last[[argName]]
        if (isModified[idx])
            V.last = paste("B", V.last, sep="")
        matches = match(V, V.last)
        V[!is.na(matches)] = sprintf("C%09d", matches[!is.na(matches)])
        # If this args vector was already modified, we don't need to prepend "B".
        if (!isModified[idx])
            V[is.na(matches)] = paste("B", V[is.na(matches)], sep="")
        args[[idx]] = V
        }

    # For each argument name in lead0 (or index 1 if lead0 wasn't specified), add
    # leading zeroes to number strings in the argument.
    for (idx in lead0)
        {
        if (grepl("^[0-9]+$", idx))
            idx = as.integer(idx)
        V = args[[idx]]
        # Find length of longest digit string so we know the number of digits including
        # leading 0's that each digit substring will be set to.  Construct a regular expression and
        # its replacement, for adding leading zeroes.
        digitStrs = strsplit(as.character(args[[1]]), "[^0-9]+")
        NdigitsMax = max(sapply(digitStrs, function(V) max(c(0,nchar(V)))))
        if (NdigitsMax > 1)
            {
            RE = paste("(^|[^0-9])([0-9]{1,", NdigitsMax-1, "})([^0-9]|$)", sep="")
            repRE = "\\10\\2\\3"
            # Keep applying the regular expression until no further changes occur.
            while (TRUE)
                {
                Vnew = gsub(RE, repRE, V)
                if (all(Vnew == V, na.rm=TRUE))
                    break
                V = Vnew
                }
            # Replace the argument by the modified strings with leading zeroes.
            args[[idx]] = V
            }
        }
    # Now call order with the arguments, and return its return value.
    return(do.call(order, args))
    }

#######################################################################################
# Compute explained variance from 1-way ANOVA results by decomposing MSH and MSE.
# In a 1-way ANOVA, the mean square value is not an estimate of the explained variance,
# but is a combination of it and the within-group variance: MSH = VarE + n*VarH, where
# n = number of replicates in the group.  MSE is an estimate of VarE.  Therefore, VarH
# can be estimated as (MSH-MSE)/n.  If the estimate is negative, it is set to 0.
# Arguments:
#   anovaResult: result of anova() on a single-factor model.
# Returns: list with these elements:
#   varH: estimated variance due to the single factor.
#   varE: estimated error variance (MSE from ANOVA).
#   varT: estimated total variance = varH + varE
#   frac: fraction of variance that is explained = varH/varT
#######################################################################################
getANOVA1varianceExplained = function(anovaResult)
    {
    if (nrow(anovaResult) != 2)
        stop("getANOVA1varianceExplained requires 1-way ANOVA results")
    n = (sum(anovaResult[,1])+1)/(anovaResult[1,1]+1)
    MSH = anovaResult[1,3]
    varE = anovaResult[2,3]
    varH = (MSH-varE)/n
    if (varH < 0)
        varH = 0
    varT = varH+varE
    frac = varH/varT
    return(list(varH=varH, varE=varE, varT=varT, frac=frac))
    }

#######################################################################################
# Uncentered correlation of two vectors.
#######################################################################################
cosv = function(v1, v2)
    {
    v1 = as.numeric(v1)
    v2 = as.numeric(v2)
    return(sum(v1*v2)/(sqrt(sum(v1*v1)*sum(v2*v2))))
    }

#######################################################################################
# Apply cosv to a whole matrix or data frame.
#######################################################################################
cosvDf = function(df)
    {
    m = t(df)
    mprod = m %*% t(m)
    mdiag = diag(mprod)
    return(mprod/sqrt(mdiag %*% t(mdiag)))
    }

#######################################################################################
# Apply cosv to two matrices or data frames of size NxM and MxK, to produce a resulting
# distance matrix of size NxK.
#######################################################################################
cosvDfPair = function(df1, df2)
    {
    m1 = t(df1)
    m2 = as.matrix(df2)
    mprod = m1 %*% m2
    m1dim = 1:nrow(m1)
    m2dim = 1:ncol(m2)
    m1magSqrd = sapply(m1dim, function(x) sum(m1[x,]^2))
    m2magSqrd = sapply(m2dim, function(x) sum(m2[,x]^2))
    return(mprod/sqrt(m1magSqrd %*% t(m2magSqrd)))
    }

#######################################################################################
# Uncentered correlation that automatically detects argument classes and calls the
# appropriate function above.
#######################################################################################
cosu = function(a1, a2=NULL)
    {
    if (is.vector(a1) && is.vector(a2))
        return(cosv(a1, a2))
    if (is.vector(a1) || is.matrix(a1) || is.data.frame(a1))
        {
        if (is.null(a2))
            return(cosvDf(a1))
        if (is.vector(a1) || is.matrix(a1) || is.data.frame(a1))
            return(cosvDfPair(a1, a2))
        }
    stop("Unable to do uncentered correlation on arg types", class(a1), class(a2))
    }

#######################################################################################
# Partial Pearson correlation.
# From Biometry, Sokal and Rohlf, 3rd ed., 1995, pg. 649, the partial correlation
# of x and y with z held constant is done as in this function.
#######################################################################################
partialCor = function(x, y, z, method=c("pearson", "spearman"),
    use=c("everything", "all.obs", "complete.obs", "na.or.complete", "pairwise.complete.obs"))
    {
    r_xy = cor(x, y, method=method[1], use=use[1])
    r_xz = cor(x, z, method=method[1], use=use[1])
    r_yz = cor(y, z, method=method[1], use=use[1])
    return((r_xy-r_xz*r_yz)/sqrt((1-r_xz^2)*(1-r_yz^2)))
    }

#######################################################################################
# Perform correlation between x and y, or between rows of x.  See function cor() in R
# help for description of arguments.  This function supports additional methods beyond
# what cor() supports:
#   uncentered: use cosu() above instead of cor().
#   abs.uncentered: abs(cosu(x, y))
#   abs.(other): abs(cor(x, y, method=(other)))
# Note that currently cosu() does not support the "use" argument.
#######################################################################################
corTed = function(x, y=NULL, method="pearson", use="everything")
    {
    doAbs = FALSE
    if (grepl("^abs.", method))
        {
        doAbs = TRUE
        method = sub("^abs.", "", method)
        }
    if (method == "uncentered")
        corVal = cosu(x, y)
    else
        corVal = cor(x, y, method=method, use=use)
    if (doAbs)
        corVal = abs(corVal)
    return(corVal)
    }

#######################################################################################
# Compute a distance matrix giving distance between each pair of rows of x.
#   x: a numeric matrix or data frame.  Currently, NA values are not allowed.
#   method: distance measure used, one of:
#       euclidean: Usual Euclidean distance between the two row vectors.
#       maximum: Maximum of difference between components of the two row vectors.
#       manhattan: "Taxicab" distance, sum of absolute difference between components.
#       canberra: sum(|x_i - y_i| / |x_i + y_i|)
#       minkowski: pth root of the sum of the pth powers of the differences of components.
#       pearson: 1 - pearson correlation of the two row vectors.
#       spearman: 1 - spearman correlation of the two row vectors.
#       kendall: 1 - kendall correlation of the two row vectors.
#       uncentered: 1 - uncentered correlation of the two row vectors.
#       abs.pearson: 1 - abs(pearson correlation)
#       abs.spearman: 1 - abs(spearman correlation)
#       abs.kendall: 1 - abs(kendall correlation)
#       abs.uncentered: 1 - abs(uncentered correlation)
#######################################################################################
distTed = function(x, method="euclidean", p=3)
    {
    distFuncMethods = c("euclidean", "maximum", "manhattan", "canberra", "minkowski")
    if (method %in% distFuncMethods)
        return(as.matrix(dist(x, method=method, p=p)))
    return(1 - corTed(t(x), method=method))
    }
distMethods = c( # kendall omitted because extremely slow
    "euclidean",    "maximum",      "manhattan",    "canberra",     "minkowski",
    "pearson",      "spearman",     "uncentered",   "abs.pearson",  "abs.spearman",
    "abs.uncentered")

#######################################################################################
# Divide the range of vector x into n equal-sized intervals, then group y-values using
# those intervals and apply a function to the y-values that lie in each interval.
# Arguments:
#   x: vector of x-values to be subdivided into intervals.  If NULL, names(y) is used
#       for x, and those names must be numeric.  Length must equal length(y).
#   y: vector of y-values to be organized using intervals of x.
#   n: number of intervals into which to subdivide range of x.
#   bin.width: width of a bin, alternative way of specifying n.  Specify either n or
#       bin.width, but not both.  If bin.width is specified, n is calculated as
#       ceiling((xmax-xmin)/bin.width), and so actual bin width will be slightly less.
#   FUN: function to apply to each group of y-values in each interval, e.g. 'mean' or
#       'sum'.
#   xmin: the minimum value to use for the x-interval range.  Default is min(x).
#   xmax: the maximum value to use for the x-interval range.  Default is max(x).
#   use.pretty: if TRUE, the 'pretty' function is used to calculate the intervals, and
#       the actual number of intervals may be more than n.
#   no.y.value: value to return in the y vector for intervals that contained no values.
# Returns: a list with elements x and y where:
#       x.intervals: a vector containing the endpoints of the intervals.  If use.pretty
#           =FALSE, the length of this vector is n+1.
#       x.midpoints: a vector containing the midpoints of the intervals.  If use.pretty
#           =FALSE, the length of this vector is n.
#       y: a vector of values returned by FUN applied to the groups of y-values.  If
#           use.pretty=FALSE, the length of this vector is n.
#######################################################################################
binXY = function(x=NULL, y, n=NULL, bin.width=NULL, FUN=sum, xmin=NULL, xmax=NULL,
    use.pretty=FALSE, no.y.value=NA)
    {
    if (!is.numeric(y)) stop("binXY: y is not numeric")
    if (is.null(x)) x = suppressWarnings(as.numeric(names(y)))
    if (!is.numeric(x)) stop("binXY: x is not numeric")
    if (length(x) != length(y)) stop("binXY: x and y do not have same length")
    FUN = match.fun(FUN)
    if (is.null(xmin)) xmin = min(x, na.rm=TRUE)
    if (is.null(xmax)) xmax = max(x, na.rm=TRUE)
    xspan = xmax-xmin
    if (xspan <= 0) stop("binXY: xmin is not < xmax")
    if (is.null(n))
        {
        if (is.null(bin.width)) stop("binXY: either n or bin.width must be specified")
        if (!is.numeric(bin.width)) stop("binXY: bin.width is not numeric")
        n = ceiling(xspan/bin.width)
        }
    if (!is.numeric(n)) stop("binXY: n is not numeric")
    if (length(n) != 1) stop("binXY: n is not length 1")
    if (!is.numeric(n) || is.na(n) || (n != as.integer(n))) stop("binXY: n is not an integer")
    if (n < 2) stop("binXY: n is not >= 2")
    if (use.pretty)
        x.intervals = pretty(c(xmin, xmax), n, min.n=n)
    else
        x.intervals = c(xmin, xmin + (1:(n-1))*(xspan/n), xmax)
    which.interval = findInterval(x, x.intervals) # which.interval values are in range 0..n+1
    which.interval[which.interval < 1 | which.interval > n] = NA
    y.result = tapply(y, which.interval, FUN)
    x.result = as.numeric(names(y.result))
    y.full.result = rep(no.y.value, length(x.intervals)-1)
    y.full.result[x.result] = y.result
    x.midpoints = x.intervals[-1] - (x.intervals[2]-x.intervals[1])/2
    return(list(x.intervals=x.intervals, x.midpoints=x.midpoints, y=y.full.result))
    }

#######################################################################################
# Check a numeric vector for bad values and print an error message showing all bad ones.
# Return a new vector with all values (including the bad ones) converted to numeric type
# if not already there.  Acceptable values are unsigned integers and if allowSigned is
# TRUE, signed integers, and, if allowFloats is TRUE, floating point values containing
# signs and a decimal point, and, if allowNA is TRUE, NA (and various string forms of
# NA, and, if allowNULL is TRUE, NULL.  Currently, floating point values with e-style
# exponents are NOT SUPPORTED.  If the min or max arguments are provided, the values
# must be >= min and <= max else they are considered bad.  Values whose coefficient
# of variation is > maxCV are considered bad, unless maxCV is NULL.  Lower case
# letters within the value are converted to upper case (no error message) if
# allowLowerCase is FALSE.  Spaces within the value are removed with an error message
# if allowSpaces is FALSE.  The "name" argument can be used to provide a name for use
# in the error message.  Error messages are displayed using printLog().
#######################################################################################
checkBadNumbers = function(vals, allowSigned=TRUE, allowFloats=TRUE, allowNA=TRUE,
    allowNULL=TRUE, allowLowerCase=F, allowSpaces=F, min=NULL, max=NULL, maxCV=1,
    name="(unnamed)")
    {
    if (is.null(vals))
        {
        if (!allowNULL)
            printLog("NULL detected in data named", name, "\n")
        return(NULL)
        }
    vals = as.character(vals)
    if (!allowLowerCase)
        vals = toupper(vals)
    vals[is.na(vals)] = "NA"
    if (!allowSpaces)
        vals = gsub(" ", "", vals)
    regExpStr = "^NA$|^N/A$|^[0-9]+$"
    if (allowSigned)
        regExpStr = paste(regExpStr, "^[+\\-][0-9]+$", sep="|")
    if (allowFloats) # Must allow for no digits before decimal or no digits after it, but not both.
        regExpStr = paste(regExpStr, "^[+\\-]?[0-9]+\\.?[0-9]*$", "^[+\\-]?[0-9]*\\.?[0-9]+$", sep="|")
    whichOnesBad = which(!grepl(regExpStr, vals))
    if (length(whichOnesBad) > 0)
        printLog("Bad data detected in data named", name, " Bad values:", vals[whichOnesBad], "at these indexes:", (1+whichOnesBad), "\n")
    vals[vals == "NA" | vals == "N/A"] = NA
    if (!allowNA && any(is.na(vals)))
        printLog("NA value(s) detected in data named", name, "\n")
    vals = as.numeric(vals)
    if (!is.null(min))
        {
        whichOnesTooSmall = which(!is.na(vals) & vals < min)
        if (length(whichOnesTooSmall) > 0)
            printLog("Data values <", min, "in data named", name, " Bad values:",
                vals[whichOnesTooSmall], "at these indexes:", (1+whichOnesTooSmall), "\n")
        }
    if (!is.null(max))
        {
        whichOnesTooBig = which(!is.na(vals) & vals > max)
        if (length(whichOnesTooBig) > 0)
            printLog("Data values >", max, "in data named", name, " Bad values:",
                vals[whichOnesTooBig], "at these indexes:", (1+whichOnesTooBig), "\n")
        }
    if (!is.null(maxCV))
        {
        m = mean(vals, na.rm=TRUE)
        if (!is.na(m) && m != 0)
            {
            cv = abs(sd(vals, na.rm=TRUE)/m)
            if (!is.na(cv) && cv > maxCV)
                printLog("Bad data detected in data named", name, "Coefficient of variation is really big:", cv, "\n")
            }
        }
    return(vals)
    }

#######################################################################################
# Check a character vector for bad values and print an error message showing all bad
# ones.  Return a new vector with all values (including the bad ones) converted to
# character type if not already there.  Acceptable values are given by the argument
# vector "acceptable", and, if allowNA is TRUE, NA (and various string forms of
# NA, and, if allowNULL is TRUE, NULL.  Lower case letters within the value are
# converted to upper case (no error message) if allowLowerCase is FALSE.  Spaces
# within the value are removed with an error message if allowSpaces is FALSE.
# The "name" argument can be used to provide a name for use in the error message.
# Error messages are displayed using printLog().
#######################################################################################
checkBadValues = function(vals, acceptable, allowNA=TRUE, allowNULL=TRUE,
    allowLowerCase=F, allowSpaces=F, name="(unnamed)")
    {
    if (is.null(vals))
        {
        if (!allowNULL)
            printLog("NULL detected in data named", name, "\n")
        return(NULL)
        }
    vals = as.character(vals)
    if (!allowLowerCase)
        vals = toupper(vals)
    vals[is.na(vals)] = "NA"
    if (!allowSpaces)
        vals = gsub(" ", "", vals)
    regExpStr = "^NA$|^N/A$|^[0-9]+$"
    whichOnesBad = which(!vals %in% acceptable)
    if (length(whichOnesBad) > 0)
        printLog("Bad data detected in data named", name, " Bad values:", vals[whichOnesBad], "at these indexes:", (1+whichOnesBad), "\n")
    vals[vals == "NA" | vals == "N/A"] = NA
    if (!allowNA && any(is.na(vals)))
        printLog("NA value(s) detected in data named", name, "\n")
    return(vals)
    }

#######################################################################################
# Create a contrasts matrix for all pairwise contrasts of a factor.  The number of
# levels of the factor is n, and the returned matrix is of size n x choose(n,2).
# The first level of the factor may become the "(Intercept)" value in ANOVA output,
# and the desired contrast matrix may require that testing pairs involving that first
# level simply test the ANOVA coefficient for the second member of the pair.  The
# "useIntercept" argument is for this purpose; FALSE will give a matrix that explicitly
# includes each pair, while TRUE will make the first n-1 pairs test only the second
# member of the pair.
#######################################################################################
allPairwiseContrasts = function(n, useIntercept=TRUE)
    {
    pairs = combn(n, 2)
    m = matrix(0, nrow=n, ncol=ncol(pairs))
    for (i in 1:ncol(m))
        {
        if (useIntercept && (i < n))
            m[pairs[2,i], i] = 1
        else
            {
            m[pairs[1,i], i] = 1
            m[pairs[2,i], i] = -1
            }
        }
    return(m)
    }

#######################################################################################
# Suppose you have a vector "sa" of strings, and a separate vector "fa" of the same
# length is a factor that gives the category that each of string in sa.  You wish to
# generate a new vector of the same length as the NUMBER OF UNIQUE CATEGORIES (number
# of levels of fa), with each element of the vector being a string of the pasted
# strings in sa within that category, separated by a comma.  For example, suppose fa
# and sa are:
#   sa      fa
#   -----   ------
#   Jane    Female
#   Sally   Female
#   Pam     Female
#   Sue     Female
#   Tom     Male
#   John    Male
#   Mike    Male
#
# And you wish to generate this vector v:
#
#   names   vector v
#   ------  -------------------------------
#   Female  "Jane,Sally,Pam,Sue"
#   Male    "Tom,John,Mike"
#
# This can be done with the following expression:
# sa = c("Jane","Sally","Pam","Sue","Tom","John","Mike")
# fa = c("Female","Female","Female","Female","Male","Male","Male")
# v = tapply(sa, fa, function(S) paste(S, collapse=","))
# names(v) = unique(fa)
#
# This function does EXACTLY THE SAME THING.  However, it does it in a way that is
# MUCH FASTER than the above if the number of levels of fa is quite large.
#
# Arguments:
#   fa: vector of strings that categorize each element of sa.
#   sa: vector of strings to be pasted together, same length as fa.
#   sep: the string that is to separate the pasted-together strings.
#   uniqueSep: a unique string not appearing in sa that can be temporarily used to
#       separate concatenated sa elements for later splitting apart.
#   sorted: TRUE if fa (and sa) is sorted so all identical members are adjacent,
#       FALSE if not (this function will sort them).
#   ret.fai: TRUE if fai member is to be returned, see below.
# Returns: list with these members:
#   v: array of same length as nlevels(factor(fa)) containing pasted-together strings
#       of sa found within that level of fa, separated by the specified sep string.
#   fai: NA if ret.fai is FALSE, else number vector of same length as v, containing
#       the index into fa of the first occurrence of each unique level.  Thus, fa[fai]
#       is a vector of those unique level names.
#######################################################################################
tapply_paste = function(fa, sa, sep=",", uniqueSep="*", sorted=TRUE, fai=NULL, ret.fai=FALSE)
    {
    if (!sorted)
        {
        ord = order(fa)
        fa = fa[ord]
        sa = sa[ord]
        }
    if (is.null(fai))
        fai = match(unique(fa), fa)
    sa[fai] = paste0(uniqueSep, sa[fai])
    sa = paste0(sa, collapse=sep)
    sa = unlist(strsplit(sa, uniqueSep, fixed=TRUE))
    sa = sa[-1]
    names(sa) = fa[fai]
    L = list(v=sa, fai=NA)
    if (ret.fai)
        L$fai = fai
    return(L)
    }

#######################################################################################
# Suppose you have a vector "sa" of strings, and each string consists of one or more
# words separated by commas.  A separate vector "fa" of the same length is a factor
# that gives the category that each of string in sa.  You wish to generate a new vector
# of the same length as the NUMBER OF UNIQUE CATEGORIES (number of levels of fa),
# with each element of the vector being a string of the unique words found in sa within
# that category, separated by commas.  For example, suppose fa and sa are:
#   name    fa      sa
#   -----   ------  ---------------------
#   Jane    Female  "cat,dog"
#   Sally   Female  "cat,mouse,dog"
#   Pam     Female  "dog"
#   Sue     Female  "cat,fish"
#   Tom     Male    "cat,dog,bird,snake"
#   John    Male    "dog,rat"
#   Mike    Male    "cat,monkey"
#
# And you wish to generate this vector v:
#
#   names   vector v
#   ------  -------------------------------
#   Female  "cat,dog,mouse,fish"
#   Male    "cat,dog,bird,snake,rat,monkey"
#
# This can be done with the following expression:
# sa = c("cat,dog","cat,mouse,dog","dog","cat,fish","cat,dog,bird,snake","dog,rat","cat,monkey")
# fa = c("Female","Female","Female","Female","Male","Male","Male")
# v = as.vector(tapply(sa, fa, function(S) paste(unique(unlist(strsplit(S,","))), collapse=",")))
# names(v) = unique(fa)
#
# This function does EXACTLY THE SAME THING.  However, it does it in a way that is
# MUCH FASTER than the above if the number of unique words in the entire sa array is
# small compared to the number of levels of fa.  This function has a loop that loops
# for the number of unique words in sa; the "tapply()" method basically loops for each
# factor level.
#
# Arguments:
#   fa: vector of strings that categorize each element of sa.
#   sa: vector of strings that are splittable on some substring, same length as fa.
#   split: the substring that can split each string of sa into separate substrings.
#   sep: the string that is to separate each word in the result.
#   uniqueSep: a unique string not appearing in sa that can be temporarily used to
#       separate concatenated sa elements for later splitting apart.
#   sorted: TRUE if fa (and sa) is sorted so all identical members are adjacent,
#       FALSE if not (this function will sort them).
#   fai: either the vector return by "match(unique(fa), fa)" or NULL.
#   ret.fai: TRUE if fai member is to be returned, see below.
# Returns: list with these members:
#   v: array of same length as nlevels(factor(fa)) containing unique substrings of
#       sa found within that level of fa, separated by the specified split string.
#   fai: NA if ret.fai is FALSE, else number vector of same length as v, containing
#       the index into fa of the first occurrence of each unique level, as computed
#       by "match(unique(fa), fa)".  Thus, fa[fai] is a vector of those unique level
#       names.
#######################################################################################
tapply_unique_strsplit = function(fa, sa, split=",", sep=split, uniqueSep="*",
    sorted=TRUE, fai=NULL, ret.fai=FALSE)
    {
    if (!sorted)
        {
        ord = order(fa)
        fa = fa[ord]
        sa = sa[ord]
        }
    uniqueStrings = unique(unlist(strsplit(sa, split, fixed=TRUE)))
    if (is.null(fai))
        fai = match(unique(fa), fa)
    sa[fai] = paste0(uniqueSep, split, sa[fai])
    sa = paste0(sa, collapse=split)
    sa = paste0(sa, split)
    sa = unlist(strsplit(sa, uniqueSep, fixed=TRUE))
    sa = sa[-1]
    v = character(length(sa))
    for (s in uniqueStrings)
        {
        s.bounded = paste0(split, s, split)
        has.s = grepl(s.bounded, sa, fixed=TRUE)
        v[has.s] = paste(v[has.s], s, sep=sep)
        }
    v = substring(v, 1+nchar(sep))
    names(v) = fa[fai]
    L = list(v=v, fai=NA)
    if (ret.fai)
        L$fai = fai
    return(L)
    }

#######################################################################################
# Log-likelihood tests of independence & goodness of fit
# Does Williams' and Yates' correction.
# Does Monte Carlo simulation of p-values, via gtestsim.c.
#
# G & q calculation from Sokal & Rohlf (1995) Biometry 3rd ed.
# TOI Yates' correction taken from Mike Camann's 2x2 G-test fn.
# GOF Yates' correction as described in Zar (2000)
# More stuff taken from ctest's chisq.test()
#
# V3.3 Pete Hurd Sept 29 2001. phurd@ualberta.ca
#######################################################################################
g.test = function(x, y=NULL, correct=c("williams", "yates", "none"),
    p=rep(1/length(x), length(x)), simulate.p.value=FALSE, B=2000)
    {
    DNAME = deparse(substitute(x))
    if (is.data.frame(x)) x = as.matrix(x)
    if (is.matrix(x))
        {
        if (min(dim(x)) == 1)
            x = as.vector(x)
        }
    if (!is.matrix(x) && !is.null(y))
        {
        if (length(x) != length(y))
            stop("x and y must have the same length")
        DNAME = paste(DNAME, "and", deparse(substitute(y)))
        OK = complete.cases(x, y)
        x = as.factor(x[OK])
        y = as.factor(y[OK])
        if ((nlevels(x) < 2) || (nlevels(y) < 2))
            stop("x and y must have at least 2 levels")
        x = table(x, y)
        }
    if (any(x < 0) || any(is.na(x)))
        stop("all entries of x must be nonnegative and finite")
    if ((n = sum(x)) == 0)
        stop("at least one entry of x must be positive")
    # If x is matrix, do test of independence
    if (is.matrix(x))
        {
        # Test of Independence
        nrows = nrow(x)
        ncols = ncol(x)
        if (correct[1]=="yates")
            { # Do Yates' correction?
            if(dim(x)[1]!=2 || dim(x)[2]!=2) # check for 2x2 matrix
                stop("Yates' correction requires a 2 x 2 matrix")
            if((x[1,1]*x[2,2])-(x[1,2]*x[2,1]) > 0)
                {
                x[1,1] = x[1,1] - 0.5
                x[2,2] = x[2,2] - 0.5
                x[1,2] = x[1,2] + 0.5
                x[2,1] = x[2,1] + 0.5
                }
            else
                {
                x[1,1] = x[1,1] + 0.5
                x[2,2] = x[2,2] + 0.5
                x[1,2] = x[1,2] - 0.5
                x[2,1] = x[2,1] - 0.5
                }
            }

        sr = apply(x, 1, sum)
        sc = apply(x, 2, sum)
        E = outer(sr, sc, "*")/n
        # are we doing a monte-carlo?
        # no monte carlo GOF?
        if (simulate.p.value)
            {
            METHOD = paste("Log likelihood ratio (G-test) test of independence\n\t with simulated p-value based on", B, "replicates")
            tmp = .C("gtestsim", as.integer(nrows), as.integer(ncols),
                                as.integer(sr), as.integer(sc), as.integer(n), as.integer(B),
                                as.double(E), integer(nrows * ncols), double(n+1),
                                integer(ncols), results=double(B), PACKAGE= "ctest")
            g = 0
            for (i in 1:nrows)
                {
                for (j in 1:ncols)
                    {
                    if (x[i,j] != 0) g = g + x[i,j] * log(x[i,j]/E[i,j])
                    }
                }
            STATISTIC = G = 2 * g
            PARAMETER = NA
            PVAL = sum(tmp$results >= STATISTIC)/B
            }
        else
            {
            # no monte-carlo
            # calculate G
            g = 0
            for (i in 1:nrows)
                {
                for (j in 1:ncols)
                    {
                    if (x[i,j] != 0) g = g + x[i,j] * log(x[i,j]/E[i,j])
                    }
                }
            q = 1
            if (correct[1]=="williams")
                { # Do Williams' correction
                row.tot = col.tot = 0
                for (i in 1:nrows)
                    { row.tot = row.tot + 1/(sum(x[i,]))     }
                for (j in 1:ncols)
                    { col.tot = col.tot + 1/(sum(x[,j]))     }
                q = 1+ ((n*row.tot-1)*(n*col.tot-1))/(6*n*(ncols-1)*(nrows-1))
                }
            STATISTIC = G = 2 * g / q
            PARAMETER = (nrow(x)-1)*(ncol(x)-1)
            PVAL = 1-pchisq(STATISTIC, df=PARAMETER)
            if(correct[1]=="none")
                METHOD = "Log likelihood ratio (G-test) test of independence without correction"
            if(correct[1]=="williams")
                METHOD = "Log likelihood ratio (G-test) test of independence with Williams' correction"
            if(correct[1]=="yates")
                METHOD = "Log likelihood ratio (G-test) test of independence with Yates' correction"
            }
        }
    else
        {
        # x is not a matrix, so we do Goodness of Fit
        METHOD = "Log likelihood ratio (G-test) goodness of fit test"
        if (length(x) == 1)
            stop("x must at least have 2 elements")
        if (length(x) != length(p))
            stop("x and p must have the same number of elements")
        E = n * p

        if (correct[1]=="yates")
            { # Do Yates' correction
            if(length(x)!=2)
                stop("Yates' correction requires 2 data values")
            if ( (x[1]-E[1]) > 0.25)
                {
                x[1] = x[1]-0.5
                x[2] = x[2]+0.5
                }
            else if ( (E[1]-x[1]) > 0.25)
                {
                x[1] = x[1]+0.5
                x[2] = x[2]-0.5
                }
            }
        names(E) = names(x)
        g = 0
        for (i in 1:length(x))
            {
            if (x[i] != 0) g = g + x[i] * log(x[i]/E[i])
            }
        q = 1
        if (correct[1]=="williams")
            { # Do Williams' correction
            q = 1+(length(x)+1)/(6*n)
            }
        STATISTIC = G = 2*g/q
        PARAMETER = length(x) - 1
        PVAL = pchisq(STATISTIC, PARAMETER, lower = FALSE)
        }
    names(STATISTIC) = "Log likelihood ratio statistic (G)"
    names(PARAMETER) = "X-squared df"
    names(PVAL) = "p.value"
    structure(list(statistic=STATISTIC, parameter=PARAMETER, p.value=PVAL,
                        method=METHOD, data.name=DNAME, observed=x, expected=E),
                        class="htest")
    }

#######################################################################################
# Say you have a data frame df and you've identified adjacent rows that share
# some property, which is indicated by a separate vector V of length nrow(df),
# containing TRUE if the corresponding row shares the same property as the
# PREVIOUS row.  This function creates a new vector of the same length as V,
# containing the number of following rows that share the same property as this
# row, and which is ZERO if this is a row that shares the same property as the
# preceding row, and which is ONE if this row does not share the same property
# as either the preceding or following row.
# For example, if V is:     F F T T T F F F T F F T
# then the new vector is:   1 4 0 0 0 1 1 2 0 1 2 0
#######################################################################################
countAdjacentRowGroups = function(V)
    {
    # V = c(F,F,T,T,T,F,F,F,T,F,F,T)
    start = c(V[-1] & !V[-length(V)], FALSE)
    end = V & c(!V[-1], TRUE)
    cum = cumsum(V)[end]
    num = c(cum[1], diff(cum)) + 1
    V2 = as.integer(!V)
    V2[start] = num
    return(V2)
    }

#######################################################################################
# Include various other general-purpose R files.  These files have been split out of
# this one to help keep file sizes from getting too large.
#######################################################################################

# The files to be included are located in the same directory as THIS file.  What
# directory IS this file in?  The following code determines that, I hope reliably.
frameFiles = lapply(sys.frames(), function(x) x$ofile)
frameFiles = Filter(Negate(is.null), frameFiles)
thisFile = frameFiles[[length(frameFiles)]]
thisDir = dirname(thisFile)
PATHSEP = sub("^.*([/\\]).*$", "\\1", thisFile)

# Define variables for pathnames of files in this directory.
Include_TextFunctions = paste(thisDir, "Include_TextFunctions.R", sep=PATHSEP)
Include_Heatmaps = paste(thisDir, "Include_Heatmaps.R", sep=PATHSEP)
Include_VennDiagrams = paste(thisDir, "Include_VennDiagrams.R", sep=PATHSEP)
Include_PlotFunctions = paste(thisDir, "Include_PlotFunctions.R", sep=PATHSEP)
#Include_SetOperations = paste(thisDir, "Include_SetOperations.R", sep=PATHSEP)

# Include these other files, unless Include_RootRsourceFile.R is being used.
if (!exists("Sourced.Include_RootRsourceFile"))
    {
    source(Include_TextFunctions)
    source(Include_Heatmaps)
    source(Include_VennDiagrams)
    source(Include_PlotFunctions)
    #source(Include_SetOperations)
    }

#######################################################################################
# End of file.
#######################################################################################
Sourced.Include_UtilityFunctions = TRUE
cat("  Include_UtilityFunctions.R included\n")
